/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalControl;

import java.io.FileNotFoundException;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import phalControl.commandManager.Remote.RemoteManager;
import phalControl.WaveformControl.Status;
import phalControl.commandManager.ObjectInfo;
import phalControl.commandManager.ProcInfo;
import phalControl.commandManager.StatInfo;
import phalControl.commandManager.WaveformInfo;
import phalControl.commandManager.commandManager;
import phalControl.commandManager.commandManager.ErrorConnectingException;
import phalControl.commandManager.commandManager.ErrorInCommandException;

/**
 * @author root
 */
public class PlatformControl {

    private Type type;
    private String name;
    private ProcInfo[] processors;

    public enum Type {

        REMOTE, LOCAL;
    }
    commandManager manager;

    public PlatformControl(String name, String addr, Type type) throws FileNotFoundException, ErrorConnectingException {
        this.type = type;
        this.name = name;
        switch (type) {
            case REMOTE:
                manager = new RemoteManager(name, addr);
                break;
            case LOCAL:
            default:
                throw new UnsupportedOperationException("Not yet implemented");
        }
    }

    @Override
    public String toString() {
        return name;
    }

    public void Connect() throws ErrorConnectingException {
        manager.Connect();
        try {
            refresh();
        } catch (ErrorInCommandException ex) {
            Logger.getLogger(PlatformControl.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void refresh() throws ErrorInCommandException {
        processors = manager.listProcessors();
        for (int wi = 0; wi < processors.length; wi++) {
            processors[wi].nofObjects=0;
            processors[wi].cpu_usec=0;
            processors[wi].mean_tslot=0;
            for (int wj = 0; wj < processors[wi].xitfs.length; wj++) {
                for (int ri = 0; ri < processors.length; ri++) {
                    if (processors[ri].id == processors[wi].xitfs[wj].remote_pe) {
                        processors[ri].addInputItf(processors[wi], processors[wi].xitfs[wj]);
                    }
                }
            }

        }
        /*if (processors.length==2) {
            processors[0].mean_tslot = 10000;
            processors[1].mean_tslot = 10000;
            processors[0].nofObjects = 9;
            processors[1].nofObjects = 5;
            processors[0].cpu_usec = 1503;
            processors[1].cpu_usec = 6475;
        }
         */
        WaveformInfo[] winfo;
        winfo = manager.getWaveformsInfo();
        for (int i = 0; i < winfo.length; i++) {
            try {
                if (!isWaveformLoaded(winfo[i].toString())) {
                    addWaveform(winfo[i].toString(), winfo[i].getStatus());
                } else {
                    if (!getWaveform(winfo[i].toString()).getStatus().equals(winfo[i].getStatus())) {
                        getWaveform(winfo[i].toString()).forceStatus(winfo[i].getStatus());
                    }

                }
                getWaveform(winfo[i].toString()).refresh();
                ObjectInfo[] execs = getWaveform(winfo[i].toString()).getExecInfo();
                for (int wi = 0; wi < execs.length; wi++) {
                    for (int k = 0; k < processors.length; k++) {
                        if (execs[wi].pe_id.equals(processors[k].id)) {
                            processors[k].nofObjects++;
                            processors[k].cpu_usec+=execs[wi].getCPUusec();
                            processors[k].mean_tslot += execs[wi].period;
                            processors[k].C-=execs[wi].mean_mac;

                            processors[k].nofCoreObjects[execs[wi].core_id]++;
                            processors[k].Corecpu_usec[execs[wi].core_id]+=execs[wi].getCPUusec();
                            processors[k].CoreC[execs[wi].core_id]-=execs[wi].mean_mac;
                        }
                    }
                    for (int wj=0;wj<execs[wi].itfs.length;wj++) {
                        for (int ri = 0; ri < execs.length; ri++) {
                            for (int rj=0;rj<execs[ri].itfs.length;rj++) {
                                if (execs[wi].name.equals(execs[ri].itfs[rj].remoteObjectName)
                                        && execs[wi].itfs[wj].name.equals(execs[ri].itfs[rj].remoteName)) {
                                    execs[wi].itfs[wj].remoteExec = execs[ri];
                                    execs[wi].itfs[wj].remoteItf = execs[ri].itfs[rj];
                                    execs[ri].itfs[rj].remoteExec = execs[wi];
                                    execs[ri].itfs[rj].remoteItf = execs[wi].itfs[wj];
//                                    execs[ri].itfs[rj].fifo_usage = execs[wi].itfs[wj].fifo_usage;
                                }
                            }
                        }
                    }
                }
            } catch (WaveformNotFoundException ex) {
            }


           for (int k=0;k<processors.length;k++) {
               if (processors[k].nofObjects!=0) {
                    processors[k].mean_tslot /= processors[k].nofObjects;
               }
            }
        }
    }

    public boolean isConnected() {
        return manager.isConnected();
    }

    public void Disconnect() {
        manager.Disconnect();
        waveforms.clear();
    }

    public ProcInfo[] getProcessors() throws ErrorInCommandException {
        return processors;
    }

    class wavesearch {

        private String name;

        wavesearch(String name) {
            this.name = name;
        }

        public boolean equals(Object x) {
            if (x instanceof WaveformControl) {
                return ((WaveformControl) x).toString().equals(name);
            } else {
                return false;
            }
        }
    }
    Vector waveforms = new Vector();

    public WaveformControl getWaveform(String name) throws WaveformNotFoundException {
        int i = waveforms.indexOf(new wavesearch(name));
        if (i == -1) {
            throw new WaveformNotFoundException("Waveform " + name + " not loaded.");
        }
        return (WaveformControl) waveforms.get(i);
    }

    public boolean isWaveformLoaded(String name) {
        return waveforms.contains(new wavesearch(name));
    }

    public WaveformControl loadWaveform(String name) throws ErrorInCommandException {
        manager.loadWaveform(name);
        return addWaveform(name, Status.LOADED);
    }

    public void stopWaveform(String name) throws ErrorInCommandException {
        try {
            manager.setWaveformStatus(name, Status.STOP);
            removeWaveform(name);
        } catch (ErrorInCommandException ex) {
            removeWaveform(name);
            throw ex;
        }
    }

    private WaveformControl addWaveform(String name, Status status) throws ErrorInCommandException {
        WaveformControl w = new WaveformControl(this, manager, name, status);
        waveforms.add(w);
        return w;
    }

    public void removeWaveform(String name) {
        int i = waveforms.indexOf(new wavesearch(name));
        if (i != -1) {
            waveforms.remove(i);
        }
    }

    public class WaveformNotFoundException extends Exception {

        private String desc;

        public WaveformNotFoundException(String desc) {
            this.desc = desc;
        }

        @Override
        public String getMessage() {
            return desc;
        }
    }
}
