/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalControl;
import phalgui.Node.Stats.FFT;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import phalControl.commandManager.StatInfo;
import phalControl.commandManager.StatInfo.Type;
import phalControl.commandManager.commandManager;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Node.Stats.XYValue;
import phalgui.Utils.Arrows.ValueStamp;
import phalgui.Utils.iFileReader;
import phalgui.Utils.iFileReader.LastLinesListener;


/**
 * @todo stat types
 * @author Usuario
 */
public class StatControl implements LastLinesListener {
    private commandManager control;
    private WaveformControl waveform;
    private StatInfo stat;
    private iFileReader reader;
    private ValueUpdateListener listener;
    private String ReposPath;
    private File ReposFile;



    public int getTriggerPeriod() {
        return period;
    }

    public Type getType() {
        return stat.type;
    }

    public String getWaveformName() {
        return stat.waveform;
    }

    public int getWindow() {
        return window;
    }

    public int getSize() {
        return stat.size;
    }

    public int getSlotDurationMs() {
        return objectTslot;
    }

    public double getTriggerValue() {
        return triggerValue;
    }

    public boolean isTriggerPeriodic() {
        return triggerType.equals(Trigger.PERIODIC);
    }
    public void setValue(String value) throws ErrorInCommandException {
        control.setStatValue(stat.waveform, stat.object, stat.name, value);
    }

    
    private enum Trigger { PERIODIC, VALUE };
    private Trigger triggerType = Trigger.PERIODIC;
    private int period,window;
    private double triggerValue;
    private int objectTslot;
    private int objectTsUs;
    private boolean isreporting;

    public StatControl(commandManager control, WaveformControl waveform, StatInfo stat) throws ErrorInCommandException {
        this.control=control;
        this.waveform=waveform;
        this.stat=stat;
        this.objectTsUs = waveform.getExecInfo(stat.object).period;
        this.objectTslot=objectTsUs/1000;
        this.period=100;
        if (stat.size>1) {
            this.window=1;
        } else {
            this.window=100;
        }
    }
    public StatControl(String name) {
        this.control=null;
        this.waveform=null;
        this.stat=new StatInfo();
        this.stat.name=name;
        this.stat.size=5;
        this.stat.object="obj";
        this.stat.waveform="wave";
        this.stat.type = StatInfo.Type.NUMERIC;
        this.objectTslot=100;
    }

    public void setPeriodicTrigger(int period, int window) throws ErrorInCommandException {
        triggerType=Trigger.PERIODIC;
        this.period=period;
        this.window=window;
        if (isreporting)
            control.startstopStatReport(stat.waveform, stat.object, stat.name, true, window, period);
    }

    public void setValueTrigger(double value, int window) throws ErrorInCommandException {
        triggerType=Trigger.VALUE;
        this.triggerValue=value;
        this.window=window;
        if (isreporting)
            control.startstopStatReport(stat.waveform, stat.object, stat.name, true, window, period);
    }

    public boolean belongs(String objname) {
            return objname.equals(stat.object);
    }
    public String getObjectName() {
            return stat.object;
    }
    @Override
    public String toString() {
            return stat.name;
    }

    public interface ValueUpdateListener {
        public void ValueUpdate(StatControl stat, XYValue value);
    }

    public void setValueUpdateListener(ValueUpdateListener listener) {
        this.listener=listener;
    }

    public void LastLines(String[] lines) {
        try {
            this.listener.ValueUpdate(this, getReportValues(lines));
        } catch (ErrorInCommandException ex) {
            Logger.getLogger(StatControl.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(StatControl.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvalidReportFormat ex) {
            Logger.getLogger(StatControl.class.getName()).log(Level.SEVERE, null, ex);
        }
    }


    /** @todo llegir nomes les 'window' primeres mostres
     */
    public synchronized XYValue getReportValues(String s[]) throws ErrorInCommandException, IOException, InvalidReportFormat {
        if (!isreporting)
            return null;
        double[] x = new double[s.length*stat.size];
        double[] y = new double[s.length*stat.size];
        double[] y2 = new double[s.length*stat.size];

        for (int i=0;i<s.length;i++) {
            String[] time = s[i].split(":");
            if (time.length != 2) {
                throw new InvalidReportFormat(reader.getFile().getName(),s[i]);
            }
            String[] sval2 = time[1].split(",");
            stat.size = sval2.length;
            int tstamp =Integer.valueOf(time[0]).intValue();
            for (int j=0;j<stat.size;j++) {
                y[i*stat.size+j] = Double.valueOf(sval2[j]).doubleValue();
                x[i*stat.size+j] = (tstampTodouble(tstamp, j)*objectTsUs)/1000000;
                //System.out.println("Added value: X=" + xt + ",Y=" + yt);
            }
        }

        s=null;
        return new XYValue(x,y);
    }

    public ValueStamp[] getValue() throws ErrorInCommandException, IOException, InvalidReportFormat {
        return this.getValue(0);
    }


    public ValueStamp[] getValue(int window) throws ErrorInCommandException, IOException, InvalidReportFormat {
        ValueStamp[] x;
        if (waveform.getStatus().isRunning()) {
            x = control.getStatValue(stat.waveform, stat.object, stat.name,window);
            stat.size = x.length;
        } else {
            x = new ValueStamp[1];
            x[0]=new ValueStamp("0",0);
        }
        return x;
    }

    private double tstampTodouble(int tstamp, int idx) {
        return (double) (tstamp + (double) idx / stat.size) ;
    }

    private long tstampToms(int tstamp, int idx) {
        if (objectTslot == 0) 
            objectTslot = waveform.getExecInfo(stat.object).period;
        return (long) ((long) tstamp * (1 + (double) idx / stat.size) * (objectTslot / 1000));
    }

    public void startReport() throws ErrorInCommandException {
        if (isreporting)
            return;

        try {
            control.startstopStatReport(stat.waveform, stat.object, stat.name, true, window, period);
            isreporting = true;
            reader = new iFileReader(ReposFile);
            reader.setLastLinesListener(this);
            reader.seekToEnd();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(StatControl.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(StatControl.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    public void configReport(String ReposPath) throws FileNotFoundException {
        this.ReposFile=new File(ReposPath + File.separator + "reports" + File.separator + waveform.toString() + File.separator + stat.object + "." + stat.name + ".stat");
        this.ReposPath = ReposPath;
    }
    public void stopReport() throws ErrorInCommandException {
        if (isreporting) {
            isreporting=false;
            reader.stopReader();
            reader=null;
            control.startstopStatReport(stat.waveform, stat.object, stat.name, false,0,0);            
        }
    }
    
    public boolean isReporting() {
        return isreporting;
    }

    public class InvalidReportFormat extends Exception {
        private String file,line;
        public InvalidReportFormat(String file, String line) {
            this.file=file;
            this.line=line;
        }
        @Override
        public String getMessage() {
            return "Error reading stats file " + file + ": invalid format at line: " + line;
        }
    }
}
