/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalControl;

import java.util.Vector;
import phalControl.commandManager.ObjectInfo;
import phalControl.commandManager.ProcInfo;
import phalControl.commandManager.StatInfo;
import phalControl.commandManager.commandManager;
import phalControl.commandManager.commandManager.ErrorInCommandException;

/**
 *
 * @todo implement transitions
 * @author root
 */
public class WaveformControl {

    
    private String name;    
    private commandManager control;
    private Status status;
    private PlatformControl platform;
    private ObjectInfo[] execs;
    
    public WaveformControl(PlatformControl platform, commandManager control, String waveform, Status status) throws ErrorInCommandException {
        this.platform = platform;
        this.control = control;
        name = waveform;
        this.status=status;
        refresh();
/*
        for (int i=0;i<execs.length;i++) {
            StatInfo s = new StatInfo();
            s.name=StatInfo.CPU_STAT_NAME;
            s.object=execs[i].name;
            s.size=1;
            s.type=StatInfo.Type.NUMERIC;
            s.waveform=toString();
            stats.add(new StatControl(control,this,s));
        }
 */
    }

    public void refresh() throws ErrorInCommandException {
        if (!status.isLoaded()) {
            execs = new ObjectInfo[0];
            return;
        }
        execs = control.getExecInfo(name, null);
        Vector procs = new Vector();
        for (int i=0;i<execs.length;i++) {
            if (!procs.contains(execs[i].getPEString())) {
                procs.add(execs[i].getPEString());
            }
        }
        for (int i=0;i<execs.length;i++) {
            execs[i].pecore_idx = procs.indexOf(execs[i].getPEString());
        }
        if (status.isInitiated()) {
            StatInfo[] s = control.getStatsInfo(name, null);
            if (s==null)
                return;
            for (int i=0;i<s.length;i++) {
                if (!stats.contains(s[i].object,s[i].name)) {
                    stats.add(new StatControl(control,this,s[i]));
                }
            }
        }
    }



    @Override
    public String toString() {
        return name;
    }

    public ObjectInfo getExecInfo(String object) {
        for (int i=0;i<execs.length;i++) {
            if (execs[i].name.equals(object)) {
                return execs[i];
            }
        }
        return null;
    }
    public ObjectInfo[] getExecInfo() {
        return execs;
    }
 
    statsVector stats = new statsVector();
    public StatControl getStatControl(String objname, String statname) throws ErrorInCommandException {
        return stats.get(objname, statname);
    }

    public StatControl[] getStatControl(String objname) {
        return stats.getOfObject(objname);
    }
    public StatControl[] getStatControl() {
        return stats.getAll();
    }


    void forceStatus(Status status) {
        this.status=status;
    }

    

    class statsVector extends Vector<StatControl> {
        StatControl[] getOfObject(String objname) {
            Vector x = new Vector();
            for (int i=0;i<this.size();i++) {
                if (get(i).belongs(objname)) {
                    x.add(get(i));
                }
            }
            if (x.size() > 0)
                return (StatControl[]) x.toArray(new StatControl[x.size()]);
            else
                return null;
        }

        boolean contains(String objname, String statname) {
            return get(objname,statname) != null;
        }

        StatControl get(String objname, String statname) {
            for (int i=0;i<this.size();i++) {
                if (get(i).belongs(objname) &&
                        get(i).toString().equals(statname)) {
                    return get(i);
                }
            }
            return null;
        }
        StatControl[] getAll() {
            if (size() == 0) {
                return null;
            } else {
                return (StatControl[]) this.toArray(new StatControl[size()]);
            }
        }
    }




    public void setStatus(Status new_status) throws ErrorInCommandException {
        try {
            control.setWaveformStatus(name, new_status);
            status=new_status;
            if (new_status.isInitiated()) {
                refresh();
            }
        } catch (ErrorInCommandException ex) {
            System.out.println("Error in command for waveform " + name + " deleting waveform...");
            platform.removeWaveform(name);
            throw ex;
        }
        
    }

    public Status getStatus() {
        return status;
    }

    

    


   public enum Status {
        LOADED     ("Loaded", "Loaded", 1, ""),
        INIT       ("INIT", "Initialize waveform", 2, "phinit"),
        RUN        ("RUN", "Run waveform", 3, "phrun"),
        PAUSE      ("PAUSE", "Pause waveform", 4, "phpause"),
        STEP       ("STEP", "Step waveform", 4, "phstep"),
        STOP       ("STOP", "Stop waveform", 5, "phstop"),
        ABNORMAL   ("ABNORMAL", "Abnormal status", 7, "");

        private String ActionString, name, PHcmd;
        private int PHStatus;

        Status(String name, String ActionString, Integer PHStatus, String PHcmd) {
            this.name = name;
            this.ActionString = ActionString;
            this.PHStatus = PHStatus;
            this.PHcmd = PHcmd;
        }

        public String getPHCmd() {
            return PHcmd;
        }

        public int getPHStatus() {
            return this.PHStatus;
        }

        @Override
        public String toString() {
            return this.name;
        }

        public String ActionString() {
            return this.ActionString;
        }

        boolean isRunning() {
            return PHStatus>Status.INIT.PHStatus && PHStatus!=Status.STOP.PHStatus;
        }

        boolean isInitiated() {
            return PHStatus>=Status.INIT.PHStatus && PHStatus!=Status.STOP.PHStatus;
        }

        boolean isLoaded() {
            return PHStatus>=Status.LOADED.PHStatus && PHStatus!=Status.STOP.PHStatus;
        }

        public static Status getByInt(Integer status) {
           Status[] list = Status.values();
           for (int i=0;i<list.length;i++) {
               if (list[i].getPHStatus()==status) {
                   return list[i];
               }
           }
           return null;
        }

        public static Status[] listStatus() {
            Status r[] = new Status[Status.values().length - 1];
            for (int i = 1; i < Status.values().length; i++) {
                r[i - 1] = Status.values()[i];
            }

            return r;
        }
    }


}
