/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalControl.commandManager.Remote;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;
import phalControl.commandManager.ProcInfo;
import phalControl.commandManager.Remote.RemoteSocket.SocketErrorException;
import phalControl.WaveformControl.Status;
import phalControl.StatControl;
import phalControl.commandManager.WaveformInfo;
import phalControl.commandManager.commandManager;
import phalControl.commandManager.ObjectInfo;
import phalControl.commandManager.StatInfo;
import phalgui.Utils.Arrows.ValueStamp;

/**
 *
 * @author Usuario
 */
public class RemoteManager extends commandManager {

    String addr;
    RemoteSocket s = null;

    public RemoteManager(String name, String addr)  throws  ErrorConnectingException {
        super(name);
        this.addr = addr;
        if (addr.indexOf(":") == -1) {
            throw new ErrorConnectingException("Error invalid address: " + addr);
        }
        String ip = addr.substring(0, addr.lastIndexOf(":"));
        int port = Integer.parseInt(addr.substring(addr.lastIndexOf(":") + 1, addr.length()));
        s = new RemoteSocket(ip, port);
    }

    public synchronized boolean isConnected() {
        return s.isConnected();
    }

    public synchronized void Connect() throws ErrorConnectingException {
        try {
            s.Connect();
        } catch (UnknownHostException ex) {
            Logger.getLogger(RemoteManager.class.getName()).log(Level.SEVERE, null, ex);
            throw new ErrorConnectingException("Unkown host: " + addr);
        } catch (SocketErrorException ex) {
            Logger.getLogger(RemoteManager.class.getName()).log(Level.SEVERE, null, ex);
            throw new ErrorConnectingException("Couldn't get I/O for "
                               + "the connection to: " + addr);
        }
    }

    public synchronized void Disconnect() {
        try {
            s.Disconnect();
        } catch (SocketErrorException ex) {
            Logger.getLogger(RemoteManager.class.getName()).log(Level.SEVERE, null, ex);
        }

    }
 

    private synchronized StringAnswer sendCommand(String cmd, String args) throws SocketErrorException {
        s.WriteString(cmd + " " + (args!=null?args:""));
        return new StringAnswer(s.ReadCmd(),s.ReadString());
    }

    @Override
    public synchronized void loadWaveform(String waveform) throws ErrorInCommandException {
        try {
            StringAnswer ans = sendCommand(Command.LOAD.toCmdString(), waveform);
            if (!ans.getCmdOk()) {
                throw new ErrorInCommandException("Error loading waveform: Bad answer");
            }
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error loading waveform: " + ex.getMessage());
        }
    }

    @Override
    public synchronized void setWaveformStatus(String waveform, Status status) throws ErrorInCommandException {
        try {
            StringAnswer ans = sendCommand(status.getPHCmd(), waveform);
            if (!ans.getCmdOk()) {
                throw new ErrorInCommandException("Error setting waveform status: Bad answer");
            }
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error setting waveform status: " + ex.getMessage());
        }
    }

    @Override
    public synchronized ObjectInfo[] getExecInfo(String waveform, String objname) throws ErrorInCommandException {
        ObjectInfo[] execs = null;
        try {
            StringAnswer answer = sendCommand(Command.EXECINFO.toCmdString(), waveform + (objname!=null?(" " + objname):""));
            if (!answer.getCmdOk())
                throw new ErrorInCommandException("Error getting exec info: Bad answer");
            execs = new ObjectInfo[answer.getCmd()];
            int k=0;
            for (int i=0;i<execs.length;i++) {
                execs[i] = new ObjectInfo(
                        answer.readString(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readDouble(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++),
                        answer.readInt(k++));
                execs[i].createXitfs(answer.readInt(k++));
                for (int j=0;j<execs[i].itfs.length;j++) {
                    execs[i].itfs[j].name = answer.readString(k++);
                    execs[i].itfs[j].mode = answer.readInt(k++);
                    execs[i].itfs[j].remoteName = answer.readString(k++);
                    execs[i].itfs[j].remoteObjectName = answer.readString(k++);
                    execs[i].itfs[j].xitf_id = answer.readInt(k++);
                    execs[i].itfs[j].fifo_usage = answer.readInt(k++);
                }
            }
            return execs;
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error getting exec info: " + ex.getMessage());
        }
    }

    @Override
    public synchronized WaveformInfo[] getWaveformsInfo() throws ErrorInCommandException {
        WaveformInfo[] appinfo = null;
        try {
            StringAnswer ans = this.sendCommand(Command.APPLIST.toCmdString(), null);
            if (!ans.getCmdOk())
                throw new ErrorInCommandException("Error listing waveforms: Bad answer");
            appinfo = new WaveformInfo[ans.getCmd()];
            for (int i=0;i<appinfo.length;i++) {
                appinfo[i] = new WaveformInfo(ans.readString(4*i),ans.readInt(4*i+2));
            }
            return appinfo;
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error listing waveforms: " + ex.getMessage());
        }
    }

    public synchronized StatInfo[] getStatsInfo(String waveform, String objname) throws ErrorInCommandException {
        StatInfo[] stats = null;
        try {
            StringAnswer answer = sendCommand(Command.STATLIST.toCmdString(), waveform + (objname!=null?(" " + objname):""));
            if (!answer.getCmdOk())
                return new StatInfo[0];
                //throw new ErrorInCommandException("Error getting stats info: Bad answer");
            stats = new StatInfo[answer.getCmd()];
            for (int i=0;i<stats.length;i++) {
                stats[i] = new StatInfo();
                stats[i].waveform = waveform;
                stats[i].object = answer.readString(4*i);
                stats[i].name = answer.readString(4*i+1);
                stats[i].type = StatInfo.Type.convertFromPhal(answer.readInt(4*i+2));
                stats[i].size = answer.readInt(4*i+3);
            }
            return stats;
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error getting stats info: " + ex.getMessage());
        }
    }

    @Override
    public synchronized ValueStamp[] getStatValue(String waveform, String objname, String statname) throws ErrorInCommandException {
        return this.getStatValue(waveform, objname, statname,0);
    }

    public synchronized ValueStamp[] getStatValue(String waveform, String objname, String statname, int window) throws ErrorInCommandException {
         try {
            StringAnswer ans = this.sendCommand(Command.STATGET.toCmdString(), waveform + " " + objname + " " + statname + ((window!=0)?" " + window:""));
            if (!ans.getCmdOk())
                throw new ErrorInCommandException("Error requesting stat value: Bad answer");
            ValueStamp values[] = new ValueStamp[ans.getCmd()];
            double tstamp = (double) ans.readInt(0);
            StatInfo.PhType type = StatInfo.PhType.getFromInteger(ans.readInt(1));
            for (int i=0;i<values.length;i++) {
                switch(type) {
                    case SHORT:
                    case INT:
                    case CHAR:
                        values[i] = new ValueStamp(String.valueOf(ans.readInt(i+2)),tstamp);
                        break;
                    case FLOAT:
                        values[i] = new ValueStamp(String.valueOf(ans.readDouble(i+2)),tstamp);
                        break;
                    default:
                        return null;
                }
            }
            return values;
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error requesting stat value: " + ex.getMessage());
        }
    }

    @Override
    public synchronized void setStatValue(String waveform, String objname, String statname, String value) throws ErrorInCommandException {
         try {
            StringAnswer ans = this.sendCommand(Command.STATSET.toCmdString(), waveform + " " + objname + " " + statname + " " + value);
            if (!ans.getCmdOk()) {
                throw new ErrorInCommandException("Error setting stat: Bad answer");
            }
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error setting stat: " + ex.getMessage());
        }
    }


    @Override
    public synchronized void startstopStatReport(String waveform, String objname, String statname, boolean start, int window, int period) throws ErrorInCommandException {
         try {
            StringAnswer ans = this.sendCommand(Command.STATREPORT.toCmdString(), (start?"start":"stop") + " " + waveform + " " + objname + " " + statname + " " + window + " " + period);
            if (!ans.getCmdOk()) {
                throw new ErrorInCommandException("Error " + (start?"starting":"stoppping") + " stat report: Bad answer");
            }
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error " + (start?"starting":"stoppping") + " stat report: " + ex.getMessage());
        }
    }

    @Override
    public ProcInfo[] listProcessors() throws ErrorInCommandException {
        try {
            StringAnswer ans = this.sendCommand(Command.PELIST.toCmdString(),null);
            if (!ans.getCmdOk())
                throw new ErrorInCommandException("Error listing processors: Bad answer");
            ProcInfo procs[] = new ProcInfo[ans.getCmd()];
            int k=0;
            for (int i=0;i<procs.length;i++) {
                procs[i] = new ProcInfo();
                procs[i].name = ans.readString(k++);
                procs[i].id = ans.readInt(k++);
                procs[i].plat_family = ans.readInt(k++);
                int nof_cores = ans.readInt(k++);
                procs[i].C = ans.readInt(k++);
                procs[i].intBW = ans.readInt(k++);
                procs[i].totalC = ans.readInt(k++);
                procs[i].totalintBW = ans.readInt(k++);
                procs[i].createXitfs(ans.readInt(k++));
                for (int j=0;j<procs[i].xitfs.length;j++) {
                    procs[i].xitfs[j].remote_id = ans.readInt(k++);
                    procs[i].xitfs[j].remote_pe = ans.readInt(k++);
                    procs[i].xitfs[j].id = ans.readInt(k++);
                    procs[i].xitfs[j].mode = ans.readInt(k++);
                    procs[i].xitfs[j].delay = ans.readInt(k++);
                    procs[i].xitfs[j].BW = ans.readInt(k++);
                    procs[i].xitfs[j].totalBW = ans.readInt(k++);
                }
               procs[i].setCores(nof_cores);
            }
            return procs;
        } catch (SocketErrorException ex) {
            throw new ErrorInCommandException("Error listing processos: " + ex.getMessage());
        }
    }







    public enum Command {

        CONNECT("Connect",""),
        DISCONNECT("Disconnect",""),
        APPLIST("List apps","applist"),
        PELIST("List processors","pelist"),
        LOAD("Load waveform","phload"),
        STATGET("Get stat value","statget"),
        STATLIST("List stats","statlist"),
        EXECINFO("Get Exec info","execinfo"),
        STATREPORT("Report stat value","statreport"),
        STATSET("Set stat value","statset");

        Command(String Desc, String phcmd) {
            this.Desc=Desc;
            this.phcmd=phcmd;
        }

        public String toCmdString() {
            return this.phcmd;
        }
        @Override
        public String toString() {
            return this.Desc;
        }
        private String Desc;
        private String phcmd;
    }



}
