/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Control;

import java.util.concurrent.ExecutionException;
import phalControl.PlatformControl;
import java.io.FileNotFoundException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.SwingWorker;
import phalControl.PlatformControl.Type;
import phalControl.commandManager.commandManager.ErrorConnectingException;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Node.Processor.ProcessorNode;
import phalgui.Tree.MainTree;
import phalgui.Utils.FileToTextArea;
import phalgui.Utils.JTabbedPaneWithIcon;

/**
 *
 * @author ismael
 */
public class MainControl {

    private PlatformControl platform;
    private StatusBar statusBar;
    private OutputLog output;
    private JTabbedPaneWithIcon tabPane;
    private JFrame mainFrame;
    private FileToTextArea output_reader;
    private boolean isconnecting;
    private MainTree tree;

    public MainControl(OutputLog output, JTabbedPaneWithIcon tabPane, JFrame mainFrame,
            StatusBar statusBar, MainTree tree) {
        this.tabPane = tabPane;
        this.mainFrame = mainFrame;
        this.statusBar = statusBar;
        this.output = output;
        this.tree = tree;
    }

    public void StatusReset() {
        this.statusBar.reset();
    }

    public void StatusShow(String string) {
        this.statusBar.show(string);
    }

    public JFrame getFrame() {
        return mainFrame;
    }

    public PlatformControl getPlatform() throws PlatformNotConnected {
        if (platform == null) {
            throw new PlatformNotConnected();
        }
        return platform;
    }

    public JTabbedPaneWithIcon getTabPane() {
        return tabPane;
    }

    public MainTree getTree() {
        return tree;
    }

    class ConnectRemoteThread extends SwingWorker<Boolean,Void> {

        private String name,addr,outputFile,errorMsg;

        private ConnectRemoteThread(String name, String addr, String outputFile) {
            this.name = name;
            this.addr = addr;
            this.outputFile = outputFile;
        }

        @Override
        protected Boolean doInBackground() throws Exception {
            errorMsg = "Unknown error";
            try {
                platform = new PlatformControl(name, addr, PlatformControl.Type.REMOTE);
                platform.Connect();
                return true;
            } catch (ErrorConnectingException ex) {
                Logger.getLogger(MainControl.class.getName()).log(Level.SEVERE, null, ex);
                errorMsg = "Error connecting: " + ex.getMessage();
                return false;
            }
        }

        @Override
        protected void done() {
            isconnecting=false;
            try {
                if (get()) {
                    Log("Connected ok");
                    output.addFileReader(name, outputFile);
                    statusBar.setConnected(platform.toString());
                    tree.getRoot().Refresh();
                    return;
                }
            } catch (FileNotFoundException ex) {
                Logger.getLogger(MainControl.class.getName()).log(Level.SEVERE, null, ex);
                Log("Error can't find output file " + outputFile);
            } catch (InterruptedException ex) {
                Logger.getLogger(MainControl.class.getName()).log(Level.SEVERE, null, ex);
            } catch (ExecutionException ex) {
                Logger.getLogger(MainControl.class.getName()).log(Level.SEVERE, null, ex);
            }
            Log(errorMsg);
            statusBar.setDisconnected();
            platform=null;
        }
    }

    public void connectRemotePlatform(String name, String addr, String outputFile) {
        if (platform != null) {
            Log("Error connecting: Already connected to " + platform.toString());
        }
        Log("Connecting to platform " + name + "...");
        statusBar.setConnecting(name);
        isconnecting=true;
        new ConnectRemoteThread(name,addr,outputFile).execute();
    }

    public boolean isConnecting() {
        return isconnecting;
    }

    public void disconnectPlatform() {
        if (platform == null) {
            Log("Not connected.");
            return;
        }
        output.removeFileReader(platform.toString());
        Log("Disconnecting from " + platform.toString());
        statusBar.setDisconnected();
        platform.Disconnect();
        platform = null;
    }

    public boolean isConnected() {
        if (platform == null) {
            return false;
        } else {
            return platform.isConnected();
        }
    }

    public void Log(String text) {
        output.Log(text);
    }

    public void StatusBar(String text) {
        statusBar.setText(text);
    }

    /** Exceptions */
    public class PlatformNotConnected extends Exception {

        public PlatformNotConnected() {
            super();
        }

        @Override
        public String getMessage() {
            return "Platform not connected.";
        }
    }
}
