/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Control;

import java.util.concurrent.ExecutionException;
import phalgui.*;
import phalControl.PlatformControl.WaveformNotFoundException;
import phalControl.WaveformControl;
import phalControl.PlatformControl;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import phalControl.WaveformControl.Status;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Control.MainControl;
import phalgui.Control.MainControl.PlatformNotConnected;
import phalgui.Node.Waveform.WaveformEntry;

/**
 *
 * @author ismael
 */
public class StatusPanel {

    private String waveformName;
    private MainControl main;
    private buttons buttons;
    private WaveformEntry waveform;

    public StatusPanel(MainControl main, JButton LoadButton, JButton InitButton, JButton RunButton,
            JButton PauseButton, JButton StopButton) {

        this.main = main;
        this.buttons = new buttons(LoadButton, InitButton, RunButton, PauseButton, StopButton);
        buttons.reset();
        this.buttons.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().equals(Status.LOADED.toString())) {
                    Load();
                    
                } else if (e.getActionCommand().equals(Status.INIT.toString())) {
                    setStatus(Status.INIT);
                } else if (e.getActionCommand().equals(Status.RUN.toString())) {
                    setStatus(Status.RUN);
                } else if (e.getActionCommand().equals(Status.PAUSE.toString())) {
                    if (buttons.status.equals(Status.PAUSE)) {
                        setStatus(Status.STEP);
                    } else {
                        setStatus(Status.PAUSE);
                    }
                } else if (e.getActionCommand().equals(Status.STOP.toString())) {
                    setStatus(Status.STOP);
                }
            }
        });
    }

  

    public void selectWaveform(WaveformEntry waveform) {
        waveformName = waveform.toString();
        this.waveform = waveform;
        try {
            buttons.setStatus(main.getPlatform().getWaveform(waveformName).getStatus());
        } catch (WaveformNotFoundException ex) {
            buttons.setSelected();
        } catch (PlatformNotConnected ex) {
            Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void setStatus(final Status new_status) {
        if (waveformName == null) {
            main.Log("Error loading waveform: Any waveform selected.");
            return;
        }
        main.Log("Setting status " + new_status.toString() + " for waveform " + waveformName);
        main.StatusShow("Setting status " + new_status.toString() + " for waveform " + waveformName);
        buttons.setEnabled(false);

        SwingWorker w = new SwingWorker<Boolean, Void>() {

            String errorMsg = new String();

            @Override
            protected Boolean doInBackground() throws Exception {
                try {
                    if (new_status == Status.STOP) {
                        main.getPlatform().stopWaveform(waveformName);
                    } else {
                        main.getPlatform().getWaveform(waveformName).setStatus(new_status);
                    }
                    return true;
                } catch (WaveformNotFoundException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                    errorMsg = ex.getMessage();
                } catch (PlatformNotConnected ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                    errorMsg = ex.getMessage();
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                    errorMsg = ex.getMessage();
                }
                return false;
            }

            @Override
            public void done() {
                try {
                    if (get()) {
                        buttons.setStatus(new_status);
                        if (new_status == Status.INIT) {
                            waveform.Refresh();
                        }
                        main.StatusReset();
                        return;
                    }
                } catch (InterruptedException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                } catch (ExecutionException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                }
                main.Log("Error setting status " + new_status.toString() + " for waveform " + waveformName.toString() + ": " + errorMsg);
                buttons.reset();
                main.StatusReset();
            }
        };
        w.execute();
    }

    public void Load() {
        if (waveformName == null) {
            main.Log("Error loading waveform: Any waveform selected.");
            return;
        }
        main.Log("Loading waveform " + waveformName + "...");
        main.StatusShow("Loading waveform " + waveformName + "...");
        buttons.setEnabled(false);

        SwingWorker w = new SwingWorker<Boolean, Void>() {

            String errorMsg = new String();

            @Override
            protected Boolean doInBackground() throws Exception {
                try {
                    main.getPlatform().loadWaveform(waveformName);
                    return true;
                } catch (PlatformNotConnected ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                    errorMsg = ex.getMessage();
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                    errorMsg = ex.getMessage();
                }
                return false;
            }

            @Override
            public void done() {
                try {
                    if (get()) {
                        buttons.setStatus(Status.LOADED);
                        main.Log("Waveform " + waveformName + " loaded");
                        main.StatusReset();
                        return;
                    }
                } catch (InterruptedException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                } catch (ExecutionException ex) {
                    Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
                }
                main.Log("Error loading waveform " + waveformName.toString() + ": " + errorMsg);
                buttons.reset();
                main.StatusReset();
            }
        };
        w.execute();
    }

    private class buttons {

        JButton LoadButton, InitButton, RunButton, PauseButton, StopButton;
        Status status;

        buttons(JButton LoadButton, JButton InitButton, JButton RunButton,
                JButton PauseButton, JButton StopButton) {
            this.LoadButton = LoadButton;
            this.InitButton = InitButton;
            this.RunButton = RunButton;
            this.PauseButton = PauseButton;
            this.StopButton = StopButton;
        }

        public void addActionListener(ActionListener act) {
            this.LoadButton.addActionListener(act);
            this.LoadButton.setActionCommand(Status.LOADED.toString());

            this.InitButton.addActionListener(act);
            this.InitButton.setActionCommand(Status.INIT.toString());

            this.RunButton.addActionListener(act);
            this.RunButton.setActionCommand(Status.RUN.toString());

            this.PauseButton.addActionListener(act);
            this.PauseButton.setActionCommand(Status.PAUSE.toString());

            this.StopButton.addActionListener(act);
            this.StopButton.setActionCommand(Status.STOP.toString());
        }

        void reset() {
            setEnabled(false, false, false, false, false);
            status = null;
        }

        void setSelected() {
            setEnabled(true, false, false, false, false);
            status = null;
        }

        void setStatus(Status new_status) {
            final boolean paused;
            switch (new_status) {
                case LOADED:
                    setEnabled(false, true, false, false, true);
                    paused = false;
                    break;
                case INIT:
                    setEnabled(false, false, true, false, true);
                    paused = false;
                    break;
                case RUN:
                    setEnabled(false, false, false, true, true);
                    paused = false;
                    break;
                case PAUSE:
                case STEP:
                    setEnabled(false, false, true, true, true);
                    paused = true;
                    break;
                case STOP:
                    setEnabled(true, false, false, false, false);
                    paused = false;
                    break;
                default:
                    paused = false;
                    break;
            }
            SwingUtilities.invokeLater(new Runnable() {

                public void run() {
                    if (paused) {
                        PauseButton.setIcon(PhalguiApp.getApplication().getView().getResources().getIcon("jStepButton.icon"));
                    } else {
                        PauseButton.setIcon(PhalguiApp.getApplication().getView().getResources().getIcon("jPauseButton.icon"));
                    }
                }
            });
            status = new_status == Status.STEP ? Status.PAUSE : new_status;
        }

        private void setEnabled(boolean b) {
            this.setEnabled(b, b, b, b, b);
        }

        private void setEnabled(boolean load, boolean init, boolean run, boolean pause, boolean stop) {
            LoadButton.setEnabled(load);
            InitButton.setEnabled(init);
            RunButton.setEnabled(run);
            PauseButton.setEnabled(pause);
            StopButton.setEnabled(stop);
        }
    }
}
