/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.Executable;


import java.awt.Color;
import java.awt.Font;
import java.text.DecimalFormat;
import java.util.Vector;
import javax.swing.JToggleButton;
import no.geosoft.cc.geometry.Geometry;
import no.geosoft.cc.graphics.GPosition;
import no.geosoft.cc.graphics.GSegment;
import no.geosoft.cc.graphics.GStyle;
import no.geosoft.cc.graphics.GText;
import phalControl.commandManager.ObjectInfo;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Control.MainControl;
import phalgui.Control.MainControl.PlatformNotConnected;
import phalgui.Node.Stats.MyColors;
import phalgui.Utils.Graphics.GPort;
import phalgui.Utils.Graphics.GRectLine;
import phalgui.Utils.Graphics.GraphicsObject;

/**
 *
 * @author ismael
 */
public class ExecutableGraphic extends GraphicsObject implements GRectLine.itfBlock,GPort.PosInterface  {
    private GSegment box;
    private ObjectInfo execInfo;

    private double width=100;
    private double height=width;
    private MainControl main;

    private GRectLine lines[];
    private GText mipsText;
    private ViewInfo actualView;
    private int ninputs=0;
    private GPort[] ports;
    private int noutputs=0;
    private final JToggleButton showMapping;

    public enum ViewInfo {
        USECMAX("CPU max (usec)"),
        USECMEAN("CPU mean (usec)"),
        USAGE("CPU usage (%)"),
        MACS("Max MAC");

        ViewInfo(String desc) {
            this.desc = desc;
        }
        private String desc;
        @Override
        public String toString() {
            return desc;
        }
    };

    public ExecutableGraphic(MainControl main, ObjectInfo execInfo, double x, double y, JToggleButton showMapping) throws PlatformNotConnected, ErrorInCommandException {
        super(execInfo.name,x,y);
        this.main = main;
        this.execInfo = execInfo;
        this.showMapping=showMapping;

        box = new GSegment();
        addSegment (box);
        GText title = new GText (execInfo.name, GPosition.MIDDLE | GPosition.BOTTOM);
        GStyle s = new GStyle();
        s.setFont(new Font ("Dialog", Font.BOLD, 9));
        title.setStyle(s);
        box.addText (title);
        mipsText = new GText (" us", GPosition.MIDDLE | GPosition.TOP);
        s = new GStyle();
        s.setFont(new Font ("Dialog", Font.PLAIN, 9));
        mipsText.setStyle(s);
        box.addText(mipsText);

        int w=execInfo.name.length()*7+10;
        if (w>100) {
            width=w;
        }

        actualView = ViewInfo.USECMAX;
        updateInfo(execInfo);

        ports = new GPort[ninputs];
        int k=0;
        for (int i=0;i<execInfo.itfs.length;i++) {
            if (execInfo.itfs[i].isReadOnly()) {
                ports[k++] = new GPort(this,i,10,GPort.Mode.LEFT);
            }
        }
    }

    public void selectView(ViewInfo view) {
        printUsage(view, mipsText);
        this.actualView = view;
    }

    public void printUsage(ViewInfo view, GText text) {
        switch(view) {
            case USECMAX:
                text.setText(String.valueOf(execInfo.max_usec) + " us");
                break;
            case MACS:
                text.setText(String.valueOf(execInfo.max_mac));
                break;
            case USECMEAN:
                text.setText(String.valueOf(execInfo.getCPUusec()) + " us");
                break;
            case USAGE:
                if (execInfo.period > 0) {
                    Double usage2 = new Double(execInfo.getCPUusage()*100);
                    DecimalFormat format2 = new DecimalFormat("##.#");
                    text.setText(format2.format(usage2) + "%");
                } else {
                    text.setText("N/A");
                }
                break;
        }
    }

    @Override
    public String toString() {
        return execInfo.name;
    }

    private Color colorUtilization(double usage) {
        return new Color((int) ((1-usage)*255),(int) (usage*255),0);
    }

    public void updateInfo(ObjectInfo execInfo) {
        this.execInfo = execInfo;
        printUsage(actualView, mipsText);

        ninputs=0;
        noutputs=0;
        for (int i=0;i<execInfo.itfs.length;i++) {
            if (execInfo.itfs[i].isReadOnly()) {
                ninputs++;
                execInfo.itfs[i].order=ninputs;
            } else {
                noutputs++;
                execInfo.itfs[i].order=noutputs;
            }
        }
        
        GStyle boxStyle = new GStyle();
        if (showMapping.isSelected()) {
            boxStyle.setBackgroundColor(execInfo.getPECoreColor());
        } else {
            if (execInfo.period>0) {
                boxStyle.setBackgroundColor(colorUtilization(1-execInfo.getCPUusage()));
            } else {
                boxStyle.setBackgroundColor(colorUtilization(1));
            }
        }
        box.setStyle(boxStyle);

        udpateLines();

        refresh();
    }

    public void udpateLines() {

        lines = (GRectLine[]) linesVect.toArray(new GRectLine[linesVect.size()]);

        if(lines != null) {
            for (int i=0;i<lines.length;i++) {
                int j = lines[i].getLocalIdx();
                GStyle lineStyle = new GStyle();
                //lineStyle.setForegroundColor(colorUtilization(execInfo.itfs[j].fifo_usage));
                lineStyle.setLineWidth(1);
                lineStyle.setFont(new Font ("Dialog", Font.PLAIN, 9));
                lines[i].setStyle(lineStyle);
                //lines[i].setText(execInfo.itfs[j].fifo_usage + " B");
            }
        }
    }

    
    public boolean connectsWith(ExecutableGraphic pg) {
        if (pg == this)
            return false;
        for (int i=0;i<execInfo.itfs.length;i++) {
            if (execInfo.itfs[i].remoteExec == pg.execInfo
                    && execInfo.itfs[i].isWriteOnly()) {
                return true;
            }
        }
        return false;
    }

    Vector linesVect = new Vector();

    public void connectWith(ExecutableGraphic pg) {
        if (pg == this)
            return;
        for (int i=0;i<execInfo.itfs.length;i++) {
            for (int j=0;j<pg.execInfo.itfs.length;j++) {
                if (execInfo.itfs[i] == pg.execInfo.itfs[j].remoteItf
                        && execInfo.itfs[i].isWriteOnly()) {
                    boolean found=false;
                    if (lines != null) {
                        for (int k=0;k<lines.length;k++) {
                            if (lines[k] != null && lines[k].equals(this,pg,i,j)) {
                                found=true;
                                linesVect.add(lines[k]);
                            }
                        }
                    }
                    if (!found) {
                        GRectLine line = new GRectLine(null,this,pg,i,j,30,80);
                        linesVect.add(line);
                    }
                }
            }
        }
    }
     

    @Override
    public void draw() {
        box.setGeometryXy (Geometry.createRectangle(getX(), getY(), width, height));
        if (lines != null) {
            for (int i=0;i<lines.length;i++) {
                if (lines[i] != null)
                    lines[i].draw();
            }
        }
        if (ports != null) {
            for (int i=0;i<ports.length;i++) {
                if (ports[i] != null) {
                    ports[i].draw();
                }
            }
        }
    }

    @Override
    public void translate (GSegment segment, int dx, int dy) {
        super.translate(segment, dx, dy);
    }


    public double getPortX(int idx) {
        if (execInfo.itfs.length>idx) {
            if (execInfo.itfs[idx].isReadOnly()) {
                return getX();
            } else {
                return getX()+width;
            }
        } else {
            return 0;
        }
    }

    public double getPortY(int idx) {
        if (execInfo.itfs.length>idx) {
            if (execInfo.itfs[idx].isReadOnly()) {
                return getY()+(execInfo.itfs[idx].order)*height/(ninputs+1);
            } else {
                return getY()+(execInfo.itfs[idx].order)*height/(noutputs+1);
            }
        } else {
            return 0;
        }
    }

    private boolean throughput_visible=false;

    @Override
    public void setSelected(boolean selected) {
        if (!selected)
            return;
        if (throughput_visible) {
            throughput_visible=false;
            for (int i=0;i<lines.length;i++) {
                if (lines[i] != null)
                    lines[i].setText("");
            }
        } else {
            throughput_visible=true;
            for (int i=0;i<lines.length;i++) {
                if (lines[i] != null)
                    lines[i].setText(execInfo.itfs[lines[i].getLocalIdx()].fifo_usage/1000 + " KB " + execInfo.itfs[lines[i].getLocalIdx()].fifo_usage*1000/execInfo.period + "KB/s");
            }
        }
    }


}
