/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.MultiPanel;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Properties;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import phalgui.Control.MainControl;
import phalgui.Node.NodeObject;
import phalgui.Tree.TreeMenuItem;
import phalgui.Tree.TreePopup;
import phalgui.Utils.DnD.TransfObject;
import phalgui.Utils.DnD.TransfObject.TransfType;
import phalgui.Utils.DnD.TransfResizePanel;

/**
 *
 * @author root
 */
public class MultiPanelNode extends NodeObject {

    String name;
    MainControl main;
    JMenu removeComponentsMenu;
    public MultiPanelNode(MainControl main, String name) {
        this(main, name, null);
    }
   
    public MultiPanelNode(MainControl main, String name, ImageIcon icon) {
        super(name, icon);
        this.main = main;
        this.name = name;

        setPopup(new TreePopup(buildMenu()));
        removeComponentsMenu = getPopup().addMenu("Remove Components");
    }

    public TreeMenuItem[] buildMenu() {
        TreeMenuItem items[] = new TreeMenuItem[3];
        items[0] = new TreeMenuItem("View Panel") {
            public void action() { Edit(); }
            public boolean actionEnabled() { return panel_editor == null; }
        };
        items[1] = new TreeMenuItem("Rename") {
            public void action() { Rename(); }
            public boolean actionEnabled() { return true; }
        };
        items[2] = new TreeMenuItem("Remove") {
            public void action() { Remove(); }
            public boolean actionEnabled() { return true; }
        };
        return items;
    }

    @Override
    public void DoubleClick(MouseEvent evt) {
        Edit();
        for (int i=0;i<panels.size();i++) {
            ((TransfResizePanel) panels.get(i)).refreshPanel();
        }
    }

    MultiPanelView panel_editor;
    public void Edit() {
        if (panel_editor == null) {
            panel_editor  = new MultiPanelView(this,main.getTabPane());
            if (config != null)
                ReadConfigAfter(config);
        }
        if (!main.getTabPane().hasComponent(panel_editor)) {
            main.getTabPane().addTab("Panel " + toString(), panel_editor, this);
        }
        main.getTabPane().setSelectedComponent(panel_editor);
    }

    @Override
    public String toString() {
        return name;
    }


    @Override
    public void Refresh() {
        for (int i=0;i<panels.size();i++) {
            ((TransfResizePanel) panels.get(i)).refreshPanel();
        }
    }

    @Override
    public boolean WriteConfig(Properties config) {
        if (panel_editor == null) {
            return true;
        }
        TransfResizePanel[] panelsComp =  panel_editor.getPanels();
        if (panelsComp == null) {
            return true;
        }
        String p = new String();
        for (int i=0;i<panelsComp.length;i++) {
            config.setProperty("panel.component." + toString() + "." + panelsComp[i].toString() + ".type", TransfType.valueOf(panelsComp[i]).toString());
            String info[] = panelsComp[i].getTransferData();
            config.setProperty("panel.component." + toString() + "."+ panelsComp[i] + ".nofParams",String.valueOf(info.length));
            for (int j=0;j<info.length;j++) {
                if (info[j]!=null)
                    config.setProperty("panel.component." + toString() + "."+ panelsComp[i] + ".param" + j, info[j]);
            }
            config.setProperty("panel.component." + toString() + "."+ panelsComp[i].toString() + ".x", new Integer( panelsComp[i].getX()).toString());
            config.setProperty("panel.component." + toString() + "."+ panelsComp[i].toString() + ".y", new Integer( panelsComp[i].getY()).toString());
            config.setProperty("panel.component." + toString() + "."+ panelsComp[i].toString() + ".width", new Integer( panelsComp[i].getWidth()).toString());
            config.setProperty("panel.component." + toString() + "."+ panelsComp[i].toString() + ".height", new Integer( panelsComp[i].getHeight()).toString());
            p = p.concat(panelsComp[i].toString() + ",");
        }
        /* remove last ',' and write list */
        if (p.length() > 0) {
            p = p.substring(0, p.length() - 1);
            if (p.length() > 0) {
                config.setProperty("panel." + toString() + ".components", p);
            }
        }
        return true;
    }
    Properties config;
    @Override
    public boolean ReadConfig(Properties config) {
        this.config = config;
        return true;
    }

    public boolean ReadConfigAfter(Properties config) {
        String p = config.getProperty("panel." + toString() + ".components");
        if (p == null)
            return true;
        String panelsNames[] = p.split(",");
        for (int i=0;i<panelsNames.length;i++) {
            String type = config.getProperty("panel.component." + toString() + "."+ panelsNames[i] + ".type");
            TransfType transfType = TransfType.valueOfStr(type);
            String info[] = new String[Integer.valueOf(config.getProperty("panel.component." + toString() + "."+ panelsNames[i] + ".nofParams"))];
            for (int j=0;j<info.length;j++) {
                info[j] = config.getProperty("panel.component." + toString() + "." + panelsNames[i] + ".param" + j);
            }
            if (transfType != null) {
                TransfResizePanel panel = new TransfObject(transfType, info).newObject(TransfResizePanel.PANEL);
                int x = new Integer(config.getProperty("panel.component." + toString() + "." + panelsNames[i] + ".x")).intValue();
                int y = new Integer(config.getProperty("panel.component." + toString() + "." + panelsNames[i] + ".y")).intValue();
                int width = new Integer(config.getProperty("panel.component." + toString() + "." + panelsNames[i] + ".width")).intValue();
                int height = new Integer(config.getProperty("panel.component." + toString() + "." + panelsNames[i] + ".height")).intValue();
                panel_editor.addPanel(panel,new Rectangle(x,y,width,height));
            }
                        
                        /*
            if (transfType != null) {
                switch(transfType) {
                    case LOG:
                        LogView log;
                        try {
                            log = new LogView(panelsNames[i]);
                            int x = new Integer(config.getProperty("panel.component." + panelsNames[i] + ".x")).intValue();
                            int y = new Integer(config.getProperty("panel.component." + panelsNames[i] + ".y")).intValue();
                            int width = new Integer(config.getProperty("panel.component." + panelsNames[i] + ".width")).intValue();
                            int height = new Integer(config.getProperty("panel.component." + panelsNames[i] + ".height")).intValue();
                            panel_editor.addPanel(log,new Rectangle(x,y,width,height));
                            log.ReadConfig(config);
                        } catch (FileNotFoundException ex) {
                            Logger.getLogger(MultiPanelNode.class.getName()).log(Level.SEVERE, null, ex);
                        }
                        break;
                    case STAT:
                        break;
                    case PROC:
                        ProcessorView proc;

                        break;
                }

            }
             */
        }
        Refresh();
        return true;
    }

    @Override
    public boolean Close() {
        for (int i=0;i<panels.size();i++) {
            ((TransfResizePanel) panels.get(i)).Close();
        }
        return true;
    }
    Vector panels = new Vector<TransfResizePanel>();
    void addPanel(final TransfResizePanel panel) {
        panels.add(panel);
        if (panel_editor == null)
            return;
        TreeMenuItem i = new TreeMenuItem(panel.toString()) {
            public void action() {
                removeComponentsMenu.remove(this);
                panel_editor.droppanel.remove(panel);
                panel.Close();
                panel_editor.droppanel.validate();
                panel_editor.droppanel.repaint();
            }
            public boolean actionEnabled() { return panel_editor.isShowing(); }
        };
        i.addActionListener(i);
        removeComponentsMenu.add(i);
    }

}
