/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Node;

import javax.swing.tree.MutableTreeNode;
import phalgui.Tree.*;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.datatransfer.Transferable;
import java.awt.event.MouseEvent;
import java.util.Properties;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import phalgui.PhalguiApp;

/**
 *
 * @author root
 */
public abstract class NodeObject  extends DefaultMutableTreeNode{

    TreePopup popup = null;

    private String name;
    JLabel cell = new JLabel();
    DefaultTreeCellRenderer defaultRenderer = new DefaultTreeCellRenderer();
    Color backgroundSelectionColor;
    Color backgroundNonSelectionColor;
    Color SelectionBorder;
    private ImageIcon icon;

    Vector listeners = new Vector();

    public NodeObject() {
        this(null,null);
    }

    public NodeObject(String name) {
        this(name,null);
    }

    public NodeObject(String name, ImageIcon icon) {
        super();
        this.name = name;
        this.icon = icon;

        backgroundSelectionColor = defaultRenderer
        .getBackgroundSelectionColor();
    backgroundNonSelectionColor = defaultRenderer
        .getBackgroundNonSelectionColor();
    
        SelectionBorder = defaultRenderer.getBorderSelectionColor();
    }

    public void addNodeListener(NodeListener listener) {
        listeners.add(listener);
    }

    public NodeObject findNodeName(String name) {
        if (toString().equals(name)) {
            return this;
        }
        for (int i=0;i<this.getChildCount();i++) {
            NodeObject x = this.getChildAt(i).findNodeName(name);
            if (x != null) {
                return x;
            }
        }
        return null;
    }

    public void removeNodeListener(NodeListener listener) {
        listeners.remove(listener);
    }

    public void add(MutableTreeNode node) {
        super.add(node);
        callListeners(node);
    }

    public void remove(int idx) {
        callListeners(((MutableTreeNode) this.getChildAt(idx)));
        super.remove(idx);
    }
    public void remove(MutableTreeNode node) {
        super.remove(node);
        callListeners(node);
    }

    public void removeAllChilds() {
       while(getChildCount()>0) {
           this.remove(0);
       }
    }

    @Override
    public NodeObject getChildAt(int idx) {
        return (NodeObject) super.getChildAt(idx);
    }

    public NodeObject getRoot() {
        return (NodeObject) super.getRoot();
    }

    public NodeObject getParent() {
        return (NodeObject) super.getParent();
    }

    public void removeSelf() {
        removeAllChilds();
        ((NodeObject) this.getParent()).remove(this);
    }



    public boolean hasIcon() {
        return getIcon() != null;
    }

    public ImageIcon getIcon() {
        return icon;
    }

    public void setIcon(ImageIcon icon) {
        this.icon = icon;
    }

    public void Redraw() {

    }

    @Override
    public String toString() {
        if (name != null) {
            return name;
        } else {
            return "";
        }
    }
   
    public Component getCellComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        if (!this.hasIcon()) {
            Component render = defaultRenderer.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);

            /*FontMetrics fm = render.getFontMetrics(render.getFont());
            int width = fm.stringWidth(value.toString());
            render.setMinimumSize(new Dimension(width + 50, fm.getHeight()));
            render.setPreferredSize(new Dimension(width + 50, fm.getHeight()));
            */
            return render;
        }
        cell.setText(this.toString());
        cell.setIcon(this.getIcon());
        if (selected) {
            cell.setForeground(Color.white);
            cell.setBackground(backgroundSelectionColor);
            cell.setBorder(BorderFactory.createLineBorder(SelectionBorder));
            cell.setOpaque(true);
        } else {
            cell.setForeground(Color.black);
            cell.setBackground(backgroundNonSelectionColor);
            cell.setOpaque(false);
            cell.setBorder(null);
        }

        FontMetrics fm = cell.getFontMetrics(cell.getFont());
        int width = fm.stringWidth(value.toString());
        cell.setPreferredSize(new Dimension(width + 50, fm.getHeight()));
       // cell.setBounds(cell.getBounds().x,cell.getBounds().y,width,fm.getHeight());

        return cell;
    }


    public void setPopup(TreePopup popup) {
        this.popup = popup;
    }
    public TreePopup getPopup() {
        return this.popup;
    }
    public void RightClick(MouseEvent evt) {
        if (popup != null) {
            popup.show((JComponent) evt.getSource(), evt.getX(), evt.getY());
            popup.setVisible(true);
        }
    }

    public void LeftClick(MouseEvent evt) {}
    public void DoubleClick(MouseEvent evt) {}



    public Transferable createTransferable(JComponent c) {
        return null;
    }
    public void exportDone(JComponent c, Transferable data, int action) {
    }

    public void Rename() {
        String s = JOptionPane.showInputDialog("Enter new name:");
        if (s != null)
            name = s;
    }


    public void Remove() {
        this.removeSelf();
    }

    

    public void Selected(TreeSelectionEvent e) {}
    public void Expanded(TreeExpansionEvent e) {}

    public boolean WriteConfig(Properties config) {return true;}
    public boolean ReadConfig(Properties config) {return true;}

    public void Refresh() {
        for (int i=0;i<getChildCount();i++) {
            getChildAt(i).Refresh();
        }
    }
    public boolean isSaved() {return true;}
    public void Save() {}
    public boolean Close() {
        if (isSaved())
            return true;
       String[] options = {"Yes", "No", "Cancel"};
       int n = JOptionPane.showOptionDialog(PhalguiApp.getApplication().getMainFrame(),
            "You are closing the tab. Save changes before?",
            "Close Tab",
            JOptionPane.YES_NO_CANCEL_OPTION,
            JOptionPane.QUESTION_MESSAGE,
            null, options, options[2]);

        if (n == JOptionPane.YES_OPTION) {
            Save();
            return true;
        } else if (n == JOptionPane.NO_OPTION) {
            return true;
        } else {
           return false;
        }
    }

    private void callListeners(MutableTreeNode node) {
        for (int i=0;i<listeners.size();i++) {
            ((NodeListener) listeners.get(i)).nodeAdded(node);
        }
        if (!isRoot())
            getParent().callListeners(node);
    }

    public interface NodeListener {
        public void nodeAdded(MutableTreeNode node);
    }
}

