/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.Processor;

import java.awt.Color;
import java.awt.Font;
import java.text.DecimalFormat;
import java.util.Vector;
import javax.swing.border.TitledBorder;
import no.geosoft.cc.geometry.Geometry;
import no.geosoft.cc.graphics.GPosition;
import no.geosoft.cc.graphics.GSegment;
import no.geosoft.cc.graphics.GStyle;
import no.geosoft.cc.graphics.GText;
import phalControl.commandManager.ProcInfo;
import phalControl.commandManager.ProcInfo.xitf;
import phalgui.Control.MainControl;
import phalgui.Utils.FormatNumber;
import phalgui.Utils.Graphics.GPort;
import phalgui.Utils.Graphics.GRectLine;
import phalgui.Utils.Graphics.GraphicsObject;

/**
 *
 * @author ismael
 */
public class ProcessorGraphic extends GraphicsObject implements GRectLine.itfBlock,GPort.PosInterface {
    private GSegment box[];
    private ProcInfo procInfo;
   
    private final double width=300,height=300;
    private MainControl main;

    private int ninputs=0;
    private int noutputs=0;

    private GPort ports[];
    private GRectLine lines[];
    private GText mipsText;
    private GText numText;
    private ProcessorView procView;
    private boolean isSelected;
    private ViewInfo actualView;
    private GText[] cmipsText;
    private GText[] cnumText;

    public enum ViewInfo {
        MIPS("Free Capacity"),
        TOTALMIPS ("Total Capacity"),
        BW("Free BW"),
        TOTALBW("Total BW"),
        USAGE("CPU time usage (%)");

        ViewInfo(String desc) {
            this.desc = desc;
        }
        private String desc;
        @Override
        public String toString() {
            return desc;
        }
    };

    public ProcessorGraphic(MainControl main, ProcInfo procInfo, double x, double y, ProcessorView procView) {
        super(procInfo.name,x,y);
        this.main = main;
        this.procInfo = procInfo;
        this.procView = procView;

        box = new GSegment[procInfo.nof_cores+1];
        box[0] = new GSegment();
        addSegment(box[0]);

        GStyle s = new GStyle();
        s = new GStyle();
        s.setFont(new Font ("Dialog", Font.BOLD, 9));
        cmipsText = new GText[procInfo.nof_cores];
        cnumText = new GText[procInfo.nof_cores];
        if (procInfo.nof_cores>1) {
            for (int i=0;i<procInfo.nof_cores;i++) {
                GText title = new GText (procInfo.toStringCore(i), GPosition.MIDDLE);
                title.setStyle(s);
                box[i+1] = new GSegment();
                addSegment (box[i+1]);
                cmipsText[i] = new GText (" MACS", GPosition.SOUTHWEST);
                cmipsText[i].setStyle(s);
                cnumText[i] = new GText (" Modules", GPosition.SOUTHEAST);
                cnumText[i].setStyle(s);
                box[i+1].addText(title);
                box[i+1].addText(cnumText[i]);
                box[i+1].addText(cmipsText[i]);
            }
        } else {
          GText title = new GText (procInfo.toStringCore(0), GPosition.MIDDLE);
          s.setFont(new Font ("Dialog", Font.BOLD, 14));
          title.setStyle(s);
          box[0].addText (title);
        }


        actualView = ViewInfo.MIPS;

        mipsText = new GText (" MACS", GPosition.SOUTHWEST);
        mipsText.setStyle(s);
        numText = new GText (" Modules", GPosition.SOUTHEAST);
        numText.setStyle(s);

        box[0].addText(numText);
        box[0].addText(mipsText);
        

        updateInfo(procInfo);

        /*
        for (int i=0;i<procInfo.xitfs.length;i++) {
            if (procInfo.xitfs[i].isReadOnly()) {
                procInfo.xitfs[i].order = ninputs;
                ninputs++;
            } else if (procInfo.xitfs[i].isWriteOnly()) {
                procInfo.xitfs[i].order = noutputs;
                noutputs++;
            }
        }
        
        ports = new GPort[ninputs];
        int k=0;
        for (int i=0;i<procInfo.xitfs.length;i++) {
            if (procInfo.xitfs[i].isReadOnly()) {
                ports[k++] = new GPort(this,i,10,GPort.Mode.LEFT);
            }
        }
         */
    }
    
    public void selectView(ViewInfo view) {
        printUsage(view, mipsText,-1);
        for (int i=0;i<procInfo.nof_cores;i++) {
            printUsage(view,cmipsText[i],i);
        }
        this.actualView = view;
    }

    public void printUsage(ViewInfo view, GText text, int core) {
        switch(view) {
            case MIPS:
                text.setText(FormatNumber.divideInt(core==-1?procInfo.C:procInfo.CoreC[core]) + "MACS");
                break;
            case TOTALMIPS:
                text.setText(FormatNumber.divideInt(core==-1?procInfo.totalC:procInfo.CoretotalC[core]) + "MACS");
                break;
            case BW:
                text.setText(FormatNumber.divideInt(procInfo.intBW) + "Bps");
                break;
            case TOTALBW:
                text.setText(FormatNumber.divideInt(procInfo.totalintBW) + "Bps");
                break;
            case USAGE:
                Double usage2 = new Double((100*(core==-1?procInfo.cpu_usec:procInfo.Corecpu_usec[core])/procInfo.mean_tslot));
                DecimalFormat format2 = new DecimalFormat("##.#");
                text.setText(format2.format(usage2) + "%");
                break;
        }
    }


    @Override
    public String toString() {
        return String.valueOf(procInfo.id);
    }

    private Color colorUtilization(double usage) {
        return new Color((int) ((usage)*255),(int) ((1-usage)*255),0);
    }

    public void updateInfo(ProcInfo procInfo) {
        this.procInfo = procInfo;
        
        printUsage(actualView, mipsText,-1);
        numText.setText(String.valueOf(procInfo.nofObjects==null?0:procInfo.nofObjects));

        if (procInfo.nof_cores>1) {
            for (int i=0;i<procInfo.nof_cores;i++) {
                printUsage(actualView,cmipsText[i],i);
                cnumText[i].setText(String.valueOf(procInfo.nofCoreObjects[i]) + " Modules");
            }
        }
        GStyle boxStyle = new GStyle();
        GStyle cboxStyle[] = new GStyle[procInfo.nof_cores];
        for (int i=0;i<procInfo.nof_cores;i++) {
            cboxStyle[i] = new GStyle();
        }
        if (procInfo.mean_tslot>0) {
            boxStyle.setBackgroundColor(colorUtilization(1-(double) procInfo.C/procInfo.totalC));
            for (int i=0;i<procInfo.nof_cores;i++) {
                cboxStyle[i].setBackgroundColor(colorUtilization(1-(double) procInfo.CoreC[i]/procInfo.CoretotalC[i]));
            }
        } else {
            boxStyle.setBackgroundColor(colorUtilization(0.0));
            for (int i=0;i<procInfo.nof_cores;i++) {
                cboxStyle[i].setBackgroundColor(colorUtilization(0.0));
            }
        }
        boxStyle.setLineStyle(isSelected?GStyle.LINESTYLE_DOTTED:GStyle.LINESTYLE_SOLID);
        box[0].setStyle(boxStyle);
        if (procInfo.nof_cores>1) {
            for (int i=0;i<procInfo.nof_cores;i++) {
                box[i+1].setStyle(cboxStyle[i]);
            }
        }

        udpateLines();

        refresh();
    }

    public void setSelected(boolean select) {
        GStyle boxStyle = box[0].getStyle();
        isSelected = select;
        boxStyle.setLineStyle(select?GStyle.LINESTYLE_DOTTED:GStyle.LINESTYLE_SOLID);
        box[0].setStyle(boxStyle);

        procView.setBorder(new TitledBorder("Processor " + procInfo.name));

        if (select) {
            procView.setProcessor(procInfo);
        }
    }

    public void udpateLines() {
        if(lines != null) {
            for (int i=0;i<lines.length;i++) {
                int j = lines[i].getLocalIdx();
                GStyle lineStyle = new GStyle();
                lineStyle.setBackgroundColor(Color.black);
               // lineStyle.setForegroundColor(colorUtilization(procInfo.xitfs[j].BW/procInfo.xitfs[j].totalBW));
                lineStyle.setLineWidth(1);
                //lines[i].setText(procInfo.xitfs[j].BW + " kBps");
            }
        }
    }
    
    public boolean connectsWith(ProcessorGraphic pg) {
        if (pg == this)
            return false;
        for (int i=0;i<procInfo.xitfs.length;i++) {
            if (procInfo.xitfs[i].remoteProc == pg.procInfo
                    && procInfo.xitfs[i].isWriteOnly()) {
                return true;
            }
        }
        return false;
    }
    
    public void connectWith(ProcessorGraphic pg) {
        if (pg == this)
            return;
        Vector linesVect = new Vector();
        for (int i=0;i<procInfo.xitfs.length;i++) {
            for (int j=0;j<pg.procInfo.xitfs.length;j++) {
                if (procInfo.xitfs[i] == pg.procInfo.xitfs[j].remoteItf
                        && procInfo.xitfs[i].isWriteOnly()) {
                    boolean found=false;
                    if (lines != null) {
                        for (int k=0;k<lines.length;k++) {
                            if (lines[k] != null && lines[k].equals(this,pg,i,j)) {
                                found=true;
                                linesVect.add(lines[k]);
                            }
                        }
                    }
                    if (!found) {
                        GRectLine line = new GRectLine(procInfo.xitfs[i].BW + " kBps",this,pg,i,j,width/2,height);
                        linesVect.add(line);
                    }
                }
            }
        }
        lines = (GRectLine[]) linesVect.toArray(new GRectLine[linesVect.size()]);
        udpateLines();
    }

    @Override
    public void draw() {
        box[0].setGeometryXy (Geometry.createRectangle(getX(), getY(), width, height));
        if (procInfo.nof_cores>1) {
            for (int i=0;i<procInfo.nof_cores;i++) {
                box[i+1].setGeometryXy (Geometry.createRectangle(getX()+i*(width)/procInfo.nof_cores, getY()+50, (width)/procInfo.nof_cores, height-50));
            }
        }
        /*
        for (int i=0;i<ports.length;i++) {
            ports[i].draw();
        }
        if (lines != null) {
            for (int i=0;i<lines.length;i++) {
                if (lines[i] != null)
                    lines[i].draw();
            }
        }
         */
    }

    public double getPortX(int idx) {
        if (procInfo.xitfs.length>idx) {
            if (procInfo.xitfs[idx].isReadOnly()) {
                return getX();
            } else {
                return getX()+width;
            }
        } else {
            return 0;
        }
    }

    public double getPortY(int idx) {
        if (procInfo.xitfs.length>idx) {
            if (procInfo.xitfs[idx].isReadOnly()) {
                return getY()+(procInfo.xitfs[idx].order)*height/(ninputs+1);
            } else {
                return getY()+(procInfo.xitfs[idx].order)*height/(noutputs+1);
            }
        } else {
            return 0;
        }
    }

    @Override
    public void translate (GSegment segment, int dx, int dy) {
        super.translate(segment, dx, dy);
        /*
        for (int i=0;i<ports.length;i++) {
            ports[i].translate(segment, dx, dy);
        }
        */
    }
}
