/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Node.Repositories;

import phalgui.Node.*;
import phalgui.Node.Waveform.WaveformEntry;
import phalgui.Node.Platform.PlatformEntry;
import phalgui.Node.MultiPanel.MultiPanelNode;
import phalgui.Utils.Arrows.ErrorParsingException;
import java.io.File;
import java.io.FilenameFilter;
import java.net.URL;
import javax.swing.ImageIcon;
import phalgui.Control.MainControl;
import phalgui.PhalguiView;
import phalgui.Utils.JTabbedPaneWithIcon;

/**
 *
 * @author root
 */
public class RepositoryObject {

    public enum FileType {

        PLATFORMS("plt", "Platform Manager", "New Platform", "", "", "resources/platform.gif"),
        WAVEFORMS("apps", "Application Repository", "New Application", "swman_apps", ".app", "resources/waveform.gif"),
        WAVEFORMINFO("appinfo", "Application Info", "", "", "", ""),
        PARAMETERS("statsman", "Parameters", "New Parameters Config", "statsman",".params", "resources/parameters.gif"),
        EXECSTATS("exec_stats", "RT stats", "Exec info", "execlogs","", ""),
        STATS("stats", "Variable stats", "Stat", "", "", "resources/graph2.gif"),
        LOG("log", "Logs", "New object's log", "logs",".log", "resources/logs.gif"),
        PANELS("panel", "View Panel", "New View Panel", "","", "resources/graph.gif");


        public final String shortName,  Name,  NewString,  Filter;
        private final ImageIcon icon;
        public final String subdir;

        FileType(String shortName, String Name, String NewString, String subdir, String Filter, String iconPath) {
            this.shortName = shortName;
            this.Name = Name;
            this.NewString = NewString;
            this.Filter = Filter;
            this.subdir = subdir;
            URL url = PhalguiView.class.getResource(iconPath);
            if (url != null) {
                this.icon = new ImageIcon(url);
            } else {
                this.icon = null;
            }
        }

        @Override
        public String toString() {
            return this.Name;
        }

        public ImageIcon getIcon() {
            return icon;
        }

        public String[] ScanFiles(String path) {
            String[] names;
            File dir = new File(path);

            names = dir.list(new myFilter(Filter));

            return names;
        }

        class myFilter implements FilenameFilter {

            String filter;

            myFilter(String filter) {
                this.filter = filter;
            }

            public boolean accept(File dir, String name) {
                if (!filter.equals("<dir>")) {
                    if (name.endsWith(filter) && !name.contains("~")) {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    File f = new File(dir.getPath() + File.separatorChar + name);
                    return f.isDirectory() && !name.startsWith(".");
                }
            }
        }
    };
    
    NodeObject obj;

    public static FileType getType(String property) throws ErrorParsingException {
        for (FileType type : FileType.values()) {
            if (type.shortName.equals(property)) {
                return type;
            }
        }
        throw new ErrorParsingException("Type " + property + " unknown.");
    }

    public RepositoryObject(FileType type, MainControl main,
            String name) throws UnknownRepositoryException {

        switch (type) {
            case PLATFORMS:
                obj = new PlatformEntry(main, name);
                break;
            case PANELS:
                obj = new MultiPanelNode(main, name);
                break;
            default:
                throw new UnknownRepositoryException(type);
        }
    }

    public NodeObject getObject() {
        return obj;
    }

    class UnknownRepositoryException extends Exception {
        private FileType type;
        public UnknownRepositoryException(FileType type) {
            super();
            this.type=type;
        }
        @Override
        public String getMessage() {
            return "Unknown repository type: " + type.toString();
        }
    }
}
