/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Node.Repositories;


import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Control.MainControl.PlatformNotConnected;
import phalgui.Node.*;
import java.io.File;
import java.io.IOException;
import java.util.Properties;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import phalgui.Control.MainControl;
import phalgui.Tree.*;
import phalgui.Node.Repositories.RepositoryObject.FileType;
import phalgui.Node.Waveform.WaveformEntry;
import phalgui.Control.StatusPanel;

/**
 *
 * @author root
 */
public class WaveformRepository extends NodeObject {

    private String ReposPath = new String();
    MainControl main;
    String name;
    private StatusPanel statusPanel;
    private Properties config;

    public WaveformRepository(MainControl main, StatusPanel statusPanel, String name, String ReposPath) {
        this(main, statusPanel, name, ReposPath, null);
    }

    public WaveformRepository(MainControl main, StatusPanel statusPanel, String name, String ReposPath, ImageIcon icon) {
        super(name,icon);
        this.statusPanel = statusPanel;
        this.ReposPath = ReposPath;
        this.main = main;
        this.name = name;
        setPopup(new TreePopup(buildMenu()));
    }
    
    public String getReposPath() {
        return this.ReposPath;
    }
    
    public TreeMenuItem[] buildMenu() {
        TreeMenuItem items[] = new TreeMenuItem[3];
        items[0] = new TreeMenuItem("New Waveform") {
            public void action() { New(); }
            public boolean actionEnabled() { return true; }
        };
        items[1] = new TreeMenuItem("Edit") {
            public void action() { Edit(); }
            public boolean actionEnabled() { return true; }
        };
/*        items[2] = new TreeMenuItem("Remove") {
            public void action() { Remove(); }
            public boolean actionEnabled() { return true; }
        };
 */
        items[2] = new TreeMenuItem("Refresh") {
            public void action() { Reload(); }
            public boolean actionEnabled() { return true; }

        };
        return items;
    }

    public void New() {
        String s = JOptionPane.showInputDialog("Enter a name for waveform");
        if (s != null) {
            if (s.length() > 0) {
                try {
                    add(new WaveformEntry(main, statusPanel, s, new File(ReposPath + File.separator + name + FileType.WAVEFORMS.Filter), FileType.WAVEFORMS.getIcon()));
                } catch (Exception ex) {
                    System.out.println("Error creating repository obejct: " + ex.getMessage());
                }
            }
        }
    }
    
    WaveformRepositoryEdit entry_edit;

    public void Edit() {
        if (entry_edit == null) {
            entry_edit = new WaveformRepositoryEdit(this);
        }
        if (!main.getTabPane().hasComponent(entry_edit)) {
            main.getTabPane().addTab("Edit " + toString(), entry_edit, this);
        }
        main.getTabPane().setSelectedComponent(entry_edit);
    }

    @Override
    public String toString() {
        return name;
    }

    public void Save() {
        name = entry_edit.getReposName();
        ReposPath = entry_edit.getReposPath();

        Refresh();            
    }

    @Override
    public boolean isSaved() {
        if (!toString().equals(entry_edit.getReposName())) {
            return false;
        } else if (!ReposPath.equals(entry_edit.getReposPath())) {
            return false;
        }
        return true;
    }


    private void Reload() {
        SwingWorker w = new SwingWorker<Void,Void>() {

            @Override
            protected Void doInBackground() throws Exception {
                try {
                    main.getPlatform().refresh();
                    return null;
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(WaveformRepository.class.getName()).log(Level.SEVERE, null, ex);
                    return null;
                }
            }
            @Override
            protected void done() {
                Refresh();
            }
        };
        w.execute();
    }


    @Override
    public void Refresh() {
        final String repos_subdir;
        repos_subdir = ReposPath + File.separator + FileType.WAVEFORMS.subdir;
        removeAllChilds();

        SwingWorker w = new SwingWorker<String[],Void>() {

            @Override
            protected String[] doInBackground() throws Exception {
                return FileType.WAVEFORMS.ScanFiles(repos_subdir);
            }
            @Override
            protected void done() {
                String[] local_names;
                try {
                    local_names = get();
                } catch (InterruptedException ex) {
                    Logger.getLogger(WaveformRepository.class.getName()).log(Level.SEVERE, null, ex);
                    return;
                } catch (ExecutionException ex) {
                    Logger.getLogger(WaveformRepository.class.getName()).log(Level.SEVERE, null, ex);
                    return;
                }
                if (local_names == null)
                    return;

                for (int i = 0; i < local_names.length; i++) {
                    try {
                        String wavename;
                        if (local_names[i].lastIndexOf(FileType.WAVEFORMS.Filter) >= 0) {
                            wavename = local_names[i].substring(0, local_names[i].lastIndexOf("."));
                        } else {
                            wavename = local_names[i];
                        }
                        WaveformEntry waveform = new WaveformEntry(main, statusPanel, wavename, new File(ReposPath + File.separator + FileType.WAVEFORMS.subdir + File.separator + local_names[i]), FileType.WAVEFORMS.getIcon());
                        add(waveform);
                        waveform.Refresh();
                    } catch (IOException ex) {
                        Logger.getLogger(WaveformRepository.class.getName()).log(Level.SEVERE, null, ex);
                        main.Log("Error creating waveform entry: " + ex.getMessage());
                    }
                }
            }
        };
        w.execute();
    }

    @Override
    public boolean WriteConfig(Properties props) {
        if (ReposPath != null) {
            props.setProperty("repository.waveforms.path", ReposPath);
        }
        for (int i=0;i<this.getChildCount();i++) {
            ((NodeObject) this.getChildAt(i)).WriteConfig(props);
        }
        return true;
    }

    public Properties getConfig() {
        return this.config;
    }

    @Override
    public boolean ReadConfig(Properties props) {
        ReposPath = props.getProperty("repository.waveforms.path");
        this.config = props;
        return true;
    }
}
