/*
 * GraphicInterface.java
 *
 * Created on 20 de febrero de 2002, 19:00
 */

package phalgui.Node.Stats;

import phalControl.StatControl;
import phalgui.Node.Stats.ViewStatisticsGraphic.CaptureMode;

/**
 * This class is used by the GraphicPanel and the GraphicOptions to use this methods from the
 * classes that implements this interface. This classes are the GraphicFrame and StatisticsGraphic.
 * @author  Santi Oliveras
 * @version 1.0  2/05/2002
 * @see GraphicPanel
 * @see GraphicOptions
 */
public interface GraphicInterface {

    public void addStat(StatControl statControl);

    
    /** Returns the selected variable in the comboBox. */
    public String getSelectedVariable();
    
    /** Adds a new graphic to a numeric graph. Used in logs. Implemented in GraphicFrame. 
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see GraphicFrame#addGraphic(String,String).
     */
    public void addGraphic(String module,String variable);
    
    /** Adds a new graphic to a numeric graph. Used in statistics. Implemented in StatisticsGraphic.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @param id Integer with the id of the variable
     * @param index Integer with the index of the variable
     * @see StatisticsGraphic#addGraphic(String,String,int,int)
     */
    public void addGraphic(String module,String variable,int id, int index);
    
    /** Deletes a variable in a numeric graphic.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see GraphicFrame#deleteGraphic
     * @see StatisticsGraphic#deleteGraphic
     */
    public void deleteGraphic(String module,String variable);

    
    /** Reloads all the data. Is used by the GraphicOptions when the data has to be processed.
     */
    public void reloadAll();
    
    /** Returns the component of the class that implements this method. */
    public java.awt.Component getComponent();
}

