/*
 * GraphicOptions.java
 *
 * Created on 12 de enero de 2002, 23:07
 */

package phalgui.Node.Stats;


import phalgui.Utils.Arrows.Trace;
import phalgui.Utils.Arrows.StringParser;
import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

import phalgui.PhalguiApp;
import phalgui.graph.*;
import phalgui.PhalguiView;
import phalgui.Utils.*;

/**
 * This class shows a dialog to configure the graphic. You can configure the 
 * general options, the axis options and the options of the variables; so, it
 * permits a great number of possibilities. This dialog is shown when the options
 * menu in the graphic panel popup is pressed.
 * @author  Santi Oliveras
 * @version 1.0  2/05/2002
 * @see GraphicPanel
 */
public class GraphicOptions extends javax.swing.JDialog {

    // ******** private variables **********************************************
    
    private String markerFile;      //It's the marker file with its format explained in the file
    private Markers marks;          //The available marker read in the marker file
    private MarkerViewer mw;        //A private class which renders the markers
    private GraphicPanel gp;        //A link to the graphic panel associated
    private java.awt.Frame parent;  //The parent frame
    private boolean focus=true;     //Used when a numeric textField is changed
    private int currentVariable=0;  //The current variable in the variables options
    private int initialUnit=0;      //initial unit of the grapic panel
        
    
    // ******** constructor ****************************************************
    
    /** Creates new form GraphicOptions.
     * @param gi GraphicInterface used to union the options with the object tha contains the 
     * GraphicPanel
     * @param panel A pointer to the panel to be configurated 
     */
    public GraphicOptions(JFrame frame, GraphicInterface gi, GraphicPanel panel) {
        super(frame, true);
        this.parent=frame;
        gp=panel;
        setDefaults();        
        initComponents();
        postInitComponents();
        mw = new MarkerViewer();
        jPanel7.add(mw);
        pack();
        //ComponentManager.centerComponent(gi.getComponent(),this);
        setResizable(false);
    }
    
    
    // ******** private methods ************************************************
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        gridColor = new javax.swing.JComboBox(MyColors.getColors());
        jLabel2 = new javax.swing.JLabel();
        zeroColor = new javax.swing.JComboBox(MyColors.getColors());
        jLabel3 = new javax.swing.JLabel();
        blackGroundColor = new javax.swing.JComboBox(MyColors.getColors());
        gridCheckBox = new javax.swing.JCheckBox();
        jLabel11 = new javax.swing.JLabel();
        visiblePoints = new javax.swing.JTextField();
        visiblePointsCheckBox = new javax.swing.JCheckBox();
        jPanel2 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        axisTitle = new javax.swing.JTextField();
        axisTimeUnit = new javax.swing.JComboBox();
        jLabel16 = new javax.swing.JLabel();
        axisType = new javax.swing.JComboBox();
        jLabel17 = new javax.swing.JLabel();
        axisScale = new javax.swing.JComboBox();
        jPanel9 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        minXRange = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        maxXRange = new javax.swing.JTextField();
        XAutoCheckBox = new javax.swing.JCheckBox();
        jPanel4 = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        minYRange = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        maxYRange = new javax.swing.JTextField();
        YAutoCheckBox = new javax.swing.JCheckBox();
        jPanel8 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jLabel12 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        lineColor = new javax.swing.JComboBox(MyColors.getColors());
        markerColor = new javax.swing.JComboBox(MyColors.getColors());
        markerStyle = new javax.swing.JComboBox(getMarkers());
        jPanel7 = new javax.swing.JPanel();
        markerScale = new javax.swing.JSlider();
        legendText = new javax.swing.JTextField();
        jPanel6 = new javax.swing.JPanel();
        jLabel10 = new javax.swing.JLabel();
        variable = new javax.swing.JComboBox(gp.getDataArray());
        CancelButton = new javax.swing.JButton();
        OKButton = new javax.swing.JButton();

        setTitle("Graphic Options");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });
        getContentPane().setLayout(new java.awt.GridBagLayout());

        jPanel1.setLayout(new java.awt.GridBagLayout());

        jLabel1.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel1.setText("   Grid color:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel1.add(jLabel1, gridBagConstraints);

        gridColor.setRenderer(new ComboBoxRenderer());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
        jPanel1.add(gridColor, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel2.setText("   Zero color:");
        jLabel2.setToolTipText("The color of the line at the zero label.");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel1.add(jLabel2, gridBagConstraints);

        zeroColor.setRenderer(new ComboBoxRenderer());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
        jPanel1.add(zeroColor, gridBagConstraints);

        jLabel3.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel3.setText("   Blackground color:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 0);
        jPanel1.add(jLabel3, gridBagConstraints);

        blackGroundColor.setRenderer(new ComboBoxRenderer());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 10);
        jPanel1.add(blackGroundColor, gridBagConstraints);

        gridCheckBox.setFont(new java.awt.Font("Dialog", 0, 12));
        gridCheckBox.setSelected(true);
        gridCheckBox.setText("With grid");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanel1.add(gridCheckBox, gridBagConstraints);

        jLabel11.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel11.setText("   Max visible points number:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel1.add(jLabel11, gridBagConstraints);

        visiblePoints.setPreferredSize(new java.awt.Dimension(80, 21));
        visiblePoints.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                visiblePointsActionPerformed(evt);
            }
        });
        visiblePoints.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                visiblePointsFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel1.add(visiblePoints, gridBagConstraints);

        visiblePointsCheckBox.setFont(new java.awt.Font("Dialog", 0, 12));
        visiblePointsCheckBox.setText("Only visible points");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanel1.add(visiblePointsCheckBox, gridBagConstraints);

        jTabbedPane1.addTab("Graph options", null, jPanel1, "");

        jPanel2.setLayout(new java.awt.GridBagLayout());

        jLabel4.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel4.setText("Y axis title:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 0);
        jPanel2.add(jLabel4, gridBagConstraints);

        jLabel5.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel5.setText("X axis time unit:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 0);
        jPanel2.add(jLabel5, gridBagConstraints);

        axisTitle.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                axisTitleActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 40, 10, 10);
        jPanel2.add(axisTitle, gridBagConstraints);

        axisTimeUnit.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "seconds", "minutes" }));
        axisTimeUnit.setPreferredSize(new java.awt.Dimension(96, 26));
        axisTimeUnit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                axisTimeUnitActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 50, 10, 10);
        jPanel2.add(axisTimeUnit, gridBagConstraints);

        jLabel16.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel16.setText("Y axis type:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 0);
        jPanel2.add(jLabel16, gridBagConstraints);

        axisType.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "normal", "difference", "integration" }));
        axisType.setMinimumSize(new java.awt.Dimension(95, 26));
        axisType.setPreferredSize(new java.awt.Dimension(96, 26));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 10);
        jPanel2.add(axisType, gridBagConstraints);

        jLabel17.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel17.setText("Y axis scale:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 0);
        jPanel2.add(jLabel17, gridBagConstraints);

        axisScale.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "lineal", "logarithmic" }));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 10);
        jPanel2.add(axisScale, gridBagConstraints);

        jPanel9.setLayout(new java.awt.GridLayout(1, 0, 10, 0));

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "X Range"));
        jPanel3.setMinimumSize(new java.awt.Dimension(72, 99));
        jPanel3.setPreferredSize(new java.awt.Dimension(72, 99));
        jPanel3.setLayout(new java.awt.GridBagLayout());

        jLabel6.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel6.setText("min:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 0);
        jPanel3.add(jLabel6, gridBagConstraints);

        minXRange.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                minXRangeActionPerformed(evt);
            }
        });
        minXRange.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                minXRangeFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 10);
        jPanel3.add(minXRange, gridBagConstraints);

        jLabel7.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel7.setText("max:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 0);
        jPanel3.add(jLabel7, gridBagConstraints);

        maxXRange.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                maxXRangeActionPerformed(evt);
            }
        });
        maxXRange.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                maxXRangeFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 10);
        jPanel3.add(maxXRange, gridBagConstraints);

        XAutoCheckBox.setFont(new java.awt.Font("Dialog", 0, 12));
        XAutoCheckBox.setSelected(true);
        XAutoCheckBox.setText("Auto");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanel3.add(XAutoCheckBox, gridBagConstraints);

        jPanel9.add(jPanel3);

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "Y Range"));
        jPanel4.setLayout(new java.awt.GridBagLayout());

        jLabel8.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel8.setText("min:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 0);
        jPanel4.add(jLabel8, gridBagConstraints);

        minYRange.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                minYRangeActionPerformed(evt);
            }
        });
        minYRange.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                minYRangeFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 10);
        jPanel4.add(minYRange, gridBagConstraints);

        jLabel9.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel9.setText("max:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 0);
        jPanel4.add(jLabel9, gridBagConstraints);

        maxYRange.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                maxYRangeActionPerformed(evt);
            }
        });
        maxYRange.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                maxYRangeFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 10);
        jPanel4.add(maxYRange, gridBagConstraints);

        YAutoCheckBox.setFont(new java.awt.Font("Dialog", 0, 12));
        YAutoCheckBox.setSelected(true);
        YAutoCheckBox.setText("Auto");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanel4.add(YAutoCheckBox, gridBagConstraints);

        jPanel9.add(jPanel4);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        jPanel2.add(jPanel9, gridBagConstraints);

        jTabbedPane1.addTab("Axis options", null, jPanel2, "");

        jPanel8.setLayout(new java.awt.GridBagLayout());

        jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "Variable options"));
        jPanel5.setLayout(new java.awt.GridBagLayout());

        jLabel12.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel12.setText("   Line color:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel5.add(jLabel12, gridBagConstraints);

        jLabel13.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel13.setText("   Marker color:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel5.add(jLabel13, gridBagConstraints);

        jLabel14.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel14.setText("   Marker style:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel5.add(jLabel14, gridBagConstraints);

        jLabel15.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel15.setText("   Legend text:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanel5.add(jLabel15, gridBagConstraints);

        lineColor.setRenderer(new ComboBoxRenderer());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
        jPanel5.add(lineColor, gridBagConstraints);

        markerColor.setRenderer(new ComboBoxRenderer());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
        jPanel5.add(markerColor, gridBagConstraints);

        markerStyle.setRenderer(new MarkerRenderer());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
        jPanel5.add(markerStyle, gridBagConstraints);

        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), "Marker scale"));

        markerScale.setMaximum(8);
        markerScale.setMinimum(2);
        markerScale.setMinorTickSpacing(1);
        markerScale.setPaintTicks(true);
        markerScale.setSnapToTicks(true);
        markerScale.setValue(2);
        markerScale.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                markerScaleStateChanged(evt);
            }
        });
        jPanel7.add(markerScale);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 5, 10);
        jPanel5.add(jPanel7, gridBagConstraints);

        legendText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                legendTextActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
        jPanel5.add(legendText, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        jPanel8.add(jPanel5, gridBagConstraints);

        jPanel6.setLayout(new java.awt.BorderLayout());

        jLabel10.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel10.setText("Variable:  ");
        jPanel6.add(jLabel10, java.awt.BorderLayout.WEST);

        variable.setToolTipText("Changing this box involves save the changes in this variable");
        variable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                variableActionPerformed(evt);
            }
        });
        jPanel6.add(variable, java.awt.BorderLayout.EAST);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(10, 15, 0, 25);
        jPanel8.add(jPanel6, gridBagConstraints);

        jTabbedPane1.addTab("Variables options", null, jPanel8, "");

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        getContentPane().add(jTabbedPane1, gridBagConstraints);

        CancelButton.setText("Cancel");
        CancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                CancelButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 0);
        getContentPane().add(CancelButton, gridBagConstraints);

        OKButton.setText("OK");
        OKButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                OKButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 10, 10);
        getContentPane().add(OKButton, gridBagConstraints);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /** Saves the properties of the variable and loads the properties of the new current variable.
     * This method is called when the variable combobox is changed.
     */
    private void variableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_variableActionPerformed
        // Saving the changes
        DataSet ds=((VariableDataSet)gp.getDataArray().elementAt(currentVariable)).getDataSet();
        ds.linecolor=MyColors.decodeColor(lineColor.getSelectedItem().toString());
        ds.markercolor=MyColors.decodeColor(markerColor.getSelectedItem().toString());
        ds.marker=markerStyle.getSelectedIndex()+1;
        ds.markerscale=(double)markerScale.getValue()/2;
        if(!legendText.getText().equalsIgnoreCase(""))            
            ds.legend(20,currentVariable*12+30,legendText.getText()); 
        else
            ds.legend_text=null;        
        
        // Loading the parameters
        currentVariable=variable.getSelectedIndex();        
        ds=((VariableDataSet)gp.getDataArray().elementAt(currentVariable)).getDataSet();
        lineColor.setSelectedIndex(MyColors.getIndexColor(ds.linecolor));
        markerColor.setSelectedIndex(MyColors.getIndexColor(ds.markercolor));
        markerStyle.setSelectedIndex(ds.marker-1);
        markerScale.setValue((int)(ds.markerscale*2));
        if(ds.legend_text!=null)
            legendText.setText(ds.legend_text.getText());
    }//GEN-LAST:event_variableActionPerformed

    /** Transfers the focus of the legendText textField.
     * This method is called when the legend is been changed.
     */
    private void legendTextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_legendTextActionPerformed
        legendText.transferFocus();
    }//GEN-LAST:event_legendTextActionPerformed

    /** Transfers the focus of the axisTitle textField.
     * This method is called when the axis title is been changed.
     */
    private void axisTitleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_axisTitleActionPerformed
        axisTitle.transferFocus();
    }//GEN-LAST:event_axisTitleActionPerformed

    /** Checks that the number introduced in the maxY textField is correct.
     * If it's correct, transfers the focus.
     * This method is called when maxYRange textfield has lost the focus.
     */
    private void maxYRangeFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_maxYRangeFocusLost
        if(!checkDouble(maxYRange.getText()))
            maxYRange.grabFocus();
    }//GEN-LAST:event_maxYRangeFocusLost

    /** Transfers the focus of the maxYRange textField.
     * This method is called when the maxYRange is been changed.
     */
    private void maxYRangeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_maxYRangeActionPerformed
        maxYRange.transferFocus();
    }//GEN-LAST:event_maxYRangeActionPerformed

    /** Checks that the number introduced in the minYRange textField is correct.
     * If it's correct, transfers the focus.
     * This method is called when minYRange textfield has lost the focus.
     */
    private void minYRangeFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_minYRangeFocusLost
       if(!checkDouble(minYRange.getText()))            
            minYRange.grabFocus();
    }//GEN-LAST:event_minYRangeFocusLost

    /** Transfers the focus of the minYRange textField.
     * This method is called when the minYRange is been changed.
     */
    private void minYRangeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_minYRangeActionPerformed
        minYRange.transferFocus();
    }//GEN-LAST:event_minYRangeActionPerformed

    /** Checks that the number introduced in the maxXRange textField is correct.
     * If it's correct, transfers the focus.
     * This method is called when maxXRange textfield has lost the focus.
     */
    private void maxXRangeFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_maxXRangeFocusLost
        if(!checkDouble(maxXRange.getText()))
            maxXRange.grabFocus();
    }//GEN-LAST:event_maxXRangeFocusLost

    /** Transfers the focus of the maxXRange textField.
     * This method is called when the maxXRange is been changed.
     */
    private void maxXRangeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_maxXRangeActionPerformed
        maxXRange.transferFocus();
    }//GEN-LAST:event_maxXRangeActionPerformed

    /** Checks that the number introduced in the minXRange textField is correct.
     * If it's correct, transfers the focus.
     * This method is called when minXRange textfield has lost the focus.
     */
    private void minXRangeFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_minXRangeFocusLost
        if(!checkDouble(minXRange.getText()))
            minXRange.grabFocus();
    }//GEN-LAST:event_minXRangeFocusLost

    /** Transfers the focus of the minXRange textField.
     * This method is called when the minXRange is been changed.
     */
    private void minXRangeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_minXRangeActionPerformed
        minXRange.transferFocus();
    }//GEN-LAST:event_minXRangeActionPerformed

    /** Checks that the number introduced in the visiblePoints textField is correct.
     * If it's correct, transfers the focus.
     * This method is called when visiblePoints textfield has lost the focus.
     */
    private void visiblePointsFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_visiblePointsFocusLost
        if(!checkInteger(visiblePoints.getText()))
            visiblePoints.grabFocus();
    }//GEN-LAST:event_visiblePointsFocusLost

    /** Transfers the focus of the visiblePoints textField.
     * This method is called when the visiblePoints is been changed.
     */
    private void visiblePointsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_visiblePointsActionPerformed
        visiblePoints.transferFocus();
    }//GEN-LAST:event_visiblePointsActionPerformed

    /** Saves all the properties to the graphic panel associated. If the time unit, or
     * the axis type, or the axis scale has changed, the data of the graphic is reloaded
     * and processed according to this changes.
     * This method is called when OK button is pressed.
     * @see GraphicPanel#reloadAll
     */
    private void OKButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_OKButtonActionPerformed
        //saves the graph options
        Graph2D g=gp.getGraph();
        g.gridcolor=MyColors.decodeColor(gridColor.getSelectedItem().toString());
        g.zerocolor=MyColors.decodeColor(zeroColor.getSelectedItem().toString());
        g.setDataBackground(MyColors.decodeColor(blackGroundColor.getSelectedItem().toString()));
        gp.setBackground(MyColors.decodeColor(blackGroundColor.getSelectedItem().toString()));
        gp.setMaxVisiblePoints(Integer.parseInt(visiblePoints.getText()));
        gp.setOnlyVisiblePoints(visiblePointsCheckBox.isSelected());
        g.drawgrid=gridCheckBox.isSelected();
        
        //saves the axis options
        Axis a=gp.getXAxisBottom();
        gp.xmin=Double.parseDouble(minXRange.getText());
        gp.xmax=Double.parseDouble(maxXRange.getText());
        a.setManualRange(!XAutoCheckBox.isSelected());
        if(!a.isManualRange())
            a.resetRange();
        
        a=gp.getYAxisRight();
        gp.ymin=Double.parseDouble(minYRange.getText());
        gp.ymax=Double.parseDouble(maxYRange.getText());
        a.setManualRange(!YAutoCheckBox.isSelected()); 
        gp.YTitle=axisTitle.getText();
               
        //saves main axis options and checks if the graphic has to be reloaded
        boolean reload=false;
        if(initialUnit!=axisTimeUnit.getSelectedIndex()){
            if(axisTimeUnit.getSelectedIndex()==0){
                gp.unitDivision=1;
                gp.getXAxisBottom().setTitleText("Time-Slot");
            }
            if(axisTimeUnit.getSelectedIndex()==1){
                gp.unitDivision=1000d;
                gp.getXAxisBottom().setTitleText("1000 x Time-Slot");
            }
            reload=true;
        }
                
        if(gp.type!=axisType.getSelectedIndex()){
            gp.type=axisType.getSelectedIndex();            
            reload=true;
            a.setManualRange(false);            
        }
        
        if(gp.scale!=axisScale.getSelectedIndex()){
            gp.scale=axisScale.getSelectedIndex();
            reload=true;
            a.setManualRange(false);
        }
        
        if(!a.isManualRange())
            a.resetRange();
        
        //reloads the graphic if necessary
        if(reload)
            gp.reloadAll();
        
        setYTitle();
        
        // Saving the changes
        DataSet ds=((VariableDataSet)gp.getDataArray().elementAt(currentVariable)).getDataSet();
        ds.linecolor=MyColors.decodeColor(lineColor.getSelectedItem().toString());
        ds.markercolor=MyColors.decodeColor(markerColor.getSelectedItem().toString());
        ds.marker=markerStyle.getSelectedIndex()+1;
        ds.markerscale=(double)markerScale.getValue()/2;
        if(!legendText.getText().equalsIgnoreCase(""))            
            ds.legend(20,currentVariable*12+30,legendText.getText()); 
        else
            ds.legend_text=null; 
        
        //redraws the graphic
        gp.drawGraphic();
        setVisible(false);
        dispose();
    }//GEN-LAST:event_OKButtonActionPerformed

    /** Cancels the changes.
     * This method is called when cancel button is pressed.
     */
    private void CancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_CancelButtonActionPerformed
        setVisible(false);
        dispose();
    }//GEN-LAST:event_CancelButtonActionPerformed

    /** Repaints the markerViewer.
     * This method is called when makerScale slider has benn changed.
     */
    private void markerScaleStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_markerScaleStateChanged
        if(mw!=null)
            mw.repaint();
    }//GEN-LAST:event_markerScaleStateChanged

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();        
    }//GEN-LAST:event_closeDialog

    private void axisTimeUnitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_axisTimeUnitActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_axisTimeUnitActionPerformed

    /** Reads the marker file. This file is placed in the 'Graphics' directory and
     * named 'marker.txt'.
     * @see Markers
     */ 
    private void setDefaults(){
        markerFile = PhalguiApp.class.getResource("resources/marker.txt").getPath().replace("%20"," ");
        try{
            marks= new Markers((new File(markerFile)));
        }
        catch(java.io.IOException exc){
            System.out.println("Error reading markers: "+exc.toString());
        }
    }
    
    /** Sets the YTitle of the graphic panel depending on its type or scale. */
    private void setYTitle(){
        String title=gp.YTitle;
        if(gp.type==gp.DIFF_TYPE)
            title+=" (DIFF)";
        if(gp.type==gp.INT_TYPE)
            title+=" (INT)";
        if(gp.scale==gp.LOG_SCALE)
            title+=" (LOG)";
        
        if(!title.equalsIgnoreCase(""))
            gp.getYAxisRight().setTitleText(title);        
        else gp.getYAxisRight().setTitleText(" ");        
    }
    
    /** Gets the initial properties of the graphic panel. 
     * @see GraphicPanel
     */    
    private void postInitComponents(){
        //gets the graph options
        Graph2D g=gp.getGraph();
        gridColor.setSelectedIndex(MyColors.getIndexColor(g.gridcolor));
        zeroColor.setSelectedIndex(MyColors.getIndexColor(g.zerocolor));
        blackGroundColor.setSelectedIndex(MyColors.getIndexColor(g.getBackground()));
        visiblePoints.setText(Integer.toString(gp.getMaxVisiblePoints()));
        visiblePointsCheckBox.setSelected(gp.isOnlyVisiblePoints());
        gridCheckBox.setSelected(g.drawgrid);
        
        //gets the axis options
        Axis a=gp.getXAxisBottom();
        minXRange.setText(Double.toString(Math.round(a.minimum*100)/100));
        maxXRange.setText(Double.toString(Math.round(a.maximum*100)/100));
        XAutoCheckBox.setSelected(!a.isManualRange());
        
        a=gp.getYAxisRight();
        minYRange.setText(Double.toString(Math.round(a.minimum*100)/100));
        maxYRange.setText(Double.toString(Math.round(a.maximum*100)/100));
        YAutoCheckBox.setSelected(!a.isManualRange());        
        axisTitle.setText(gp.YTitle);
        
        if(gp.unitDivision==1)
            axisTimeUnit.setSelectedIndex(0);
        if(gp.unitDivision==1000)
            axisTimeUnit.setSelectedIndex(1); 
        initialUnit=axisTimeUnit.getSelectedIndex();
        
        axisType.setSelectedIndex(gp.type);        
        axisScale.setSelectedIndex(gp.scale);
        
        //gets the options of the variables
        DataSet ds=((VariableDataSet)gp.getDataArray().elementAt(0)).getDataSet();
        currentVariable=variable.getSelectedIndex();
        lineColor.setSelectedIndex(MyColors.getIndexColor(ds.linecolor));
        markerColor.setSelectedIndex(MyColors.getIndexColor(ds.markercolor));
        if (ds.marker-1<markerStyle.getItemCount()) {
            markerStyle.setSelectedIndex(ds.marker-1);
        }
        markerScale.setValue((int)(ds.markerscale*2));
        if(ds.legend_text!=null)
            legendText.setText(ds.legend_text.getText());
    }
        
    /** Returns a Vector of Strings with the names of the markers. This names
     * are defined in the merkers file.
     * This function is used to initiate the markerStyle combobox.
     */
    private Vector getMarkers(){
        Vector markers= new Vector(14);
        
        try{
            BufferedReader in = new BufferedReader(new FileReader(markerFile));
            // prepares inputstream for extracting lines
        
            String nextLine = new String();   // generates string object
            StringParser sp = new StringParser();
            
            while((nextLine = in.readLine()) != null){
                if(nextLine.startsWith("#Marker")) {     
                    sp.setString(nextLine); 
                    sp.getNextToken(':');       // #Marker N:
                    markers.add(sp.getNextToken().trim());
                }
            }
        }
        catch(java.io.FileNotFoundException exc){
            Trace.print("Error reading markers: "+exc.toString(),Trace.ERRORS,"GraphicOptions",1001);
        }
        catch(java.io.IOException exc){
            Trace.print("Error reading markers: "+exc.toString(),Trace.ERRORS,"GraphicOptions",1004);
        }
        
        return markers;
    } 
        
    /** Checks that the number is a double. If not, an error message is shown.
     * @param num String with the number to check
     * @return True is the value is a double, false if not
     */
    private boolean checkDouble(String num){
        try{
            Double.parseDouble(num);
        }
        catch(NumberFormatException exc){
            focus=!focus;
            if(focus) return false;
            /*
            ArrowsError ae= new ArrowsError(this.parent);
            ae.showErrorDialog(ArrowsError.MESSAGE,
            new String[]{"Invalid number format",
            "You must put a double in the text field"},
            new String[]{"OK"});   
            return false;
             */
        }
        return true;        
    }
    
    /** Checks that the number is an integer. If not, an error message is shown.
     * @param num String with the number to check
     * @return True is the value is an integer, false if not
     */
    private boolean checkInteger(String num){
        try{
            int i=Integer.parseInt(num);
            if(i<2){
                focus=!focus;
                if(focus) return false;
                /*
                ArrowsError ae= new ArrowsError(this.parent);
                ae.showErrorDialog(ArrowsError.MESSAGE,
                new String[]{"Invalid range",
                "You must put an integer upper to 1 in the text field"},
                new String[]{"OK"});
                 */
                return false;
            }
                
        }
        catch(NumberFormatException exc){
            focus=!focus;
            if(focus) return false;
            /*
            ArrowsError ae= new ArrowsError(this.parent);
            ae.showErrorDialog(ArrowsError.MESSAGE,
            new String[]{"Invalid number format",
            "You must put an integer in the text field"},
            new String[]{"OK"});
             */
            return false;
        }
        return true;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton CancelButton;
    private javax.swing.JButton OKButton;
    private javax.swing.JCheckBox XAutoCheckBox;
    private javax.swing.JCheckBox YAutoCheckBox;
    private javax.swing.JComboBox axisScale;
    private javax.swing.JComboBox axisTimeUnit;
    private javax.swing.JTextField axisTitle;
    private javax.swing.JComboBox axisType;
    private javax.swing.JComboBox blackGroundColor;
    private javax.swing.JCheckBox gridCheckBox;
    private javax.swing.JComboBox gridColor;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextField legendText;
    private javax.swing.JComboBox lineColor;
    private javax.swing.JComboBox markerColor;
    private javax.swing.JSlider markerScale;
    private javax.swing.JComboBox markerStyle;
    private javax.swing.JTextField maxXRange;
    private javax.swing.JTextField maxYRange;
    private javax.swing.JTextField minXRange;
    private javax.swing.JTextField minYRange;
    private javax.swing.JComboBox variable;
    private javax.swing.JTextField visiblePoints;
    private javax.swing.JCheckBox visiblePointsCheckBox;
    private javax.swing.JComboBox zeroColor;
    // End of variables declaration//GEN-END:variables

    /** This private class is a renders the color comboboxes to draw a cloured square at
     * each label of the combobox.
     */
    private class ComboBoxRenderer extends JLabel
                       implements ListCellRenderer {
        public ComboBoxRenderer() {
            setOpaque(true);
            setHorizontalAlignment(LEADING);
            setVerticalAlignment(CENTER);
        }
        
        public Component getListCellRendererComponent(
            JList list,
            Object value,
            int index,
            boolean isSelected,
            boolean cellHasFocus)
        {
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }

            java.awt.image.BufferedImage b=new java.awt.image.BufferedImage(12,12,
            java.awt.image.BufferedImage.TYPE_INT_RGB);
            java.awt.Graphics g=b.getGraphics();
            
            g.setColor(MyColors.decodeColor(value.toString()));            
            g.fill3DRect(0,0,12,12,true);
            setText(value.toString());      
            setIcon(new javax.swing.ImageIcon(b));        
            return this;
        }
    }   

   /** This private class renders the marker style combobox drawing the marker in
    * the left side of each label in the combobox.
    * @see Markers#draw
    */ 
    private class MarkerRenderer extends JLabel
                       implements ListCellRenderer {
        public MarkerRenderer() {
            setOpaque(true);
            setHorizontalAlignment(LEADING);
            setVerticalAlignment(CENTER);
        }
    
        public Component getListCellRendererComponent(
            JList list,
            Object value,
            int index,
            boolean isSelected,
            boolean cellHasFocus)
        {
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }

            java.awt.image.BufferedImage b=new java.awt.image.BufferedImage(15,15,
            java.awt.image.BufferedImage.TYPE_INT_RGB);
            java.awt.Graphics g=b.getGraphics();
            
            g.setColor(MyColors.decodeColor("default"));            
            g.fill3DRect(0,0,15,15,true);
            g.setColor(Color.black);
            
            //draws the markers in the buffered image
            marks.draw(g,index+1,2.0,7,7);
            setText(value.toString());      
            setIcon(new javax.swing.ImageIcon(b));        
            return this;
        }
    }   

    /** This private class is a viewer of the selected marker. Depending on the marker
     * scale and the type, one or other marker is drawn.
     * @see Markers#draw
     */
    private class MarkerViewer extends JComponent {
        
        public MarkerViewer(){
        setPreferredSize(new Dimension(40,40));
        setBorder(new javax.swing.border.EtchedBorder());
        }
        
        public void paint(Graphics g){
            super.paint(g);        
            g.setColor(MyColors.decodeColor(blackGroundColor.getSelectedItem().toString()));
            g.fillRect(4,4,31,31);
            g.setColor(MyColors.decodeColor(markerColor.getSelectedItem().toString()));
            marks.draw(g,markerStyle.getSelectedIndex()+1,(double)markerScale.getValue()/2d,19,19);        
        }
    
    }
}
