/*
 * GraphicPanel.java
 *
 * Created on 11 de enero de 2002, 18:08
 */

package phalgui.Node.Stats;


import java.awt.dnd.DropTargetDropEvent;
import phalgui.graph.*;
import java.awt.*;
import java.io.File;
import java.util.*;
import phalgui.PhalguiApp;
import phalgui.PhalguiView;
import phalgui.Utils.Arrows.MyBoolean;
import phalgui.Utils.Arrows.StringParser;
import phalgui.Utils.Arrows.Trace;
import phalgui.Utils.DnD.DropablePanel;
import phalgui.Utils.DnD.TransfResizePanel;

//import Utilities.*;

/**
 * This class is a panel which supports the methods to draw the graphic using a Graph2D object. 
 * So it's the union between the grahicFrame or StatisticsGraphic and the Graph2D. It's properties
 * can be configured in the GraphicOptions. The data of each variable is placed in the VariableDataSet.  
 * @author  Santi Oliveras
 * @version 1.0  2/05/2002
 * @see GraphicOptions
 * @see VariableDataSet
 * @see Graph2D
 * @see Axis
 */
public class GraphicPanel extends DropablePanel {
 
    // ******** private variables **********************************************
    
    /** The Graph2D object. */
    private Graph2D graph = null;

    /** The VariableDataSet array. */
    private Vector dataArray = new Vector(20);

    /** The axis at the right side. */
    private Axis yaxis_right = null;

    /** The axis at the bottom. */
    private Axis xaxis_bottom = null;

    /** The offscreen image. */
    private Image osi = null;

    /** The number of visible points (def.: 20). */
    private int maxVisiblePoints = 20;

    /** The graphic type :
     *  if false, we view all the points in the visible range.
     *  if true, we only view the last maxVisiblePoints .
     */
    private boolean onlyVisiblePoints = false;

    /** The width of the offscreen image. */
    private int iwidth = 0;

    /** The height of the offscreen image. */
    private int iheight = 0;
     
    /** The marker file (def.: "marker.txt"). */
    //private String markerfile = PhalguiView.class.getResource("resources/marker.txt").getPath().replace("%20"," ");

    private String markerfile = PhalguiView.class.getResource("resources/marker.txt").getPath().replace("%20"," ");

    /** The current data point. */
    private double point[] = new double[2];
   
    /** The graphics context of the graph.  */
    private Graphics g = null;

    /** The graphics context of the offscreen. */
    private Graphics osg = null;
  
    /** The popup menu. */
    private PopupMenu popup;
    
    /** The options item of the popup menu. */
    private MenuItem options;
    
    /** The options item of the popup menu. */
    private MenuItem autoRange;
    
    /** The addGraph menu of the popup menu. */
    private Menu addGraph;
    
    /** The deleteGraph menu of the popup menu. */
    private Menu deleteGraph;    
    
    /** The GraphicInterface to union the panel with the GraphicFrame
     * or the StatisticsGraphic. 
     */
    private GraphicInterface gi;
    
    /** String with the nearest point coordinates. */
    private String nearestPoint=null;
    
    /** Rectangle used to mark the selected area to zoom. */
    private Rectangle selectedRect=null;
  
    // ******** public variables ***********************************************
    
    /** Integer with the normal type of the X axis. */
    public static final int NORMAL_TYPE=0;
    
    /** Integer with the difference type of the X axis. */
    public static final int DIFF_TYPE=1;
    
    /** Integer with the integration type of the X axis. */
    public static final int INT_TYPE=2;
    
    /** Integer with the lineal scale of the X axis. */
    public static final int LINEAL_SCALE=0;
    
    /** Integer with the logarithmic scale of the X axis. */
    public static final int LOG_SCALE=1;
    
    /** The division we have to do to convert the miliseconds to seconds or to minutes.
     * i.e: In case of seconds is 1000; in case of minutes is 60000.
     */    
    public double unitDivision=1000d;
    
    /** The limits of the axis in case of manual range. */
    public double xmin,xmax,ymin,ymax;
    
    /** The scale of the graphic (lineal or logarithmic). */
    public int scale=0;
    
    /** The type of the graphic (normal, difference or integration). */
    public int type=0;
    
    /** The name of the Title of the Y axis .*/
    public String YTitle="";
    
    /** Boolean which shows if the load has failed when we read from a file. */
    public boolean loadFailed;
    
    
    // ******** constructors ***************************************************
    
    /** Creates new form GraphicPanel used by the loaded Graphics. 
     * @param gi The graphic interface 
     * @see GraphicFrame
     */
    public GraphicPanel(GraphicInterface gi) {
        super(TransfResizePanel.OBJECT);
        this.gi=gi;
        initComponents();
        setBackground(MyColors.decodeColor("lightYellow"));
    }
    
    /** Creates new form GraphicPanel from a file. Is called when a MovingStatisticsGraphic 
     * is loaded from the panels agrupation file.
     * @param gi The graphic interface 
     * @param br Reader with the pointer of the file
     * @see MovingStatisticsGraphic#MovingStatisticsGraphic(java.awt.Frame,BufferedReader,GraphicAgrupation)
     */
    public GraphicPanel(GraphicInterface gi, java.io.BufferedReader br) throws java.io.FileNotFoundException,
                                                          java.io.IOException {
        super(TransfResizePanel.OBJECT);
        this.gi=gi;
        initComponents();      
        loadFailed=!load(br);
        popupConfiguration(); 
        upDateTitles();
        addItems();
        setBackground(MyColors.decodeColor("lightYellow"));
    }

    
    /** Creates new form GraphicPanel called when a variable is selected. This variable can
     * be selected doubleclicking a variable from the DinamicTreeManager in case of a GraphicFrame 
     * as a GraphicInterface (Dinamic Viewer), or pressing the OK button in the StatisticsDialog in case of a 
     * StatisticsGraphic as a GraphicInterface (real time Statistics).
     * @param gi The graphic interface 
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see GraphicFrame
     * @see StatisticsGraphic
     */ 
     public GraphicPanel(GraphicInterface gi, String module, String variable) {
        super(TransfResizePanel.OBJECT);
        this.gi=gi;
        initComponents();
        setDefaults(module, variable);
        popupConfiguration();
        addItems();
          setBackground(MyColors.decodeColor("lightYellow"));
  }
    
    /** Creates new form GraphicPanel from another GraphicPanel. This method is
     * used when a StatisticsGraphic is added to a GraphicAgrupation in the panels frame;
     * so a copy of a StatisticGraphic is placed in this panel creating a new 
     * MovingStatisticsGraphic (which extends a StatisticsGraphic); from this moment,
     * the StatisticsGraphic and its copy (the MovingStatisticsGraphic) are independent.
     * @param gi The graphic interface (StatisticsGraphic) 
     * @param gp The GraphicPanel to be copied
     * @see MovingStatisticsGraphic#MovingStatisticsGraphic(java.awt.Frame,StatisticsGraphic,GraphicAgrupation)
      */
    public GraphicPanel(GraphicInterface gi,GraphicPanel gp) {
        super(TransfResizePanel.OBJECT);
        this.gi=gi;
        initComponents();
        Vector da=gp.getDataArray();        
        StringParser sp= new StringParser();        
        for(int i=0;i<da.size();i++){
            sp.setString(da.elementAt(i).toString());
            String module=sp.getNextToken('-').trim();
            if(i==0){
                setDefaults(module,sp.getNextToken().trim());
                popupConfiguration();
            }
            else addNewData(module,sp.getNextToken().trim());
            DataSet data=((VariableDataSet)da.elementAt(i)).getDataSet();               
            DataSet ds=((VariableDataSet)dataArray.lastElement()).getDataSet();            
            ds.linecolor=data.linecolor;
            ds.markercolor=data.markercolor;
            ds.marker=data.marker;
            ds.markerscale=data.markerscale; 
            if(data.legend_text!=null)
                ds.legend(20,(dataArray.size()-1)*12+30,data.legend_text.getText());                                    
            else
                ds.legend_text=null;                
            }           
            Graph2D gr=gp.getGraph();
            Graph2D g=getGraph();            
            g.gridcolor=gr.gridcolor;
            g.zerocolor=gr.zerocolor;
            setBackground(gp.getBackground());
            g.setDataBackground(getBackground());       
            setMaxVisiblePoints(gp.getMaxVisiblePoints());
            setOnlyVisiblePoints(gp.isOnlyVisiblePoints());
            g.drawgrid=gr.drawgrid;            
            
            Axis a=getXAxisBottom();
            xmin=gp.xmin;
            xmax=gp.xmax;
            a.setManualRange(gp.getXAxisBottom().isManualRange());
            a.setTitleText(gp.getXAxisBottom().getTitleText());
        
            a=getYAxisRight();
            ymin=gp.ymin;
            ymax=gp.ymax;
            a.setManualRange(gp.getYAxisRight().isManualRange());             
            YTitle=gp.YTitle;
            
            type=gp.type;
            scale=gp.scale;
            unitDivision=gp.unitDivision;
            upDateTitles();
            addItems();
    }
    
    
    // ******** public methods *************************************************
    
    /** Redraws the graphic if the panel is visible and is showing. It also sets
     * the manual axis limits in case of manual range and controls the automatic limit
     * in case of onlyVisiblePoints is true.
     * @see Graph2D#update
     */
    public void drawGraphic(){
        if(!isShowing() || !isVisible())
            return;
        
        if(!xaxis_bottom.isManualRange()){                        
            if(onlyVisiblePoints){
                String variable=gi.getSelectedVariable();                
                StringParser sp= new StringParser(variable,false);                
                String module=sp.getNextToken('-').trim();
                variable=sp.getNextToken().trim();                
                for(int i=0;i<dataArray.size();i++)
                    if(((VariableDataSet)dataArray.elementAt(i)).isVariable(module,variable)){
                        DataSet data=((VariableDataSet)dataArray.elementAt(i)).getDataSet();
                        if(data.dataPoints()>maxVisiblePoints)
                            xaxis_bottom.minimum = data.getPoint(data.dataPoints()-maxVisiblePoints)[0];                        
                    }            
            }                     
        }
        else{
            xaxis_bottom.maximum=xmax;
            xaxis_bottom.minimum=xmin;
        }
        
        if(yaxis_right.isManualRange()){
            yaxis_right.maximum=ymax;
            yaxis_right.minimum=ymin;
        }
            
	// gets the graphics context for the graph
	g = graph.getGraphics();

	// sets the size of and the offscreen image of the graph at all
	if (osi == null || iwidth != graph.getSize().width || iheight != graph.getSize().height)
	{
	    iwidth = graph.getSize().width;
	    iheight = graph.getSize().height;
            
            if(iwidth<=0 || iheight<=0 )
                return;

	    // creates the offscreen image with a specified size
	    osi = graph.createImage(iwidth, iheight);
	    // gets the offscreen graphics context
	    osg = osi.getGraphics();
	}

	// clears the background
	osg.setColor(this.getBackground());
	osg.fillRect(0,0,iwidth,iheight);

	// gets the current color of the graph's graphics context
	osg.setColor(g.getColor());
	// sets the clipping area to the specified size
	osg.clipRect(0,0,iwidth,iheight);

	// this method is called via the Graph2D.repaint() method; all it does is blank
	// the canvas (with the background color) before calling paint
	graph.update(osg);

        if(nearestPoint!=null){
            osg.drawString(nearestPoint,10,iheight-8);
            nearestPoint=null;
        }
        
	// displays the offscreen
	g.drawImage(osi,0,0,graph);    
    }
    
    /** Returns a true if the graphic contains the variable.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     */
    public boolean hasVariable(String module, String variable){
        for(int i=0;i<dataArray.size();i++)
            if(((VariableDataSet)dataArray.elementAt(i)).isVariable(module,variable)){
                return true;
            }
        return false;                
    }
    
    /** Adds a new variable to the graphic.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see Axis#attachDataSet
     * @see Graph2D#attachDataSet
     */
    public void addNewData(String module,String variable){
        
        if(deleteGraph!=null)
            deleteGraph.setEnabled(true);
        VariableDataSet vds= new VariableDataSet(module, variable,MyColors.decodeColor(dataArray.size()-1));        
        dataArray.add(vds);  // Adding data to the array
        DataSet data= vds.getDataSet(); 
        data.legend(20,(dataArray.size()-1)*20+30,module + "_" + variable);
        
	// attaches a DataSet for the y-axis to manage
	yaxis_right.attachDataSet(data);
	
	// attaches a DataSet for the x-axis to manage
	xaxis_bottom.attachDataSet(data);	
	        
	// attaches a DataSet to the graph
	graph.attachDataSet(data);               
        
        if(addGraph!=null)
            drawGraphic();
    }
    
    /** Deletes a variable from the graphic.
     * @param module String with the name of the module
     * @param variable String with the name of the variable to delete
     * @see Axis#detachDataSet
     * @see Graph2D#detachDataSet
     */
    public void deleteData(String module,String variable){
        
        for(int i=0;i<dataArray.size();i++)
            if(((VariableDataSet)dataArray.elementAt(i)).isVariable(module,variable)){
                DataSet data=((VariableDataSet)dataArray.elementAt(i)).getDataSet();
                
                // detaches the DataSet from the y-axis
                yaxis_right.detachDataSet(data);
	
                // detaches the DataSet from the x-axis
                xaxis_bottom.detachDataSet(data);	
	        
                // detaches the DataSet from the graph
                graph.detachDataSet(data);
                
                // remove VariableDataSet from the dataArray  
                dataArray.remove(i);
                upDateLegend();                
                drawGraphic();
                break;
            }    
        if(dataArray.size()==1)
            deleteGraph.setEnabled(false);
    }
            
    /** Resets the data of the specified variable if it contains the variable.
     * @param module String with the name of the module
     * @param variable String with the name of the variable to reset
     * @see VariableDataSet#resetAll(String,String)
     */
    public void resetAll(String module, String variable){
        for(int i=0;i<dataArray.size();i++)
            ((VariableDataSet)dataArray.elementAt(i)).resetAll(module,variable);
        drawGraphic();
    }
    
    /** Resets the data of all the variables.
     * @see DataSet#deleteData
     */
    public void resetAll(){
        for(int i=0;i<dataArray.size();i++)
            ((VariableDataSet)dataArray.elementAt(i)).getDataSet().deleteData();
        drawGraphic();
    }
    
    /** Returns the sum of number of points of all the variables.
     */ 
    public int elementsCount(){
        int num=0;
        for(int i=0;i<dataArray.size();i++)
            num +=((VariableDataSet)dataArray.elementAt(i)).getDataSet().dataPoints();
        return num;
    }
    
    /** Deletes the last value of the variable specified if the graphic contains it.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see VariableDataSet#deleteLast
     */
    public void deleteLastValue(String module, String variable){
        for(int i=0;i<dataArray.size();i++)
            ((VariableDataSet)dataArray.elementAt(i)).deleteLast(module,variable);        
        drawGraphic();
    }
    
    /** Updates the legend. This method is used when a new variable is added or 
     * deleted.
     * @see DataSet#legend
     */
    public void upDateLegend(){
        for(int i=0;i<dataArray.size();i++){
            DataSet ds=((VariableDataSet)dataArray.elementAt(i)).getDataSet();
            if(ds.legend_text!=null)
                ds.legend(20,i*12+30,ds.legend_text.getText());
        }
    }

    public void nextValue(double milisec, double value, String module, String variable) {
        this.nextValue(milisec,value,module,variable,true);
    }
    /** Adds a new point to the variable specified if the graphic contains it.
     * @param milisec Integer with the current milisecond
     * @param value Double with the value of the new point
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see VariableDataSet#append
     */
    public void nextValue(double milisec, double value, String module, String variable, boolean draw) {
               
        // sets point corresponding to the current data point
	point[1] = value;
	point[0] = (double)milisec;
        
        for(int i=0;i<dataArray.size();i++)
            ((VariableDataSet)dataArray.elementAt(i)).append(point,1,module,variable);

    if (draw)
    	drawGraphic();
    }
    
    /** Updates the graphic with the new data points to the specified variable if the
     * graphic contains it.
     * @param points Array of doubles containing the points (x0,y0,x1,y1...,xn,yn)
     * @param module String with the name of the module
     * @param variable String with the name of the variable to upDate
     * @see VariableDataSet#upDateAllData
     */
    public void upDateGraphic(double[] points,String module,String variable){
        
        for(int i=0;i<dataArray.size();i++)
            ((VariableDataSet)dataArray.elementAt(i)).upDateAllData(points,
            (int)(points.length/2),module,variable); 
        
        // sets point corresponding to the current data point
	point[1] = points[points.length-1];
	point[0] = points[points.length-2];
                
        drawGraphic();
    }
    
    /** Returns the Graph2D object.
     * @see Graph2D
     */
    public Graph2D getGraph(){
        return graph;
    }
    
    /** Returns the X axis.
     * @see Axis
     */
    public Axis getXAxisBottom(){
        return xaxis_bottom;
    }
    
    /** Returns the Y axis.
     * @see Axis
     */
    public Axis getYAxisRight(){
        return yaxis_right;
    }
    
    /** Returns a vector containig the VariableDataSets.
     * @see VariableDataSet
     */
    public Vector getDataArray(){
        return dataArray;
    }
    
    /** Returns true if only maxVisiblePoints are visible.
     */
    public boolean isOnlyVisiblePoints(){
        return onlyVisiblePoints;
    }
    
    /** Sets if only maxVisiblePoints are visible.
     * @param b Boolean to set the onlyVisiblePoints varialbe
     */
    public void setOnlyVisiblePoints(boolean b){
        onlyVisiblePoints=b;
    }
    
    /** Returns an integer with the maximum number of visible points.
     */
    public int getMaxVisiblePoints(){
        return maxVisiblePoints;
    }
    
    /** Sets the maximum number of visible points.
     * @param max Integer with the maximum number of visible points.
     */
    public void setMaxVisiblePoints(int max){
        maxVisiblePoints=max;
    }
    
    /** Disables the popup of the graphic. */
    public void disablePopup(){
        popup.setEnabled(false);
    }
    
    /** Reloads all the data.
     * @see GraphicFrame#reloadAll
     * @see StatisticsGraphic#reloadAll
     */
    public void reloadAll(){
         gi.reloadAll();
    }
    
    /** Loads all the graphic properties from a file. Is used only in the loaded graphs.
     * @see #toSave
     */
    public boolean load(java.io.BufferedReader br) throws java.io.FileNotFoundException,
                                                          java.io.IOException{
        DataSet ds;
        StringParser sp=new StringParser();
        String s;
        int num;
        
        try{   
            //reads the data properties
            num=Integer.parseInt(br.readLine());
            for(int i=0;i<num;i++){
                sp.setString(br.readLine());
                String module=sp.getNextToken('-').trim();
                if(i==0){
                    setDefaults(module,sp.getNextToken().trim());
                    simplePopupConfiguration();
                }
                else addNewData(module,sp.getNextToken().trim());
                
                sp.setString(br.readLine());            
                ds=((VariableDataSet)dataArray.lastElement()).getDataSet();            
                ds.linecolor=MyColors.decodeColor(Integer.parseInt(sp.getNextToken(',')));
                ds.markercolor=MyColors.decodeColor(Integer.parseInt(sp.getNextToken(',')));
                ds.marker=Integer.parseInt(sp.getNextToken(','));
                ds.markerscale=Double.parseDouble(sp.getNextToken());
            
                s=br.readLine();
                if(!s.equalsIgnoreCase(""))
                    ds.legend(20,(dataArray.size()-1)*12+30,s);                    
                else
                    ds.legend_text=null;
                br.readLine();
            }
            
            //reads the general graphic properties
            sp.setString(br.readLine());                  
            Graph2D g=getGraph();            
            g.gridcolor=MyColors.decodeColor(Integer.parseInt(sp.getNextToken(',')));
            g.zerocolor=MyColors.decodeColor(Integer.parseInt(sp.getNextToken(',')));
            setBackground(MyColors.decodeColor(Integer.parseInt(sp.getNextToken(','))));
            g.setDataBackground(getBackground());       
            setMaxVisiblePoints(Integer.parseInt(sp.getNextToken(',')));
            setOnlyVisiblePoints(MyBoolean.getBoolean(sp.getNextToken(',')));
            g.drawgrid=MyBoolean.getBoolean(sp.getNextToken());
            
            //reads the axis properties
            sp.setString(br.readLine()); 
            Axis a=getXAxisBottom();
            xmin=Double.parseDouble(sp.getNextToken(','));
            xmax=Double.parseDouble(sp.getNextToken(','));
            a.setManualRange(MyBoolean.getBoolean(sp.getNextToken()));
        
            sp.setString(br.readLine());
            a=getYAxisRight();
            ymin=Double.parseDouble(sp.getNextToken(','));
            ymax=Double.parseDouble(sp.getNextToken(','));
            a.setManualRange(MyBoolean.getBoolean(sp.getNextToken())); 
            
            YTitle=br.readLine();
            
            sp.setString(br.readLine());
            type=Integer.parseInt(sp.getNextToken(','));
            scale=Integer.parseInt(sp.getNextToken(','));
            unitDivision=Double.parseDouble(sp.getNextToken());
            
            if(unitDivision==1)
                getXAxisBottom().setTitleText("Time-Slot");
            if(unitDivision==1000)
                getXAxisBottom().setTitleText("1000 x Time-Slot");
            br.readLine();        
        }
        catch(NullPointerException exc){
            Trace.print("Error reading Graph: "+exc.toString(),Trace.ERRORS,"GraphicPanel",613);
            return false;
        }
        catch(NumberFormatException exc){
            Trace.print("Error reading Graph: "+exc.toString(),Trace.ERRORS,"GraphicPanel",617);
            return false;
        }
        return true;
    }
    
    /** Returns a String with all the properties of the graphic. This String will be
     * written in a graphic file or in the panels agrupation file depending on the case.
     * @see #load
     */
    public String toSave(){
        String s="";
        //writes the data properties
        s+=dataArray.size()+"\n";
        for(int i=0;i<dataArray.size();i++){
            s+=dataArray.elementAt(i).toString()+"\n";
            DataSet ds=((VariableDataSet)dataArray.elementAt(i)).getDataSet();
            
            s+=MyColors.getIndexColor(ds.linecolor)+","+
            MyColors.getIndexColor(ds.markercolor)+","+
            ds.marker+","+ds.markerscale+"\n";
            s+=ds.legend_text.getText()+"\n";
            s+="\n";
        }
        
        //writes the graph properties
        Graph2D g=getGraph();
        s+=MyColors.getIndexColor(g.gridcolor)+","+
        MyColors.getIndexColor(g.zerocolor)+","+
        MyColors.getIndexColor(g.getBackground())+","+
        getMaxVisiblePoints()+","+
        isOnlyVisiblePoints()+","+
        g.drawgrid+"\n";
       
        //writes the axis properties 
        Axis a=getXAxisBottom();
        s+=a.minimum+","+a.maximum+","+a.isManualRange()+"\n";
       
        a=getYAxisRight(); 
        s+=a.minimum+","+a.maximum+","+a.isManualRange()+"\n"; 
        s+=YTitle+"\n";
        s+=type +","+ scale +","+ unitDivision+"\n";
        s+="\n";
                
        return s;
    }

    
    // ******** private methods ************************************************
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        
        setLayout(new java.awt.BorderLayout());
        
        setBorder(new javax.swing.border.EtchedBorder());
        setBackground(new java.awt.Color(160, 160, 255));
        setPreferredSize(new java.awt.Dimension(300, 200));
        setMinimumSize(new java.awt.Dimension(50, 50));
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });
        
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                formMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                formMouseReleased(evt);
            }
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
            public void mouseExited(java.awt.event.MouseEvent evt) {
                formMouseExited(evt);
            }
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                formMouseEntered(evt);
            }
        });
        
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                formMouseDragged(evt);
            }
        });
        
    }//GEN-END:initComponents

    private void formMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseReleased
        if(selectedRect.width<10 || selectedRect.height<10)
            return;
        xmin=xaxis_bottom.getDouble((int)selectedRect.getMinX()-getInsets().left);
        xmax=xaxis_bottom.getDouble((int)selectedRect.getMaxX()-getInsets().left);
        ymax=yaxis_right.getDouble((int)selectedRect.getMinY()-getInsets().top);
        ymin=yaxis_right.getDouble((int)selectedRect.getMaxY()-getInsets().top);
        xaxis_bottom.setManualRange(true);
        yaxis_right.setManualRange(true);
        drawGraphic();
    }//GEN-LAST:event_formMouseReleased

    private void formMouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseDragged
        if(selectedRect.x>evt.getX()||selectedRect.y>evt.getY())
                return;
        selectedRect.width=evt.getX()-selectedRect.x;
        selectedRect.height=evt.getY()-selectedRect.y;
        if(selectedRect.width<10 || selectedRect.height<10)
            return;
        
        g=graph.getGraphics();
        g.dispose();
        g=graph.getGraphics();
        if(osi!=null)
            g.drawImage(osi,0,0,graph);
        g.drawRect(selectedRect.x,selectedRect.y,selectedRect.width,selectedRect.height);
    }//GEN-LAST:event_formMouseDragged

    private void formMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMousePressed
        selectedRect=new Rectangle(evt.getX(),evt.getY(),0,0);
        drawGraphic();
    }//GEN-LAST:event_formMousePressed

    private void formMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseExited
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }//GEN-LAST:event_formMouseExited

    private void formMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseEntered
        setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
    }//GEN-LAST:event_formMouseEntered

    /** Sets the preferred size as the current size.
     * This method is called when the panel is resized.
     */
    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        setPreferredSize(getSize());
    }//GEN-LAST:event_formComponentResized

    /** Shows the popup of the graphic panel.
     * This method is called when the right button of the mouse is clicked.
     */
    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        if(evt.getModifiers()==evt.BUTTON3_MASK){
        /*    if(gi instanceof GraphicFrame && addGraph!=null)
                addItems();           
          */  if(popup.isEnabled())
                popup.show(graph , evt.getPoint().x, evt.getPoint().y);
            return;
        }
        
        Rectangle r=graph.datarect;
        double x=xaxis_bottom.getDouble(evt.getX()-getInsets().left);
        double y=yaxis_right.getDouble(evt.getY()-getInsets().top);
        double[] point=getDataSet(gi.getSelectedVariable()).getClosestPoint(x,y);
        nearestPoint="x="+point[0]+"  y="+point[1];
        drawGraphic();
    }//GEN-LAST:event_formMouseClicked

    /** Adds all the items and listeners of the addGraph and deleteGraph menus od the popup.
     * The addition depends on the class of the graphicInterface gi. Because the struct of
     * the variables in the DinamicViewer is very different from the struct of the varialbes in
     * the real time statistics.
     */
    private void addItems(){
        /*
        addGraph.removeAll();
        deleteGraph.removeAll();
        if(gi instanceof GraphicFrame){    
            //gets a Hashtable with the modules of the dinamicTreeManager
            Hashtable modules=((GraphicFrame)gi).dtm.getModules();
            for(Enumeration modEnum=modules.keys();modEnum.hasMoreElements();){
                final String module=(String)modEnum.nextElement();
                Hashtable vars=(Hashtable)modules.get(module);
                Menu addMenu=new Menu(module);
                Menu delMenu=new Menu(module);
                addGraph.add(addMenu);                
                boolean hasVariables=false;
                for(Enumeration varEnum=vars.keys();varEnum.hasMoreElements();){
                    final String variable=(String)varEnum.nextElement();
                    MenuItem varItem=new MenuItem(variable);
                    if(!hasVariable(module,variable)){
                        addMenu.add(varItem);
                        varItem.addActionListener(new java.awt.event.ActionListener() {
                            public void actionPerformed(java.awt.event.ActionEvent evt) {
                                gi.addGraphic(module,variable);                                
                            }               
                        });
                    }
                    else{
                        delMenu.add(varItem);
                        hasVariables=true;
                        varItem.addActionListener(new java.awt.event.ActionListener() {
                            public void actionPerformed(java.awt.event.ActionEvent evt) {
                                gi.deleteGraphic(module,variable);                                
                            }              
                        });
                    }
                }
                if(hasVariables)
                    deleteGraph.add(delMenu);
            }
        }
        if(gi instanceof StatisticsGraphic){
            //gets a hashtable with all the statistics from the panelsFrame
            Hashtable allStatistics= PanelsFrame.allStatistics;            
            for(Enumeration statEnum=allStatistics.keys();statEnum.hasMoreElements();){
                final String module=statEnum.nextElement().toString();
                Vector statistics=(Vector)allStatistics.get(module);
                Menu addMenu=new Menu(module);
                Menu delMenu=new Menu(module);
                addGraph.add(addMenu);                
                boolean hasVars=false;
                for(int i=0;i<statistics.size();i++){
                    Object var=statistics.elementAt(i); 
                    if(var instanceof TableStatisticsStruct){
                        final TableStatisticsStruct tss=(TableStatisticsStruct)var;
                        Menu addTableMenu=new Menu(tss.getName());
                        Menu delTableMenu=new Menu(tss.getName());
                        addMenu.add(addTableMenu);
                        boolean hasVariables=false;
                        for(int j=1;j<tss.column.size();j++){
                            final TableStatisticsStruct.VariableStatisticsStruct vss = tss.getVariable(j);
                            if(StatisticsFrame.getType(vss.getType())==StatisticsGraphic.NUMERIC){
                                MenuItem addVarItem=new MenuItem(vss.getName());
                                Menu delVarMenu=new Menu(vss.getName());
                                addTableMenu.add(addVarItem);                                
                                final int index=j;
                                addVarItem.addActionListener(new java.awt.event.ActionListener() {
                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                        (new IndexChooser(gi,tss,index,module)).show();
                                        addItems();
                                    }  
                                });
                                if(addDeleteItems(delVarMenu,tss.getName(),vss.getName(),module)){
                                    delTableMenu.add(delVarMenu);
                                    hasVariables=true;
                                }
                            }
                        }
                        if(hasVariables){
                            delMenu.add(delTableMenu);
                            hasVars=true;
                        }
                    }                    
                    if(var instanceof SimpleStatisticsStruct){                       
                        final SimpleStatisticsStruct sss=(SimpleStatisticsStruct)var;
                        final String variable=sss.getName();
                        MenuItem varItem=new MenuItem(variable);
                        if(!hasVariable(module,variable)){
                            if(StatisticsFrame.getType(sss.getType())==StatisticsGraphic.NUMERIC){
                                addMenu.add(varItem);
                                varItem.addActionListener(new java.awt.event.ActionListener() {
                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                        gi.addGraphic(module,variable,sss.getId(),-1);  
                                        addItems();
                                    }               
                                });
                            }
                        }
                        else{
                            delMenu.add(varItem);
                            hasVars=true;
                            varItem.addActionListener(new java.awt.event.ActionListener() {
                                public void actionPerformed(java.awt.event.ActionEvent evt) {
                                    gi.deleteGraphic(module,variable);  
                                    addItems();
                                }              
                            });
                        }
                    }
                }
                if(hasVars)
                    deleteGraph.add(delMenu);
            }
        }
         */
        
    }
    
    /** Adds a item to the deleteGraph menu with a table struct variable.
     * @param del Menu delVarMenu to add the variable
     * @param table String with the name of the table
     * @param var String with the name of the variable
     * @param module String with the name of the module
     * @return True if an item has been added (if the graphic contains this variable)
     */
    private boolean addDeleteItems(Menu del,String table,String var,final String module){
        boolean hasVariable=false;
        for(int i=0;i<dataArray.size();i++)
            if(((VariableDataSet)dataArray.elementAt(i)).getModule().equalsIgnoreCase(module)){
                final String variable=((VariableDataSet)dataArray.elementAt(i)).getVariable();
                StringParser sp=new StringParser(variable,false);
                try{
                    if(sp.hasNextToken('(')){
                        if(!sp.getNextToken('(').equalsIgnoreCase(table))
                            continue;
                        String name=sp.getNextToken(')');
                        sp.getNextToken('-');
                        if(sp.getNextToken().equalsIgnoreCase(var)){
                            MenuItem item=new MenuItem(name);
                            item.addActionListener(new java.awt.event.ActionListener() {
                                public void actionPerformed(java.awt.event.ActionEvent evt) {
                                    gi.deleteGraphic(module,variable);  
                                    addItems();
                                }                                       
                            });
                            del.add(item);
                            hasVariable=true;
                        }
                    }
                }
                catch(NullPointerException exc){ Trace.print("Error adding items into the panel Popup: "+exc.toString(),Trace.ERRORS,"GraphicPanel",868);}
            }
        return hasVariable;       
    }
    
    
    /** Sets the YTitle and XTitle of the graphic panel depending on its type or scale. 
     * @see Axis#setTitleText
     */
    private void upDateTitles(){
        String title=YTitle;
        if(type==DIFF_TYPE)
            title+=" (DIFF)";
        if(type==INT_TYPE)
            title+=" (INT)";
        if(scale==LOG_SCALE)
            title+=" (LOG)";
        
        if(!title.equalsIgnoreCase(""))
            getYAxisRight().setTitleText(title);       


        if(unitDivision==1)
            getXAxisBottom().setTitleText("Time-Slot");

        if(unitDivision==1000)
            getXAxisBottom().setTitleText("1000 x Time-Slot");
    }

    /** Sets the defaults of the graphic. Creates the Grapg2D object and the X,Y
     * axis and a DataSet to be attached to them. Also creates a new VariableDataSet.
     * And reads the markers from the 'marker.txt' file.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     */
    private void setDefaults(String module, String variable){
        
        // creates the graph instance and modifies the default behaviour
	graph = new Graph2D();
        
        // sets graph's gridcolor
	graph.gridcolor = Color.gray;
	// sets graph's gridcolor at the zero label
	graph.zerocolor = Color.yellow;
	// sets the width of the borders of the canvas
	graph.borderTop = 20;
        graph.borderBottom = 0;
	graph.borderRight = 10;
        graph.borderLeft = 10;
	// sets graph's background correspending the current background
	graph.setDataBackground(MyColors.decodeColor("lightYellow"));

        
        add(graph);
        
         String userDir = System.getProperty("user.dir"),     // find actual directory
              file_sep = System.getProperty("file.separator");        
                
        // loads a file containing marker definitions
	try
	    {
		// sets the optional markers for the plot
		graph.setMarkers(new Markers(new File(markerfile)));
	    }
	catch (Exception e)
	    {
                Trace.print(e.toString(),Trace.ERRORS,"GraphicPanel",932);
	    }

        VariableDataSet vds= new VariableDataSet(module, variable,Color.blue);
        
        dataArray.add(vds);  // Adding data to the array

        DataSet data= vds.getDataSet(); 
        data.legend(20,30,module + "_" + variable);
        
	// creates and attaches an y-axis to the graph
	yaxis_right = graph.createAxis(Axis.RIGHT);
	// attaches a DataSet for the y-axis to manage
	yaxis_right.attachDataSet(data);
	// sets the font of the labels
	yaxis_right.setLabelFont(new Font("Helvetica", Font.PLAIN, 15));
	        
	// sets the title rotation angle
	yaxis_right.setTitleRotation(90);
	
	// creates and attaches an x-axis to the graph
	xaxis_bottom = graph.createAxis(Axis.BOTTOM);
	// attaches a DataSet for the x-axis to manage
	xaxis_bottom.attachDataSet(data);
	// sets the font of the labels
	xaxis_bottom.setLabelFont(new Font("Helvetica", Font.PLAIN, 15));
	// sets the title of the axis
	xaxis_bottom.setTitleText("Time-slot");
	        
	// attaches a DataSet to the graph
	graph.attachDataSet(data);
        
    }
    
    
    /** Configures the popup of the graphic. */
    private void popupConfiguration(){
        
                
        popup= new PopupMenu();
        options= new MenuItem("Graph options"); 
        autoRange= new MenuItem("Back to auto-range");
        addGraph= new Menu("Add Graph");
        deleteGraph= new Menu("Delete Graph");
         
        popup.add(options);
        popup.add(autoRange);
        popup.addSeparator();
        popup.add(addGraph);  
        popup.add(deleteGraph);                
       
        graph.add(popup);
        
        deleteGraph.setEnabled(false);
        
        options.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                (new GraphicOptions(PhalguiApp.getApplication().getMainFrame(), gi,GraphicPanel.this)).show();
            }

        });  
        
        autoRange.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xaxis_bottom.setManualRange(false);
                xaxis_bottom.resetRange();
                yaxis_right.setManualRange(false);
                yaxis_right.resetRange();
                drawGraphic();
            }            
        });
        
    }
    
    /** Configures a simple popup for the graph. Is used in the loaded graphs. */
    private void simplePopupConfiguration(){        
                
        popup= new PopupMenu();
        options= new MenuItem("Graph options"); 
        autoRange= new MenuItem("Back to auto-range");
        
        popup.add(options); 
        popup.add(autoRange);
        graph.add(popup);
        
        options.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                (new GraphicOptions(PhalguiApp.getApplication().getMainFrame(),gi,GraphicPanel.this)).show();
            }            
        });
        
        autoRange.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xaxis_bottom.setManualRange(false);
                xaxis_bottom.resetRange();
                yaxis_right.setManualRange(false);
                yaxis_right.resetRange();
                drawGraphic();
            }            
        });
    }
    
    private DataSet getDataSet(String selectedVariable){
        StringParser sp=new StringParser(selectedVariable,false);
        String module=sp.getNextToken('-').trim();
        String variable=sp.getNextToken().trim();        
        for(int i=0;i<dataArray.size();i++)
            if(((VariableDataSet)dataArray.elementAt(i)).isVariable(module,variable)){
                return ((VariableDataSet)dataArray.elementAt(i)).getDataSet();
            }
        return null;                
    }

    public interface addGraphicInterface {
        public void addtoGraphic(GraphicInterface gi);
    }

    @Override
    public void dropObject(Object object, DropTargetDropEvent dtde) {
       if (object instanceof addGraphicInterface) {
           ((addGraphicInterface) object).addtoGraphic(gi);
       }
    }
                                
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
