package phalgui.Node.Stats;

/* ===========================================================
 * JFreeChart : a free chart library for the Java(tm) platform
 * ===========================================================
 *
 * (C) Copyright 2000-2004, by Object Refinery Limited and Contributors.
 *
 * Project Info:  http://www.jfree.org/jfreechart/index.html
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 * -----------------
 * BarChartDemo.java
 * -----------------
 * (C) Copyright 2002-2004, by Object Refinery Limited and Contributors.
 *
 * Original Author:  David Gilbert (for Object Refinery Limited);
 * Contributor(s):   -;
 *
 * $Id: BarChartDemo.java,v 1.16 2004/04/29 10:06:34 mungady Exp $
 *
 * Changes
 * -------
 * 11-Jun-2002 : Version 1 (DG);
 * 25-Jun-2002 : Removed redundant imports (DG);
 * 09-Oct-2002 : Added frame centering (DG);
 * 18-Nov-2002 : Changed from DefaultCategoryDataset to DefaultTableDataset (DG);
 * 28-Oct-2003 : Changed to display gradient paint (DG);
 * 10-Nov-2003 : Renamed BarChartDemo.java (DG);
 *
 */


import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.BoxLayout;
import javax.swing.JPanel;

import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * A simple demonstration application showing how to create a bar chart.
 *
 */
public class HistoPanel extends JPanel {
    private JFreeChart chart;
    final XYSeriesCollection dataset = new XYSeriesCollection();
    private Vector categories = new Vector<Category>();

    private HistoPanelOptions optionsPanel = new HistoPanelOptions();

    private boolean autointerval=true;

    private int nofBins = 100;

    private boolean relative = true;

    private boolean accumulate = false;

    public String getOptions() {
        return new String(optionsPanel.nofBins.getText() + "," + (autointerval?"yes":"no") + "," + optionsPanel.maxValue.getText() + "," + optionsPanel.minValue.getText());
    }
    public void setOptions(String opts) {
        String s[] = opts.split(",");
        nofBins=Integer.valueOf(s[0]);
        optionsPanel.nofBins.setText(String.valueOf(nofBins));
        autointerval=s[1].equals("yes");
        optionsPanel.maxValue.setText(s[2]);
        optionsPanel.minValue.setText(s[3]);

        optionsPanel.maxValue.setEnabled(!autointerval);
        optionsPanel.minValue.setEnabled(!autointerval);
        optionsPanel.autointervalButton.setSelected(autointerval);
    }

    /**
     * Creates a new demo instance.
     *
     * @param title  the frame title.
     */
    public HistoPanel(String title) {
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));

        add(optionsPanel);
        optionsPanel.setMaximumSize(new Dimension(1000,20));
        optionsPanel.clearDataButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                for (int i=0;i<categories.size();i++) {
                    ((Category) categories.get(i)).clearData();
                }
            }
        });
        optionsPanel.maxValue.setEnabled(!autointerval);
        optionsPanel.minValue.setEnabled(!autointerval);
        optionsPanel.autointervalButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (autointerval) {
                    autointerval=false;
                    optionsPanel.maxValue.setEnabled(true);
                    optionsPanel.minValue.setEnabled(true);
                } else {
                    autointerval=true;
                    optionsPanel.maxValue.setEnabled(false);
                    optionsPanel.minValue.setEnabled(false);
                }
            }
        });

        optionsPanel.nofBins.setText(String.valueOf(nofBins));

        optionsPanel.relativeButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (relative) {
                    relative = false;
                } else {
                    relative = true;
                }
            }
        });

        optionsPanel.acumulateButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (accumulate) {
                    accumulate = false;
                    optionsPanel.clearDataButton.setEnabled(false);
                } else {
                    accumulate = true;
                    optionsPanel.clearDataButton.setEnabled(true);
                }
            }
        });

        createChart(title);
        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new Dimension(500, 270));
        add(chartPanel);
    }


    void addData(double y[], String category) {
        int n = categories.indexOf(new CategorySearch(category));
        Category c;
        if (n==-1) {
            c = new Category(category,y);
            categories.add(c);
            dataset.addSeries(c.serie);
        } else {
            c = (Category) categories.get(n);            
        }
        if (!accumulate) {
            c.clearData();
        }
        c.addData(y);

        for (int i=0;i<categories.size();i++) {
            c = (Category) categories.get(i);
            double max=1000,min=0;
            if (autointerval) {
                max=-99999999;
                min=99999999;
                for (int j=0;j<c.y.length;j++) {
                    max = Math.max(max, c.y[j]);
                    min = Math.min(min, c.y[j]);
                }
                optionsPanel.maxValue.setText(String.valueOf(max));
                optionsPanel.minValue.setText(String.valueOf(min));
            } else {
                max = Double.valueOf(optionsPanel.maxValue.getText());
                min = Double.valueOf(optionsPanel.minValue.getText());
            }
            double mean=0;
            double desv=0;
            for (int j=0;j<c.y.length;j++) {
                mean+=c.y[j];
                desv+=c.y[j]*c.y[j];
            }
            mean=mean/c.y.length;
            desv=Math.sqrt(desv/c.y.length-mean*mean);
            DecimalFormat format = new DecimalFormat("##.#");
            optionsPanel.meanTxt.setText(format.format(mean));
            optionsPanel.desvTxt.setText(format.format(desv));

            nofBins = Integer.valueOf(optionsPanel.nofBins.getText());
            int hist[] = histogram(c.y,min,max,nofBins);

            double area=1.0;
            if (relative) {
                area=0;
                for (int j=0;j<hist.length;j++) {
                    area+=hist[j];
                }
            }
            c.serie.clear();
            for (int j=0;j<hist.length;j++) {
                c.serie.add(min+(

                        j+1)*(max-min)/nofBins, hist[j]/area);
            }
        }
    }


    /**
     * Creates a sample chart.
     *
     * @param dataset  the dataset.
     *
     * @return The chart.
     */
    private JFreeChart createChart(String title) {

       chart = ChartFactory.createXYBarChart(
            title,
            null,
            false,
            "Probability",
            dataset,
            PlotOrientation.VERTICAL,
            true,
            true,
            false
        );

       ((XYBarRenderer) chart.getXYPlot().getRenderer()).setShadowVisible(false);

       return chart;

    }

    void drawGraphic() {
        chart.getPlot().datasetChanged(new DatasetChangeEvent(this,dataset));
    }


    private static int[] histogram(double values[], double min, double max, int numBins) {
        int histo[]=new int[numBins];
        for (int i=0;i<values.length;i++){
            if (values[i]<max && values[i]>min) {
                int indice=(int) ((values[i] - min) * (numBins-1)/(max-min));
                histo[indice]++;
            }
        }
        return histo;
    }

    class CategorySearch {
        private final String name;
        CategorySearch(String name) {
            this.name = name;
        }
        @Override
        public boolean equals(Object x) {
            if (x instanceof Category) {
                return ((Category) x).name.equals(name);
            } else {
                return false;
            }
        }
    }

    class Category {
        private final String name;
        double y[];
        XYSeries serie;
        Category(String name, double y[]) {
            this.name = name;
            this.y = y;
            this.serie = new XYSeries(name);
        }
        void addData(double newdata[]) {
            double x[];
            x = new double[y.length + newdata.length];
            for (int i=0;i<y.length;i++) {
                x[i]=y[i];
            }
            for (int i=y.length;i<y.length+newdata.length;i++) {
                x[i]=newdata[i-y.length];
            }
            y=x;
        }
        void clearData() {
            y = new double[0];
        }
    }
}
