/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.Stats;

import java.awt.Toolkit;
import java.awt.datatransfer.Transferable;
import java.awt.event.MouseEvent;
import java.io.FileNotFoundException;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingWorker;
import phalControl.StatControl;
import phalgui.Control.MainControl;
import phalgui.Node.NodeObject;
import phalgui.Node.Stats.StatisticsGraphic.View;
import phalgui.Node.Stats.ViewStatisticsGraphic.CaptureMode;
import phalgui.Node.Waveform.WaveformEntry;
import phalgui.Tree.TreeMenuItem;
import phalgui.Tree.TreePopup;

/**
 *
 * @author Usuario
 */
public class StatNode extends NodeObject {

    
    private MainControl main;
    private StatControl stat;
    private WaveformEntry waveform;
    private boolean makePanel;


    public StatNode(MainControl main, StatControl statInfo, WaveformEntry waveform) {
        this(main,statInfo,waveform,null);
    }
    public StatNode(MainControl main, StatControl statInfo, WaveformEntry waveform, ImageIcon icon) {
        super(statInfo.toString(), icon);
        this.main = main;
        this.stat = statInfo;
        this.waveform = waveform;
        setPopup(new TreePopup(buildMenu()));
    }

    @Override
    public String toString() {
        return stat.toString();
    }

    public TreeMenuItem[] buildMenu() {
        TreeMenuItem items[] = new TreeMenuItem[3];
        items[0] = new TreeMenuItem("Start reporting") {
            public void action() {startStatView(CaptureMode.REPORT,null);}
            public boolean actionEnabled() { return true; }
        };
        items[1] = new TreeMenuItem("View") {
            public void action() {startStatView(CaptureMode.MANUAL,null);}
            public boolean actionEnabled() { return true; }
        };
        items[2] = new TreeMenuItem("Modify") {
            public void action() {startStatView(CaptureMode.MANUAL,View.MODIFY);}
            public boolean actionEnabled() { return true; }
        };
       
        return items;
    }

    @Override
    public void DoubleClick(MouseEvent evt) {
       startStatView(CaptureMode.MANUAL,null);
    }

    @Override
    public Transferable createTransferable(JComponent c) {
        try {
            return (Transferable) createStatView(false,null,null);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(StatNode.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    @Override
    public void exportDone(JComponent c, Transferable data, int action) {
        if (main.getTabPane().hasComponent(statView))
            main.getTabPane().remove(statView);

    }

    StatView statView = null;

    public StatView createStatView(boolean makePanel, CaptureMode mode, View view) throws FileNotFoundException {
        if (statView != null) {
            if (mode != null)
                statView.setCaptureMode(mode);
            return statView;
        } else {
            return new StatView(main, stat, waveform.getReposPath(),makePanel,mode,view);
        } 
    }

    private void startStatView(CaptureMode mode,View view) {
        new createReportViewThread(this,mode,view).execute();
    }
    
    private class createReportViewThread extends SwingWorker<StatView,Void> {
        private StatNode parent;
        private CaptureMode mode;
        private View view;
        public createReportViewThread(StatNode parent,CaptureMode mode,View view) {
            this.parent=parent;
            this.mode=mode;
            this.view=view;
        }

        @Override
        protected StatView doInBackground() throws Exception {
            return createStatView(true,mode,view);
        }

        @Override
        protected void done() {
            try {
                statView = get();
            } catch (InterruptedException ex) {
                Logger.getLogger(StatNode.class.getName()).log(Level.SEVERE, null, ex);
                return;
            } catch (ExecutionException ex) {
                Logger.getLogger(StatNode.class.getName()).log(Level.SEVERE, null, ex);
                return;
            }
            if (!main.getTabPane().hasComponent(statView)) {
                main.getTabPane().addTab("View stat: " + parent.toString(), statView, parent);
            }
            main.getTabPane().setSelectedComponent(statView);
            main.StatusReset();
        }
    }





    @Override
    public boolean Close() {
        if (statView==null)
            return true;

        SwingWorker w = new SwingWorker<Void,Void>() {

            @Override
            protected Void doInBackground() throws Exception {
                statView.stop();
                return null;
            }
            @Override
            protected void done() {
                   statView=null;
            }
        };
        w.execute();
        return true;
    }


    @Override
    public void Refresh() { }


}

