/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.Stats;

import java.awt.datatransfer.DataFlavor;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import phalControl.PlatformControl.WaveformNotFoundException;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JLabel;
import javax.swing.JPanel;
import phalControl.StatControl;
import phalgui.Control.MainControl;
import phalgui.Control.MainControl.PlatformNotConnected;
import phalgui.Node.Stats.StatisticsGraphic.View;
import phalgui.Node.Stats.ViewStatisticsGraphic.CaptureMode;
import phalgui.PhalguiApp;
import phalgui.Utils.DnD.TransfResizePanel;

/**
 *
 * @author Usuario
 */
public class StatView extends TransfResizePanel implements GraphicPanel.addGraphicInterface {
    private StatControl statControl;
    public DataFlavor[] statflavors = {PANEL,OBJECT};
    private ViewStatisticsGraphic comp;
    private MainControl main;
    private String ReposPath;
    private CaptureMode mode;
    private String statName;
    private String waveformName;
    private String objName;
    private View view;

    public StatView(String[] info, DataFlavor flavor) throws PlatformNotConnected, WaveformNotFoundException, ErrorInCommandException, FileNotFoundException {
        this(((PhalguiApp) PhalguiApp.getApplication()).getMainControl(),info, flavor.getHumanPresentableName().equals(PANEL.getHumanPresentableName()));
    }
    public StatView(MainControl main, String[] info, boolean initpanel) throws ErrorInCommandException, FileNotFoundException{
        super();
        try {
            this.main = main;
            comp = null;
            this.ReposPath = (String) info[3];
            if (info[4].length()>0)
                this.mode = CaptureMode.valueOf((String) info[4]);
            if (info[5].length()>0)
                this.view = StatisticsGraphic.View.valueOf((String) info[5]);
            this.statControl=null;
            this.statName = info[2];
            this.waveformName = info[0];
            this.objName = info[1];

            add(snotcon, java.awt.BorderLayout.CENTER);
            snotcon.setVisible(true);
            refresh();

            if (info.length>6 && info[6]!=null && info[6].length()>0 && comp!=null) {
                comp.capturePanel.setVisibleString(info[6]);
            }
            if (info.length>7) {
                if (comp.histoPanel != null && info[7]!=null)
                    comp.histoPanel.setOptions(info[7]);
            }
        } catch (WaveformNotFoundException ex) {
            Logger.getLogger(StatView.class.getName()).log(Level.SEVERE, null, ex);
        } catch (PlatformNotConnected ex) {
            Logger.getLogger(StatView.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public StatView(MainControl main, StatControl statControl, String ReposPath, boolean initpanel, CaptureMode mode, View view) throws FileNotFoundException  {
        super();
        this.statControl = statControl;
        this.main=main;
        comp=null;
        this.ReposPath=ReposPath;
        this.mode=mode==null?CaptureMode.MANUAL:mode;
        this.view=view==null?View.SIMPLEGRAPH:view;
        statControl.configReport(ReposPath);

        add(snotcon, java.awt.BorderLayout.CENTER);
        snotcon.setVisible(true);
        if (initpanel)
            initPanel();


      
    }


    public void initPanel() throws FileNotFoundException {
        createStatPanel();
    }

    @Override
    public String toString() {
        return statName;
    }

    @Override
    public DataFlavor[] getTransferDataFlavors() {
        return statflavors;
    }
    
    public void addtoGraphic(GraphicInterface gi) {
        main.StatusShow("Loading stat " + statName);
        gi.addStat(statControl);
    }

    StatNotCon snotcon = new StatNotCon(this);
    void createStatPanel() {
        main.StatusShow("Loading stat " + statName);
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setLayout(new java.awt.BorderLayout());
        if (statControl != null) {
            if (comp==null)
                comp = new ViewStatisticsGraphic(statControl,mode,view);
            remove(snotcon);
            snotcon.setVisible(false);
            add(comp, java.awt.BorderLayout.CENTER);
        } 

    }

    public void refreshPanel() {
        try {
            refresh();
        } catch (PlatformNotConnected ex) {
            Logger.getLogger(StatView.class.getName()).log(Level.SEVERE, null, ex);
        } catch (WaveformNotFoundException ex) {
            Logger.getLogger(StatView.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ErrorInCommandException ex) {
            Logger.getLogger(StatView.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(StatView.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void refresh() throws PlatformNotConnected, WaveformNotFoundException, ErrorInCommandException, FileNotFoundException {
        this.statControl = main.getPlatform().getWaveform(waveformName).getStatControl(objName, statName);
        statControl.configReport(ReposPath);
        initPanel();
    }

    void setCaptureMode(CaptureMode mode) {
        if (comp==null)
            return;
        comp.setCaptureMode(mode);
    }

    public String[] getTransferData() {
        String[] info = new String[8];
        info[0] = statControl.getWaveformName();
        info[1] = statControl.getObjectName();
        info[2] = statControl.toString();
        info[3] = ReposPath;
        if (comp!=null)
            mode = comp.Mode;
        if (mode==null)
            info[4] = "";
        else
            info[4] = mode.toString();
        if (comp!=null)
            view = comp.view;
        if (view==null)
            info[5] = "";
        else
            info[5] = view.toString();
        if (comp!=null && comp.capturePanel!=null)
            info[6] = comp.capturePanel.isVisibleString();
        if (view!=null && view.equals(View.HISTOGRAM)) {
            info[7]=comp.histoPanel.getOptions();
        }
        return info;
    }

    void stop() {
        if (comp != null)
            comp.stop();
    }


    @Override
    public void Close() {
        stop();
    }






}

