/*
 * StatisticGraphic.java
 *
 * Created on 4 de febrero de 2002, 19:00
 */

package phalgui.Node.Stats;


import java.awt.event.MouseEvent;
import java.util.logging.Level;
import java.util.logging.Logger;
import phalControl.StatControl;
import phalControl.StatControl.InvalidReportFormat;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Utils.Arrows.ValueStamp;
import phalgui.Utils.Arrows.Trace;
import phalgui.Utils.Arrows.Extension;
import phalgui.Utils.Arrows.MyBoolean;
import phalgui.Utils.Arrows.StringParser;
import phalgui.Utils.Arrows.ArrowsFilter;
import phalgui.Utils.Arrows.ArrowsFileView;
import java.awt.*; 
import java.util.*;
import java.io.*;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import phalControl.commandManager.StatInfo;
import phalgui.Utils.VectorId;

/**
 * This class is the panel where the statistics graphics are shown. There are four types of
 * graphics: Numeric, Text, Histogram and MobilePath. Depending on the type,
 * some functions are implemented different. This panel is used to show the graphs 
 * relationed with the real time statistics. A similar class is used to show
 * the graphs of the logs and loaded graphs named GraphicFrame, where the implemention
 * of the methods is quite different.<BR>
 * This class includes a DataVector class that is used for saving the data.
 * This savings could be enabled or disabled and also the limit number of values
 * for each graph; this options can be configured in the Arrows Options.
 * This savings permits non saturate the computer memory without loss the data. If the user
 * wants, all the data of the graphic can be saved in a graph file.<BR>
 * This Frame can contain a different types of panels depending on its type, this panels
 * are the next classes: 
 * <PRE>
 * Numeric*:    GraphicPanel(VariableDataSet, GraphicOptions), StatisticsPanel(StatisticsVector) and ValuePanel.
 * Text:        ValuePanel.
 * Histogram:   HistoGraph (HistoOptions).
 * MobilePath:  MobilePathGraph (MobilePathOptions)
 * 
 * The classes in parenthesis are classes relationed with the panels.
 *
 * *The numeric type can be shown with different views: 
 *  NumberFieldGraph, TextFieldGraph, StatisticsChartGraph, SimpleXYChartGraph and OnlyXYChartGraph
 * </PRE>
 * @author  Santi Oliveras
 * @version 1.0  2/05/2002
 * @see StatisticsFrame
 * @see ArrowsOptions
 */
public class StatisticsGraphic extends JPanel implements GraphicInterface {
    
    // ******** private variables **********************************************
    
    ValuePanel  valuePanel=null;    //A panel which shows the current value
    javax.swing.JPanel variableSelectPanel=null; //A panel which contains a combobox whith the available variables
    javax.swing.JLabel jLabel=null; //A label used in the jPanel 
    StatisticsPanel statistics=null;//A panel with the statistics    
    Hashtable statisticsArray=new Hashtable(20);    //Hashtable containing StatisticsVector, for each variable
    StatInfo.Type type;            //Tthe type of the graphic
    Vector data=new Vector();       //A vector of DataVectors with the saved data
    int id, index;                  //The id and index of the variable used in the non numeric types 
    String module=null,variable=null; //Variables used in the non numeric types 
    private boolean lastWasNull=false;      //used for null values
    StatModifyPanel statModifyPanel;
    
    // ******** protected variables ********************************************
    protected JPopupMenu popup=null;           //The popup used in numeric type
    protected javax.swing.JComboBox variableComboBox=null; //A combobox with the available variables

    /** A panel with the graph. */
    protected GraphicPanel graphic=null;
    
   
    /** An item in the popup menu to show or hide the pollingTime panel. */
    protected JCheckBoxMenuItem pollingTime;


    /** True if the pollingTime of the panel is visible. */
    public boolean pollingTimeVisible;

    /** Integer with the polling time value in miliseconds. */
    public int pollingTimeValue;

    /** The graphic identifier. Used to save the data in the 'Variables' temporal
     * directory. Because same variables could be in different graphics.
     */
    int graphID;
    
    
    // ******** public variables ***********************************************
    
    /** It's the number of StatisticsGraphs created untill this moment. 
     * When the panelsFrame is created, this number is set to 0.
     * @see PanelsFrame#PanelsFrame
     */
    public static int num;
    
    /** A boolean used when we are reading from a file, when we are reading from a 
     * panels Agrupation frame in the creation of the panel.
     * @see #StatisticsGraphic(java.awt.Frame,BufferedReader)
     */
    public boolean loadFailed;
  

    VectorId stats = new VectorId();
    protected sgPlot sgt;
    protected sgData sgdata;
    public ReportOptionsPanel capturePanel;

    protected HistoPanel histoPanel;
    protected XYPanel xyPanel;


    public enum View {
        NUMBER(0),TEXT(1),NUMBERGRAPH(2),SIMPLEGRAPH(3),ONLYGRAPH(4),SCOPE(5),MODIFY(6),HISTOGRAM(7),XY(8);

        public static View getFromInteger(int integer) {
            for (int i=0;i<values().length;i++) {
                if (values()[i].numeric==integer) {
                    return values()[i];
                }
            }
            return null;
        }
        private int numeric;
        View(int numeric) {
            this.numeric=numeric;
        }
        public int getInteger() {
            return numeric;
        }
    }

    public View view;
    

    // ******** constructors ***************************************************

  /** Creates new form StatisticGraphic called when a variable is selected. This variable can
     * be selected pressing the OK button in the StatisticsDialog or StatisticsModuleDialog 
     * in the real time Statistics.
     * @param type Integer with the type of the graphic 
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @param id Integer with the id of the variable
     * @param index Integer with the index of the variable (if it's a simple variable, this value is -1)
     * @see GraphicFrame
     */
    public StatisticsGraphic(StatControl statControl) {
        this.type=statControl.getType();
        this.index=-1;
        this.module=statControl.getObjectName();
        this.variable= statControl.toString();
        createPopup();
        initComponents();
        createPopup();
        num++;
        graphID=num;
        this.stats.add(statControl);
    }

    @Override
    public String toString() {
        return module + "_" + variable;
    }
      
    /** Creates new form StatisticGraphic from another StatisticGraphic. This method is
     * used when a StatisticsGraphic is added to a GraphicAgrupation in the panels frame;
     * so a copy of a StatisticGraphic is placed in this panel creating a new 
     * MovingStatisticsGraphic (which extends a StatisticsGraphic); from this moment,
     * the StatisticsGraphic and its copy (the MovingStatisticsGraphic) are independent.
     * @param sg The StatisticGraphic to be copied
     * @see #getParameters
     * @see MovingStatisticsGraphic#MovingStatisticsGraphic(java.awt.Frame,StatisticsGraphic,GraphicAgrupation)
      */
    public StatisticsGraphic(StatisticsGraphic sg) {
        num++;
        graphID=num;
        this.stats=sg.stats;
        createPopup();
        initComponents();
    }


    
    /** Creates new form StatisticGraphic from a file. Is called when a MovingStatisticsGraphic 
     * is loaded from the panels agrupation file.
     * @param parent The parent Frame 
     * @param br Reader with the pointer of the file
     * @see MovingStatisticsGraphic#MovingStatisticsGraphic(java.awt.Frame,BufferedReader,GraphicAgrupation)
     */
    public StatisticsGraphic(BufferedReader br) {
        num++;
        graphID=num;
        createPopup();
        initComponents();
    }

    private void createPopup() {
        popup= new JPopupMenu();
        this.add(popup);
    }

    
    // ******** public methods *************************************************

    public StatControl getSelectedStatControl() {
        /*if (view.equals(View.SCOPE)) {
            return (StatControl) stats.get(0);
        }
         */
        String v=getSelectedVariable();
        if(statisticsArray.containsKey(v)){
            StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);
            return (StatControl) stats.getById(sv.getId());
        }
        return null;
    }

    public void addStat(StatControl statControl) {
        stats.add(statControl);
        this.addGraphic(statControl.getObjectName(), statControl.toString(), stats.getId(statControl), -1);
    }


    public void deleteStat(StatControl statControl) {
        stats.remove(statControl);
        this.deleteGraphic(statControl.getObjectName(), statControl.toString());

    }
    
    /** Reloads all the data. This method is used when some data options in the 
     * GraphicOptions are changed.
     * @see GraphicOptions
     */
    public void reloadAll(){
        String module,variable;
        StringParser sp=new StringParser();
        double[] values=new double[2];
        
        for(Enumeration statEnum=statisticsArray.keys();statEnum.hasMoreElements();){
            String v = statEnum.nextElement().toString();
            sp.setString(v);
            module=sp.getNextToken('-').trim();
            variable=sp.getNextToken().trim();            
            for(int i=0;i<data.size();i++){
                DataVector dv=(DataVector)data.elementAt(i);
                if(dv.equals(module,variable)){
                    values=dv.getDoubleData();  
                    break;
                }
            }            
            for(int i=0;i<values.length;i+=2)
                values[i]=values[i]/graphic.unitDivision;                
            setData(values,module,variable);            
        }
    }
        
    /** Sets the new data to the graphic panel. The data is processed in case of require a
     * processing. See the private method ProcessData(double[], StatisticsVector).
     * It also updates the statistics vector and statistics panel and the current value.
     * @param points An array of doubles values.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see GraphicPanel#upDateGraphic
     */
    public void setData(double[] points,String module,String variable){
        if(points.length==0){
            resetAll(module,variable);
            return;
        }
        String v=new String(module + " - " + variable);        
        if(statisticsArray.containsKey(v)){
            StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);
            sv.resetAll();
            double processed;
            for(int i=1;i<points.length;i+=2){                
                processed=processData(points[i],sv);
                sv.setLinealLast(points[i]);                
                sv.addValue(processed);
                points[i]=processed;
            }            
            graphic.upDateGraphic(points, module, variable);            
            if(variableComboBox.getSelectedItem().toString().equalsIgnoreCase(v)){
                valuePanel.setCurrentValue(Double.toString(points[points.length-1]));        
                statistics.setNewValues(sv.getAverage(), sv.getDesviation(),
                sv.getMaximum(), sv.getMinimum());
            }
        }
    }
    
    /** Deletes the specified variable in the graph. If the graph only contains this
     * variable, the graph is closed.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see GraphicPanel#deleteData(String,String)
     */
    public synchronized void deleteGraphic(String module, String variable){
        if(statisticsArray.size()==1){            
            return;
        }
        String v=new String(module + " - " + variable);
        variableComboBox.removeItem(v);            
        statisticsArray.remove(v);
        graphic.deleteData(module,variable);
        
        for(int i=0;i<data.size();i++){
            DataVector dv=(DataVector)data.elementAt(i);
            if(dv.equals(module,variable)){
                data.remove(dv);
            break;
            }
        }
    }
    
    /** Disables all the panels inside the StatisticGraphic. And the popup.
     */
    public void disableGraphic(){
        popup.setEnabled(false);
        if(graphic!=null)
            graphic.disablePopup();
        if(valuePanel!=null)
            valuePanel.setEnabled(false);
        if(variableComboBox!=null)
            variableComboBox.setEnabled(false);
        if(statistics!=null)
            statistics.setEnabled(false);
        if(variableSelectPanel!=null)
            variableSelectPanel.setEnabled(false);
        if(jLabel!=null)
            jLabel.setEnabled(false);
    }
    
    /** Adds a new variable in the graphic. This happens only in numeric graphs.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @param id Integer with the id of the variable
     * @param index Integer with the index of the variable (if it's a simple variable, this value is -1)
     * @see GraphicPanel#addNewData(String,String)
     */
    public synchronized void addGraphic(String module, String variable, int id, int index){
        String v=new String(module + " - " + variable);
        statisticsArray.put(v,new StatisticsVector(id, index));
        variableComboBox.addItem(v);
        graphic.addNewData(module,variable);
        data.add(new DataVector(module,variable));
        System.out.println("Added graph: " + statisticsArray.size() + "(" + module + "," + variable + ")");
    }
   
    /** Resets the data of the specified variable is the graph contains this variable.
     * This method is called when the graph is saving the data to a temporal file in
     * the 'Variables' directory.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see DataVector#addData(double,double)
     */ 
    public void resetData(String module, String variable){
        String v=new String(module + " - " + variable);        
        if(statisticsArray.containsKey(v)){
            graphic.resetAll(module,variable);        
            StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);
            sv.resetAll();
            if(variableComboBox.getSelectedItem().toString().equalsIgnoreCase(v)){
                valuePanel.setCurrentValue("");        
                statistics.setNewValues(0,0,0,0);                
            }
        }        
    }
    
    /** Resets the data of the specified variable is the graph contains this variable.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     */ 
    public void resetAll(String module, String variable){
               
        String v=new String(module + " - " + variable);        
        if(statisticsArray.containsKey(v)){
            graphic.resetAll(module,variable);        
            StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);
            sv.resetAll();
            if(variableComboBox.getSelectedItem().toString().equalsIgnoreCase(v)){
                valuePanel.setCurrentValue("");        
                statistics.setNewValues(0,0,0,0);                
            }
            for(int i=0;i<data.size();i++){
                DataVector dv=(DataVector)data.elementAt(i);
                if(dv.equals(module,variable)){
                    data.remove(dv);
                break;
                }
            }
        }
        
    }    
    
    /** Resets all the data of all the variables. */
    public void resetAll(){
        switch(type) {
            case HISTOGRAM:
                data.removeAllElements();
                break;
            case TEXT:
                valuePanel.setCurrentValue("");
                data.removeAllElements();
                break;
            default:
                graphic.resetAll();
                for(Enumeration statEnum=statisticsArray.keys();statEnum.hasMoreElements();){
                    String v=(String)statEnum.nextElement();
                    StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);
                    sv.resetAll();
                }
                data.removeAllElements();
                valuePanel.setCurrentValue("");
                statistics.setNewValues(0,0,0,0);
                break;
        }
    }
    public void nextValue(double milisec, String valstr, int id, int index) {
        this.nextValue(milisec,valstr, id, index, false);
    }
        
    
    /** Sets the current value of the specified variable. Process
     * the data if it's required.
     * @param milisec Integer with the current milisecond
     * @param valstr String with the current value of the variable
     * @param id Integer with the id of the variable
     * @param index Integer with the index of the variable (if it's a simple variable, this value is -1)
     * @see GraphicPanel#nextValue
     */
    public void nextValue(double milisec, String valstr, int id, int index, boolean draw) {
        DataVector dv;
        //Checking errors or null values in non numerical values
        switch(type) {
            case TEXT:
                valuePanel.setCurrentValue(valstr);
                dv=(DataVector)data.elementAt(0);
                if(dv.equals(module,variable))
                    dv.addData(milisec,valstr);
                break;
            default:
                String module=null,variable=null,v=null;
                StringParser sp= new StringParser();
                boolean found=false;

                for(Enumeration stsEnum=statisticsArray.keys();stsEnum.hasMoreElements();){
                    v=stsEnum.nextElement().toString();
                    if(((StatisticsVector)statisticsArray.get(v)).equals(id,index)){
                        sp.setString(v);
                        module=sp.getNextToken('-').trim();
                        variable=sp.getNextToken().trim();
                        found=true;
                        break;
                    }
                }
                if(!found)
                    return;

                if(lastWasNull==true){
                    graphic.getXAxisBottom().setManualRange(false);
                    lastWasNull=false;

                    //Modificar
                }

                double value = 0;

            try
                {
                value = Double.valueOf(valstr).doubleValue();
                }
            // throws an exception if the String cannot be parsed as a Double
            catch (NumberFormatException e){

                    if(statisticsArray.size()>1){
                        deleteGraphic(module,variable);
                        return;
                    }
                    /*
                    ArrowsError ae= new ArrowsError(parent);
                    ae.showErrorDialog(ArrowsError.MESSAGE,
                    new String[]{"It has found some text values in this variable",
                    "If you want to continue, it will only appear a textField",
                    "Do you want to continue?"},new String[]{"Continue","Close"});
                    if(ae.value.equalsIgnoreCase("Continue")){
                        resetAll();
                        setOnlyText(module,variable);
                        valuePanel.setCurrentValue(valstr);
                    }
                    else exit();
                     */
                    return;
                }

            StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);

                for(int i=0;i<data.size();i++){
                    dv=(DataVector)data.elementAt(i);
                    if(dv.equals(module,variable)){
                        dv.addData(milisec,value);
                        break;
                    }
                }

                double processed=processData(value,sv);
                sv.setLinealLast(value);
                sv.addValue(processed);
                graphic.nextValue(milisec,processed,module,variable, draw);
                if(variableComboBox.getSelectedItem().toString().equalsIgnoreCase(v)){
                    valuePanel.setCurrentValue(Double.toString(processed));
                    statistics.setNewValues(sv.getAverage(), sv.getDesviation(),
                    sv.getMaximum(), sv.getMinimum());
                    break;
                }
        }
        
    }
    
    /** Returns an string with the selected variable in the combobox. It's format is
     * 'module - variable' .
     */
    public String getSelectedVariable(){
        return variableComboBox.getSelectedItem().toString();
    }
        
    /** Returns a vector with the variables ids. This vector is given to the
     *  MessengerClient to get the real time values.<BR>
     * This vector has the next format:
     *<PRE>
     * (0) String with the module name of statistic 0
     * (1) Integer (id of simple statistic) or IndextexStruct (id and index) of the variable 0
     * (2) String with the module name of statistic 1
     * (3) Integer (id of simple statistic) or IndextexStruct (id and index) of the variable 1
     * (n) String with the module name of statistic n/2
     * (n+1) Integer (id of simple statistic) or IndextexStruct (id and index) of the variable n/2
     *</PRE>
     *
     * @see MessengerClient#getValues
     * @see #setValues
     */          
    public Vector getVariableIds(){
        Vector ids=new Vector();
        
        if(!type.equals(StatInfo.Type.NUMERIC)){
            ids.add(new String(module));
            if(index==-1)                
                ids.add(new Integer(id));            
            else
                ids.add(new IndexedStruct(id,index));            
            return ids;
        }
        
        String v;
        StringParser sp=new StringParser();
        StatisticsVector sv;
        
        for(Enumeration stsEnum=statisticsArray.keys();stsEnum.hasMoreElements();){
            v=stsEnum.nextElement().toString();
            sv=(StatisticsVector)statisticsArray.get(v);
            sp.setString(v);
            ids.add(sp.getNextToken('-').trim());
            ids.add(sv.getVariableId());
        }
        return ids;
    }
    
    
    /** Returns a vector of parameters with the objects of the StatisticsGraphic
     * to make a copy.
     * @see StatisticsGraphic#StatisticsGraphic(java.awt.Frame,StatisticsGraphic)
     */
    public Vector getParameters(){
        Vector parameters = new Vector();
        parameters.add(type.getInteger());
        parameters.add(module);
        parameters.add(variable);
        parameters.add(new Integer(id));
        parameters.add(new Integer(index));
        parameters.add(variableComboBox);
        parameters.add(graphic);
        parameters.add(statisticsArray);
        parameters.add(data);
        parameters.add(view.getInteger());
        pollingTimeValue=10;
        pollingTimeVisible=true;
      /*  if(parent instanceof StatisticsFrame){
            pollingTimeValue=((StatisticsFrame)parent).getPollingTimeValue();
            pollingTimeVisible=((StatisticsFrame)parent).isPollingTimeVisible();
        }
       */
        parameters.add(new Integer(pollingTimeValue));
        parameters.add(new Boolean(pollingTimeVisible));
        return parameters;
    }


    /** Sets the values with the valueStamps vector.
     * @param valueStamps Vector if valueStamps with the values of the variables
     * @see ValueStamp
     * @see MessengerClient#getValues
     * @see #getVariableIds
     */
    public void setValues(Vector valueStamps)
    {
        if(valueStamps==null)
            return;
        
        if(!type.equals(StatInfo.Type.NUMERIC)) {
            ValueStamp vs=(ValueStamp)valueStamps.elementAt(0);
            if(vs!=null)
                nextValue(vs.getTime(), vs.getValue(), id, index);
            return;
        }
        
/*        int i=0;
        for(Enumeration stsEnum=statisticsArray.elements();stsEnum.hasMoreElements();){
            StatisticsVector sv=(StatisticsVector)stsEnum.nextElement();
            ValueStamp vs=(ValueStamp)valueStamps.elementAt(i++);
            System.out.println(i + "/" + statisticsArray.size() + ":" + sv.getId() + "/" + valueStamps.size() + "=" + vs.getValue());
            if(vs!=null)
                nextValue(vs.getTime(), vs.getValue(), sv.getId(), sv.getIndex());
        }
 */
        for (int i=0;i<valueStamps.size();i++) {
            ValueStamp vs = (ValueStamp) valueStamps.get(i);
            nextValue(vs.getTime(),vs.getValue(),vs.getId(),-1);
            System.out.println(i + "="+ " " + vs.getTime() + ":" + vs.getValue() + ":" + vs.getId());
        }
    }
    
    /** Returns a String with the data to be saved.
     * @param module The name of the module
     * @param variable The name of the variable to be saved
     * @see #saveData
     */
    public String getValues(String module, String variable){
        for(int i=0;i<data.size();i++){
            DataVector dv=(DataVector)data.elementAt(i);
            if(dv.equals(module,variable))
                return dv.getAllData();
        }
        return "";
    }


    /** Saves the graphic into a file with the graphic format. A file chooser is shown
     * to choose the file to write it. The file must have the extension *.graph.<BR>
     * @See #getValues
     * @see GraphicPanel#toSave
     * @see HistoGraph#toSave
     * @see MobilePathGraph#toSave
     */
    public void saveData(){
        String userDir = System.getProperty("user.dir");     // find actual directory
        File file=null;
        
        javax.swing.JFileChooser fc=new javax.swing.JFileChooser(new java.io.File(userDir));        
        fc.addChoosableFileFilter(new ArrowsFilter());
        fc.setFileView(new ArrowsFileView());
        
        int returnVal=fc.showSaveDialog(this);
        if(returnVal==fc.APPROVE_OPTION){
            file=fc.getSelectedFile();
            if(!(Extension.getExtension(file).equalsIgnoreCase(Extension.GRAPH)))                
                file=Extension.setExtension(file,Extension.GRAPH);            
            
            try{
                PrintWriter out= new PrintWriter(
                    new BufferedWriter(
                        new FileWriter(file)));
                // prepares outputstream for introducing lines from URL direction
                
                out.println("type="+type);
                out.println(toString());
                
                if(!type.equals(StatInfo.Type.NUMERIC)) {
                    out.println(module);
                    out.println(variable);
                    out.println();
                    out.println("[data]");      
                    String data=getValues(module,variable);
                    out.println(data);
                    out.close();
                    return;
                }
                
                out.print(graphic.toSave());
                out.println("[data]");
                for(int i=0;i<graphic.getDataArray().size();i++){
                    out.println("[Variable]");
                    String var=graphic.getDataArray().elementAt(i).toString();
                    out.println(var);
                    StringParser sp=new StringParser(var,false);
                    String module=sp.getNextToken('-').trim();
                    String data=getValues(module,sp.getNextToken().trim());
                    out.println(data);                    
                }
                out.close();
            }
            catch(FileNotFoundException exc){ Trace.print("Error saving data: "+exc.toString(),Trace.ERRORS,"StatisticsGraphic",716); }
            catch(IOException exc){ Trace.print("Error saving data: "+exc.toString(),Trace.ERRORS,"StatisticsGraphic",717); }
        }           
    }
        
    /** Saves the configuration of the Graphic into the panels Agrupation file.
     * This method is used when a panels Agrupation file is saved.
     * @param out Writer with the pointer to write the graphic configuration.
     * @see StatisticsGraphic#StatisticsGraphic(java.awt.Frame,BufferedReader)
     */
    public void saveGraphic(PrintWriter out){                 
       
    }
    
    /** Method required in the GraphicInterface not implemented in this class. */
    public void addGraphic(String module, String variable) {}
    
    /** Returns this class as a component. */
    public java.awt.Component getComponent() {
        return this;
    }
    
    // ******** protected methods **********************************************
    
    /** Checks if all the variables of the StatisticsGraphic are activated. This
     * method is used in the construction of the Graphic from a panels Afrupation file.
     * So, if not all the variables are activated, this Graphic is disabled in the
     * GraphicAgrupation panel.
     * @see PanelsFrame#containsStatistic
     */ 
    protected boolean areAllActived(){
        StringParser sp=new StringParser();
        String statisticName;
        
        if(!type.equals(StatInfo.Type.NUMERIC)) {
            sp.setString(this.variable);
            if(sp.hasNextToken('('))
                statisticName=sp.getNextToken('(').trim();
            else
                statisticName=variable;
          /*  if(!PanelsFrame.containsStatistic(this.module,statisticName)){
                Trace.print("Graphic titled '" + getTitle() + "' is not active.",Trace.NORMALLEVEL,"StatisticsGraphic",782);
                disableGraphic();
                return false;
            }
           */
            return true;    
        }
        
        String module, variable;
        
        for(Enumeration variables=statisticsArray.keys();variables.hasMoreElements();){
            sp.setString(variables.nextElement().toString());
            module=sp.getNextToken('-').trim();
            variable=sp.getNextToken().trim();
            sp.setString(variable);
            if(sp.hasNextToken('('))
                statisticName=sp.getNextToken('(').trim();
            else
                statisticName=variable;
           /* if(!PanelsFrame.containsStatistic(module,statisticName)){
                Trace.print("Graphic titled '" + getTitle() + "' is not active.",Trace.NORMALLEVEL,"StatisticsGraphic",801);
                disableGraphic();
                return false;
            }
            */
        }
        return true;    
    }
    
    
    // ******** private methods ************************************************
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        setLayout(new java.awt.BorderLayout());

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
        });

    }//GEN-END:initComponents

     /** Shows the popup in the numeric graphs. 
     * This method is called when right button mouse is pressed.
     */
    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        int x= evt.getPoint().x;
        int y= evt.getPoint().y;

        if(evt.getModifiers()==MouseEvent.BUTTON3_MASK){
            if(popup.isEnabled())
                popup.show(this, x, y);
        }
    }//GEN-LAST:event_formMouseClicked
   
    /** Updates the valuePanel and the StatisticsPanel with the selected variable.
     * This method is called when the variableComboBox has changed.
     */
    void variableComboBoxActionPerformed(java.awt.event.ActionEvent evt){
        String v=variableComboBox.getSelectedItem().toString();
        statModifyPanel.setStatControl(getSelectedStatControl());

        if(statisticsArray.containsKey(v)){
            StatisticsVector sv=(StatisticsVector)statisticsArray.get(v);
            if(sv.size()>0){
                valuePanel.setCurrentValue(sv.lastElement().toString());        
                statistics.setNewValues(sv.getAverage(), sv.getDesviation(),
                sv.getMaximum(), sv.getMinimum());
            }
        }        
    }
    
    
    /** Adds the available panels into the addToPanel menu. 
     * @see PanelsFrame#getPanelsName
     */
    private void addItems(){
        /*
        addToPanel.removeAll();
        Vector items=UD.getPanels().getPanelsName();
        for(int i=0;i<items.size();i++){
             MenuItem item=new MenuItem(items.elementAt(i).toString());
             addToPanel.add(item);
             final int index=i;
             item.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    GraphicAgrupation ga=UD.getPanels().getPanel(index);
                    MovingStatisticsGraphic msg=new MovingStatisticsGraphic(UD.getPanels(),StatisticsGraphic.this,ga);                    
                    msg.setBounds(0,0,getWidth(),getHeight()+59);                    
                    ga.add(msg); 
                    msg.validate();
                    ga.validate();
                }               
                        
             });       
        }
         */
    
    }
       
    /** Process the data if the type of the GraphicPanel is DIFF or INT or if the scale is
     * logarithmic. */
    private double processData(double value,StatisticsVector sv){
        double processingValue=value;
                       
                
        if(graphic.type==graphic.DIFF_TYPE)
        {
            double last=0;
            if(sv.size()>0)
                last=sv.getLinealLast();
            processingValue-=last;
        }        
        
        if(graphic.type==graphic.INT_TYPE)
            processingValue+=sv.getLinealSum();
        
        if(graphic.scale==graphic.LOG_SCALE){
            processingValue=Math.abs(processingValue);
            if(processingValue<1E-50)
                processingValue=1E-50;
            processingValue=Math.log(processingValue)/Math.log(10);
        }
        
        return processingValue;
    }
          




    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

    /** Supports the methods to save the data. */
    public class DataVector {
    
        // ******** private variables **********************************************
    
        private String module;          // the name of the module
        private String variable;        // the name of the variable
        private Vector data = new Vector(); // a vector to save the temporal data
        
        
        // ******** constructor ****************************************************
    
        /** Creates a new DataVector. 
         * @param module String with the name of the module
         * @param variable String with the name of the variable
         */
        public DataVector(String module,String variable){
            this.module=module;
            this.variable=variable;            
        }   
        
        // ******** public methods *************************************************
    
        /** Clones the DataVector. This method is used when a copy of the
         * StatisticsGraphic is made.         
         * @return Returns a clon of the DataVector
         */
        public Object clone(){
            DataVector dv=new DataVector(module,variable);
            Vector d=getData();
            Vector v=new Vector(d.size());
            for(int i=0; i<d.size(); i++){
                if(d.elementAt(i) instanceof Double)
                    v.add(new Double(((Double)d.elementAt(i)).doubleValue()));
                if(d.elementAt(i) instanceof String)
                    v.add(new String((String)d.elementAt(i)));
            }
            dv.setData(v);
            return dv;
        }
        
        /** Returns true if the module and variable given are same as the private
         * module and variable variables.
         */
        public boolean equals(String module, String variable){
            if(module.equalsIgnoreCase(this.module) && variable.equalsIgnoreCase(this.variable))
                return true;
            return false;
        }
        
        /** Sets the data vector.
         * @param data The new vector with the data
         */
        public void setData(Vector data){
            this.data=data;
        }
        
        /** Adds a new point, this method is used for numeric Graphs.
         * If the size of the vector is higher than the limit number of values /10 ,
         * and the savings are allowed, the data is saved in a temporal file
         * and the graphic data is deleted. If the savings aren't allowed, the data
         * is lost. 
         * This savings could be enabled or disabled and also the limit number of values
         * for each graph; this options can be configured in the Arrows Options.
         * This savings permits non saturate the computer memory without loss the data.
         * @param milisec Double with the current milisecond
         * @param value Double with the current value of the variable         
         */
        public void addData(double milisec, double value){
            data.add(new Double(milisec));
            data.add(new Double(value));
/*            if(data.size()>Configurations.LIMIT_NUMBER*2){
                if(Configurations.SAVINGS_ENABLED)
                    saveData();
                resetData(module,variable);
            }*/
        }
        
        /** Adds a new point, this method is used for non numeric Graphs.
         * If the size of the vector is higher than the limit number of values,
         * and the savings are allowed, the data is saved in a temporal file
         * and the graphic data is deleted. If the savings aren't allowed, the data
         * is lost. 
         * This savings could be enabled or disabled and also the limit number of values
         * for each graph; this options can be configured in the Arrows Options.
         * This savings permits non saturate the computer memory without loss the data.
         * @param milisec Double with the current milisecond
         * @param value String with the value of the variable         
         */
        public void addData(double milisec, String value){
            data.add(new Double(milisec));
            data.add(new String(value));
            /*
            if(data.size()>Configurations.LIMIT_NUMBER/5){
                if(Configurations.SAVINGS_ENABLED)
                    saveData();
                if(type==MOBILE_PATH)
                    mpg.resetPath();
            }
             */
        }
        
        /** Returns the vector with the data. */
        public Vector getData(){            
            return data;
        }
        
        /** Returns a String with the data to be saved. This data is the available
         * data in the vector and all the data from the temporal files of this variable.
         * So, all the data till the creation od the StatisticsGraphic is write in this String.
         * This data will be saved in a file with the graph format.
         * @see #saveData
         */
        public String getAllData(){  
            String nextLine,s="";        
            
            File file = new File("Variables" + System.getProperty("file.separator") +
                    module + "-" + variable + graphID + ".tmp");
            if(file.exists()){
                try{
                    BufferedReader br = new BufferedReader(new java.io.FileReader(file));        
                    while((nextLine = br.readLine()) != null){
                        s += nextLine+"\n";
                    }
                    br.close();        
                }
                catch(IOException exc){
                    Trace.print("IOError Loading data from file: " + file.getPath(),Trace.ERRORS,"StatisticsGraphic",1685);
                }       
            }
            for(int i=0;i<data.size();i+=2)
                    s+=((Double)data.elementAt(i)).intValue() + " " + data.elementAt(i+1).toString() + "\n";
            
            return s;            
        }
        
        /** Returns a double array with the data of the vector. This method is only 
         * used in the numeric graphs.
         */
        public double[] getDoubleData(){
            double[] values=new double[data.size()];
            for(int i=0;i<data.size();i++)
                values[i]=((Double)data.elementAt(i)).doubleValue();
            return values;
        }

        
        // ******** private methods ************************************************
    
        /** Saves the data in a specific file into the 'Variables' directory. This file
         * is saved with the name: 'module-variable+graphID.tmp'
         */
        private void saveData(){
            String s="";
            
            try{
                // prepares outputstream for appending the data
                PrintWriter out= new PrintWriter(
                    new BufferedWriter(
                        new FileWriter("Variables" + System.getProperty("file.separator") +
                        module + "-" + variable + graphID + ".tmp", true)));
                        
                for(int i=0;i<data.size();i+=2)
                    s+=((Double)data.elementAt(i)).intValue() + " " + data.elementAt(i+1).toString() + "\n";
                
                data.removeAllElements();    
                out.print(s);
                out.close();                
            }
            catch (IOException e) {
                Trace.print("IOError Appending data to the file '" + module + "-" + variable + ".tmp'",
                Trace.ERRORS,"StatisticsGraphic",1728);
                data.removeAllElements();
            }
            
        }
    }
}
