/*
 * StatisticsVector.java
 *
 * Created on 11 de enero de 2002, 17:26
 */

package phalgui.Node.Stats;


import java.util.*;

/** This class extends a vector with the values of the numeric graphs.
 * Is used to get the averages, maximums, minimums, and desviation values
 * of the numerical variables of the Graphs.
 * @author  Santi Oliveras
 * @version 1.0  3/05/2002
 * @see StatisticsPanel
 * @see GraphicFrame
 * @see StatisticsGraphic
 */
public class StatisticsVector extends Vector {

    // ******** private variables **********************************************
    
    private double sum=0;       //The sum of the values
    private double sum2=0;      //The sum grade two of the values: sumatory(i=0..n)(Xn*Xn)
    private double max=0;       //The maximum value    
    private double min=0;       //The minimum value
    private double linealSum=0; //The lineal sum of the values
    private double linealLast=0; //The last value in lineal format
    private boolean first=true; //boolean used in some methods
    
    private int id=-1;       // used in statistics mode
    private int index=-1;    // used in statistics mode
    
    
    // ******** constructors ***************************************************
    
    /** Creates new StatisticsVector */
    public StatisticsVector() {
        super(1000);
    }
            
    /** Creates new StatisticsVector. Used in statistics mode.
     * @param id Integer with the id of the variable
     * @param index Integer with the index of the variable
     */
    public StatisticsVector(int id, int index) {
        super(1000);
        this.id=id;
        this.index=index;
    }

    
    // ******** public methods *************************************************
    
    /** Adds a new value to the vector.
     * @param value a double with the value to add
     */
    public void addValue(double value){
        add(new Double(value));
        sum+=value;
        sum2+=value*value;
        if(first){
            max=value;
            min=value;
            first=false;
        }            
        if(value>max)
            max=value;
        if(value<min)
            min=value;
    }
    
    /** Sets the linealLast value.
     * @param last A double with the lineal last value
     */
    public void setLinealLast(double last){
        linealLast=last;
        linealSum+=last;        
    }
    
    /** Gets the linealLast value. */
    public double getLinealLast(){
        return linealLast;
    }
    
    /** Gets the lineal sum. */
    public double getLinealSum(){
        return linealSum;
    }
    
    /** Gets the id. */
    public int getId(){
        return id;
    }
    
    /** Gets the index. */
    public int getIndex(){
        return index;
    }
    
    /** Deletes the last value of the vector. */
    public void deleteLast(){
        double last=((Double)lastElement()).doubleValue();
        remove(lastElement());
        if(last==max || last==min)
            reCalculate();        
        else{
            sum-=last;
            sum2-=last*last;
        }           
    }
    
    /** Resets the vector and its variables. */
    public void resetAll(){
        removeAllElements();
        max=0;
        min=0;
        sum=0;
        sum2=0;
        linealLast=0;
        linealSum=0;
        first=true;
    }
    
    /** Recalculates the variables with all the values of the vector. */
    public void reCalculate(){
        sum=0;
        sum2=0;
        double value;
        first=true;
        for(Enumeration valuesEnum=elements();valuesEnum.hasMoreElements();){
            value=((Double)valuesEnum.nextElement()).doubleValue();       
            sum+=value; 
            sum2+=value*value;        
            if(first){
                max=value;
                min=value;
                first=false;
            }  
            if(value>max)
                max=value;
            if(value<min)
                min=value;   
        }
    }
        
    /** Returns true if the id and index are the same as the private 
     * variablesid and index. */
    public boolean equals(int id, int index){
        if(this.id==id && this.index==index)
            return true;
        return false;   
    }
    
    /** Returns the variableId. An Integer if the variable is a simple variable,
     * or a IndexedStruct if a variable from a table. If the index is -1,it's a 
     * simple variable. */
    public Object getVariableId(){
        if(index==-1)
            return new Integer(id);
        return new IndexedStruct(id,index);    
    }
    
    /** Sets the values of the vector.
     * @param values Vector of doubles with the values
     */
    public void setData(Vector values){
        removeAllElements();
        addAll(values);        
        reCalculate();
    }
    
    /** Returns a double with the maximum. */
    public double getMaximum(){
        return Math.round(max*100d)/100d;
    }
    
    /** Returns a double with the minimum. */
    public double getMinimum(){
        return Math.round(min*100d)/100d;
    }
    
    /** Returns a double with the average. */
    public double getAverage(){
        return Math.round(sum/(double)size()*100d)/100d;
    }
    
    /** Returns a double with the desviation. */
    public double getDesviation(){
        double avg=sum/(double)size();
        double desv=Math.sqrt(sum2/(double)size()-avg*avg);
        return Math.round(desv*100d)/100d;
    }    
    
    /** Returns a double with the sum. */
    public double getSum(){
        return sum;
    }
}
