/*
 * VariableDataSet.java
 *
 * Created on 15 de enero de 2002, 0:17
 */

package phalgui.Node.Stats;


import phalgui.graph.*;
import java.awt.Color;
import phalgui.Utils.Arrows.Trace;

/** This class contains the DataSet of each variable of the GraphicPanel. And
 * supports the methods to manage the DataSet.
 * @author  Santi Oliveras
 * @version 1.0  3/05/2002
 * @see GraphicPanel
 * @see DataSet
 */
public class VariableDataSet {

    // ******** private variables **********************************************
    
    private DataSet data;           // The DataSet
    private String module, variable;    //The module and variable strings
    
    
    // ******** constructor ****************************************************
    
    /** Creates new VariableDataSet.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @param lineColor The line color of the variable
     * @see DataSet     
     */
    public VariableDataSet(String module, String variable,Color lineColor) {
        this.module=module;
        this.variable=variable;
        
        data=new DataSet();
        // modifies the default data befaviour
	// sets the color of the straight line segments
	data.linecolor = lineColor;
	// sets the linestyle to a solid line
	data.linestyle = 1;
	// sets the index of the marker to use at the data points
	data.marker = 7;
	// sets the markercolor
	data.markercolor = Color.blue;
    }
    
    
    // ******** public methods *************************************************
    
    /** Appends the data to the DataSet if module and variable are correct.
     * @param d Array of Doubles with the points to append
     * @param n Integer with the number of points to append
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see DataSet#append(double[],int)   
     */     
    public void append(double[] d, int n, String module, String variable){
        if(this.module.equalsIgnoreCase(module) & this.variable.equalsIgnoreCase(variable)){
            try
	    {
		// appends data to the data set
		data.append(d, n);
	    }
            catch (Exception e)
	    {
                Trace.print("Error appending data to module " + module +
                " in variable " + variable,Trace.ERRORS,"VariableDataSet",71); 
            }        
        }
    }
    
    /** Deletes the last point of the DataSet if module and variable are correct.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see DataSet#deleteLast   
     */ 
    public void deleteLast(String module, String variable){
        if(this.module.equalsIgnoreCase(module) & this.variable.equalsIgnoreCase(variable))
            data.deleteLast();    
    }
    
    /** Resets the data of the DataSet if module and variable are correct.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see DataSet#deleteData   
     */
    public void resetAll(String module, String variable){
        if(this.module.equalsIgnoreCase(module) & this.variable.equalsIgnoreCase(variable))
            data.deleteData();    
    }
    
    /** Returns true if the module and variable given are the same as the 
     * module and variable private variables of the object.
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     */     
    public boolean isVariable(String module, String variable){
        if(this.module.equalsIgnoreCase(module) & this.variable.equalsIgnoreCase(variable))
            return true;
        return false;
    }
    
    /** UpDates the data to the DataSet if module and variable are correct. First
     * deletes the data and then appends the values.
     * @param d Array of Doubles with the points to append
     * @param n Integer with the number of points to append
     * @param module String with the name of the module
     * @param variable String with the name of the variable
     * @see DataSet#deleteData
     * @see DataSet#append(double[],int)   
     */ 
    public void upDateAllData(double d[], int n, String module, String variable){
        if(this.module.equalsIgnoreCase(module) & this.variable.equalsIgnoreCase(variable)){
            try
	    {
                // deletes all data in the data set
                data.deleteData();
		// appends data to the data set
		data.append(d, n);
	    }
            catch (Exception e)
	    {
                Trace.print("Error upDating all data to module " + module +
                " in variable " + variable,Trace.ERRORS,"VariableDataSet",128);             
            }        
        }
    }
    
    /** Returns the DataSet. */
    public DataSet getDataSet(){
        return data;
    }    

    /** Returns a String with the format 'module - variable'. */
    public String toString(){
        return module +" - "+variable;
    }
    
    /** Returns a String with the name of the module. */
    public String getModule(){
        return module;
    }
    
    /** Returns a String with the name of the variable. */
    public String getVariable(){
        return variable;
    }
}
