/*
 * MovingStatisticsGraphic.java
 *
 * Created on 18 de febrero de 2002, 17:46
 */

package phalgui.Node.Stats;


import java.awt.Dimension;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import phalControl.StatControl.InvalidReportFormat;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.SwingWorker;
import phalControl.StatControl;
import phalControl.StatControl.ValueUpdateListener;
import phalControl.commandManager.StatInfo;
import phalgui.Node.Stats.ReportOptionsPanel.ReportOptionsPanelListener;
import phalgui.Node.Stats.StatisticsGraphic.View;
import phalgui.Utils.Arrows.MyBoolean;
import phalgui.Utils.Arrows.StringParser;
import phalgui.Utils.Arrows.Trace;
import phalgui.Utils.Arrows.ValueStamp;
import phalgui.Utils.PollingThread;

/**
 * This class extends the StatisticsGraphic whith moving properties and with a polling thread.
 * This moving properties are implemented to allow the movement of the graphics into the
 * GraphicAgrupation panel. And a polling panell is added to control the polling time of
 * thread. This Thread deals with the polling of the variables of the graphic.
 * @author  Santi Oliveras
 * @version 1.0  2/05/2002
 * @see GraphicAgrupation
 * @see PanelsFrame
 */
public class ViewStatisticsGraphic extends StatisticsGraphic implements StatModifyPanel.SetListener,ValueUpdateListener,ReportOptionsPanelListener {

    // ******** private variables **********************************************
        
    
    private javax.swing.border.TitledBorder title;  //A titled border with the name of the graphic

    public static void main(String[] args) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                JFrame f = new JFrame();
                StatControl s = new StatControl("hola");
                try {
                    s.setPeriodicTrigger(100, 10);
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
                }
                ViewStatisticsGraphic m = new ViewStatisticsGraphic(s,CaptureMode.MANUAL,null);
                s = new StatControl("adeu");
                try {
                    s.setPeriodicTrigger(200, 50);
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
                }
                m.addStat(s);
                f.add(m);
                f.setVisible(true);
                f.pack();
                f.addWindowListener(new java.awt.event.WindowAdapter() {
                    public void windowClosing(java.awt.event.WindowEvent e) {
                        System.exit(0);
                    }
                });
            }
        });
    }
    private JCheckBoxMenuItem triggerItem;
    private JMenu reportOptionsMenu;
    private JCheckBoxMenuItem infoItem;
    private JCheckBoxMenuItem windowItem;
    private JCheckBoxMenuItem modify;
    private JCheckBoxMenuItem histogram;
    private JCheckBoxMenuItem xy;
    private JCheckBoxMenuItem viewFFT;

    void setCaptureMode(CaptureMode mode) {
        Mode = mode;
        viewUpdate();
        modeUpdate();
        reloadAll();
    }

    public void SetButtonAction(String data) {
        ValueStamp[] vs = new ValueStamp[1];
        vs[0] = new ValueStamp(data,0);
        this.setData(this.getSelectedStatControl(), vs);
    }



    public enum CaptureMode {
        POLLING,MANUAL,REPORT;
    }

    public CaptureMode Mode;

    private int current_window;
    private JSlider pollingTimeSlider;
    private JPanel pollingPanel;
    private JTextField pollingTimeTextField;
    private PollingThread pt;

    protected JCheckBoxMenuItem number, text, numberGraph, simpleGraph, onlyGraph; //The items of the popup
    protected JMenuItem name, save;
    protected JMenu addToPanel;                //A menu of the available panels, added in the popup menu
    private JCheckBoxMenuItem scope;
    private JMenu captureMode;
    private JCheckBoxMenuItem report;
    private JCheckBoxMenuItem manual;
    
    // ******** constructors ***************************************************
        

    public ViewStatisticsGraphic(StatControl statControl, CaptureMode mode, View view) {
        super(statControl);
        this.view=view==null?View.NUMBERGRAPH:view;
        Mode = mode==null?Mode.MANUAL:mode;
        createComponents();
        createPollingSlider();
        statModifyPanel.setStatControl(statControl);

        viewUpdate();
        _addStat(statControl);
        reloadAll();
    }

    public ViewStatisticsGraphic(StatisticsGraphic sg, CaptureMode mode, View view) {
        super(sg);
        Mode = mode==null?Mode.MANUAL:mode;
        view=view==null?View.NUMBERGRAPH:view;
        createComponents(sg.getParameters());
        viewUpdate();
        modeUpdate();
        reloadAll();
        if(loadFailed)
            return;
    }
    
    public ViewStatisticsGraphic(BufferedReader br, CaptureMode mode) {
        super(br);
        Mode = mode==null?Mode.MANUAL:mode;
        createComponents(br);
        viewUpdate();
        modeUpdate();
        reloadAll();
        graphic.setOnlyVisiblePoints(true);
        if(loadFailed)
            return;
    }

    @Override
    public void addStat(StatControl statControl) {
        super.addStat(statControl);
        _addStat(statControl);
    }

    int nofstats=0;
    public void _addStat(StatControl statControl) {
        capturePanel.addStatControl(statControl);
        statControl.setValueUpdateListener(this);
        modeUpdate();
        nofstats++;
        if (nofstats==2) {
            xy.setEnabled(true);
        } else {
            xy.setEnabled(false);
        }
    }


    public void stop() {
        switch(Mode) {
            case REPORT:
                this.pauseReport();
                break;
            case POLLING:
                this.stopPollingThread();
                break;
            case MANUAL:
        }
    }

        
    
    // ******** public methods *************************************************

    
    /** Disables all the graphic.     
     * @see StatisticsGraphic#disableGraphic
     */
    @Override
    public void disableGraphic(){
        super.disableGraphic();
        setBorder(new javax.swing.border.TitledBorder(null, toString(), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), new java.awt.Color(153, 153, 153)));
        capturePanel.setEnabled(false);
    }


    
    // ******** protected methods *************************************************
    
    
    
    // ******** private methods ************************************************

    private void setWindow(int window) {
        this.current_window=window;
        graphic.setMaxVisiblePoints(current_window);
    }

    public void WindowModified(int window) {
        setWindow(window*getSelectedStatControl().getSize());
    }

    public void TriggerPeriodModified(int period) {

    }

    public void TriggerTypeModified(boolean isPeriod) {

    }

    
    public void GetButtonAction() {
        setStatsValues();
        System.out.println("Setting manual value");
    }

    private void createText() {
        java.awt.GridBagConstraints gridBagConstraints1=new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(10, 10, 10, 10);
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        valuePanel=new ValuePanel();
        add(valuePanel, gridBagConstraints1);
        simplePopupConfiguration();
        data.add(new DataVector(module,variable));
    }

    private void createDefault(GraphicPanel graphicPanel) {
        java.awt.GridBagConstraints gridBagConstraints1=new java.awt.GridBagConstraints();
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(10, 10, 0, 10);
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;
        gridBagConstraints1.weighty = 1.0;

        graphic=graphicPanel;
        add(graphic, gridBagConstraints1);

        sgt = new sgPlot(module.toUpperCase() + ": " + variable, new Dimension(400,300));
        sgdata = sgt.getDataModel();
        add(sgt, gridBagConstraints1);
        sgt.setSize(sgt.getSize());

        sgt.setVisible(false);

        histoPanel = new HistoPanel("");
        add(histoPanel,gridBagConstraints1);
        histoPanel.setSize(histoPanel.getSize());
        histoPanel.setVisible(false);

        xyPanel = new XYPanel(module.toUpperCase() + ": " + variable);
        add(xyPanel,gridBagConstraints1);
        xyPanel.setSize(xyPanel.getSize());
        xyPanel.setVisible(false);

        gridBagConstraints1.insets = new java.awt.Insets(10, 10, 10, 10);
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.weighty = 0.0;

        variableSelectPanel = new javax.swing.JPanel();
        jLabel = new javax.swing.JLabel();
        variableComboBox = new javax.swing.JComboBox();

        variableComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                variableComboBoxActionPerformed(evt);
            }
        });

        variableSelectPanel.setLayout(new java.awt.BorderLayout());
        jLabel.setText("Variable:  ");
        jLabel.setForeground(java.awt.Color.black);
        jLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        variableSelectPanel.add(jLabel, java.awt.BorderLayout.WEST);
        variableSelectPanel.add(variableComboBox, java.awt.BorderLayout.EAST);
        add(variableSelectPanel, gridBagConstraints1);

        gridBagConstraints1.insets = new java.awt.Insets(0, 10, 5, 10);
        gridBagConstraints1.gridy = 2;

        valuePanel=new ValuePanel();
        add(valuePanel, gridBagConstraints1);

        gridBagConstraints1.gridy = 3;

        statistics=new StatisticsPanel(false);
        add(statistics, gridBagConstraints1);

        gridBagConstraints1.gridy = 4;

        statModifyPanel=new StatModifyPanel();
        statModifyPanel.setSetListener(this);
        add(statModifyPanel, gridBagConstraints1);

        data.add(new DataVector(module,variable));
        popupConfiguration();

        gridBagConstraints1.gridy = 5;
        capturePanel = new ReportOptionsPanel();
        add(capturePanel, gridBagConstraints1);
        capturePanel.addReportOptionsPanelListener(this);
    }

    /** Sets the defaults. */
    protected void createComponents(){
        setLayout(new java.awt.GridBagLayout());

        switch(type) {
            case TEXT:
                createText();
                break;
            default:
                createDefault(new GraphicPanel(this,module,variable));
                String v = new String(module + " - " + variable);
                variableComboBox.addItem(v);
                statisticsArray.put(v,
                        new StatisticsVector(id, index));
                break;
        }

    }


    /** Sets the defaults reading the parameters given. So, it makes a copy of them.
     * @param parameters The original GraphicFrame parameters
     * @see #getParameters
     */
    private void createComponents(Vector parameters){
        type=StatInfo.Type.getFromInteger((Integer)parameters.elementAt(0));
        module=new String(parameters.elementAt(1).toString());
        variable=new String(parameters.elementAt(2).toString());
        id=((Integer)parameters.elementAt(3)).intValue();
        index=((Integer)parameters.elementAt(4)).intValue();
        pollingTimeValue=((Integer)parameters.elementAt(11)).intValue();
        pollingTimeVisible=((Boolean)parameters.elementAt(12)).booleanValue();
        java.awt.GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
        setLayout(new java.awt.GridBagLayout());

        switch(type) {
            case TEXT:
                createText();
                break;
            default:
                createDefault(new GraphicPanel(this,(GraphicPanel)parameters.elementAt(7)));
                Hashtable table=(Hashtable)parameters.elementAt(8);
                statisticsArray=new Hashtable(table.size());
                for(Enumeration enu=table.keys();enu.hasMoreElements();){
                    String key=new String(enu.nextElement().toString());
                    StatisticsVector sv=(StatisticsVector)table.get(key);
                    StatisticsVector svCopy=new StatisticsVector(sv.getId(),sv.getIndex());
                    svCopy.setData(sv);
                    statisticsArray.put(key,svCopy);
                }
                Vector vect=new Vector((Vector)parameters.elementAt(9));
                for(int i=0;i<vect.size();i++)
                    data.add(((DataVector)vect.elementAt(i)).clone());
                view=View.getFromInteger((Integer)parameters.elementAt(10));
                break;
        }
       
    }

    /** Sets the defaults reading from the panles Agrupation file.
     * Adds the panels according to its type with a GridBagLayout.
     * @param br Reader with the point of the file
     */
    private void createComponents(BufferedReader br){
        loadFailed=false;
        StringParser sp=new StringParser();
        String s;
        try{
            sp.setString(br.readLine());
            sp.getNextToken('=');
            type=StatInfo.Type.getFromInteger(Integer.parseInt(sp.getNextToken()));
            module=br.readLine();
            variable=br.readLine();
            sp.setString(br.readLine());
            id=Integer.parseInt(sp.getNextToken(' '));
            index=Integer.parseInt(sp.getNextToken());
            sp.setString(br.readLine());
            pollingTimeVisible=MyBoolean.getBoolean(sp.getNextToken(' '));
            pollingTimeValue=Integer.parseInt(sp.getNextToken());


            switch(type) {
                case TEXT:
                    createText();
                    break;
                default:
                    createDefault(new GraphicPanel(this,br));
                    int n=Integer.parseInt(br.readLine());
                    String key;
                    for(int i=0;i<n;i++){
                        key=br.readLine();
                        sp.setString(br.readLine());
                        int tid=Integer.parseInt(sp.getNextToken(' '));
                        int tindex=Integer.parseInt(sp.getNextToken());
                        statisticsArray.put(key,new StatisticsVector(tid, tindex));
                        variableComboBox.addItem(key);

                        sp.setString(key);
                        String mod=sp.getNextToken('-').trim();
                        String var=sp.getNextToken().trim();
                        data.add(new DataVector(mod,var));
                    }
                    view=View.getFromInteger(Integer.parseInt(br.readLine()));
                   
            }
        }
        catch(FileNotFoundException exc){
            Trace.print("Error loading graphic: "+exc.toString(),Trace.ERRORS,"StatisticsGraphic",1365);
            loadFailed=true;
        }
        catch(IOException exc){
            Trace.print("Error loading graphic: "+exc.toString(),Trace.ERRORS,"StatisticsGraphic",1369);
            loadFailed=true;
        }
        catch(NumberFormatException exc){
            Trace.print("Error loading graphic: "+exc.toString(),Trace.ERRORS,"StatisticsGraphic",1373);
            loadFailed=true;
        }

    }

    /** Configures the popup of the non numeric graphic. */
    private void simplePopupConfiguration(){

        name= new JMenuItem("Rename graph");
        save= new JMenuItem("Save graph");
        addToPanel =new JMenu("Add to pannel");

        popup.add(name);
        popup.addSeparator();
        popup.add(save);
        popup.addSeparator();
        popup.add(addToPanel);

        
        save.setEnabled(true/*Configurations.SAVINGS_ENABLED*/);

        name.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                javax.swing.JLabel message= new javax.swing.JLabel("Insert a new name:");
                message.setForeground(java.awt.Color.black);
                message.setFont(new java.awt.Font("Dialog", 0, 12));
                String p=javax.swing.JOptionPane.showInputDialog(ViewStatisticsGraphic.this,message,"Name Editor",javax.swing.JOptionPane.QUESTION_MESSAGE);

            }
        });

        save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveData();
            }
        });
    }
    
    private void popupSelectViewCheckbox(JCheckBoxMenuItem selected) {
        number.setState(false);
        text.setState(false);
        numberGraph.setState(false);
        simpleGraph.setState(false);
        onlyGraph.setState(false);
        scope.setState(false);
        histogram.setState(false);
        selected.setState(true);
    }

    private void popupSelectModeCheckbox(JCheckBoxMenuItem selected) {
        report.setState(false);
        pollingTime.setState(false);
        manual.setState(false);
        selected.setState(true);
    }

    /** Configures the popup of the numeric graphic. */
    void popupConfiguration(){

        number= new JCheckBoxMenuItem("NumberFieldGraph");
        text= new JCheckBoxMenuItem("TextFieldGraph");
        numberGraph= new JCheckBoxMenuItem("StatisticsChartGraph");
        simpleGraph= new JCheckBoxMenuItem("SimpleXYChartGraph");
        onlyGraph= new JCheckBoxMenuItem("OnlyXYChartGraph");
        scope= new JCheckBoxMenuItem("ScopeGraph (1 var only)");
        histogram= new JCheckBoxMenuItem("Histogram");
        xy= new JCheckBoxMenuItem("XYGraph");
        modify= new JCheckBoxMenuItem("Modify Stat");

        viewFFT= new JCheckBoxMenuItem("View FFT");
        captureMode=new JMenu("Capture Mode");
        pollingTime=new JCheckBoxMenuItem("Periodic Polling");
        report=new JCheckBoxMenuItem("File Report");
        manual=new JCheckBoxMenuItem("Manual");

        reportOptionsMenu=new JMenu("Report Options");
        infoItem=new JCheckBoxMenuItem("View Info");
        windowItem=new JCheckBoxMenuItem("Select Window");
        triggerItem=new JCheckBoxMenuItem("Select Trigger");

        name= new JMenuItem("Rename graph");
        save= new JMenuItem("Save graph");

        popup.add(number);
        popup.add(text);
        popup.add(numberGraph);
        popup.add(simpleGraph);
        popup.add(onlyGraph);
        popup.add(scope);
        popup.add(histogram);
        popup.add(xy);
        popup.add(modify);
        popup.addSeparator();

        popup.add(viewFFT);
        popup.add(captureMode);
        captureMode.add(pollingTime);
        captureMode.add(report);
        captureMode.add(manual);

        popup.add(reportOptionsMenu);
        reportOptionsMenu.setVisible(false);
        reportOptionsMenu.add(infoItem);
        reportOptionsMenu.add(windowItem);
        reportOptionsMenu.add(triggerItem);

        popup.addSeparator();
        popup.add(name);
        popup.addSeparator();
        popup.add(save);

        save.setEnabled(true /*Configurations.SAVINGS_ENABLED*/);

        number.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.NUMBER;
                viewUpdate();
            }
        });

        text.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.TEXT;
                viewUpdate();
            }
        });

        numberGraph.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.NUMBERGRAPH;
                viewUpdate();
            }
        });

        simpleGraph.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.SIMPLEGRAPH;
                viewUpdate();
            }
        });

        onlyGraph.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.ONLYGRAPH;
                viewUpdate();
                
            }
        });

        scope.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.SCOPE;
                viewUpdate();
            }
        });

        histogram.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.HISTOGRAM;
                viewUpdate();
            }
        });

        xy.setEnabled(false);
        xy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.XY;
                viewUpdate();
            }
        });

        modify.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                view=View.MODIFY;
                viewUpdate();
            }
        });
        viewFFT.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sgdata.viewFFT(viewFFT.getState());
            }
        });
        pollingTime.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setCaptureMode(CaptureMode.POLLING);
            }
        });
        report.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setCaptureMode(CaptureMode.REPORT);
            }
        });
        manual.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setCaptureMode(CaptureMode.MANUAL);
            }
        });

        infoItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                capturePanel.setInfoVisible(infoItem.getState());
            }
        });

        windowItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                capturePanel.setWindowVisible(windowItem.getState());
            }
        });
        
        triggerItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                capturePanel.setTriggerVisible(triggerItem.getState());
            }
        });
        triggerItem.setState(false);

        name.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                javax.swing.JLabel message= new javax.swing.JLabel("Insert a new name:");
                message.setForeground(java.awt.Color.black);
                message.setFont(new java.awt.Font("Dialog", 0, 12));
                String p=javax.swing.JOptionPane.showInputDialog(ViewStatisticsGraphic.this,message,"Name Editor",javax.swing.JOptionPane.QUESTION_MESSAGE);
            }
        });

        save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveData();
            }
        });

    }


    /** Updates the view depending on the private variable view. */
    public void viewUpdate(){
        switch(view) {
            case NUMBER:
                variableSelectPanel.setVisible(true);
                statistics.setVisible(true);
                valuePanel.setVisible(true);
                graphic.setVisible(false);
                sgt.setVisible(false);
                histoPanel.setVisible(false);
                xyPanel.setVisible(false);
                capturePanel.setVisible(false);
                statModifyPanel.setVisible(false);
                popupSelectViewCheckbox(number);
                viewFFT.setVisible(false);
                Mode = Mode.MANUAL;
                reportOptionsMenu.setEnabled(false);
                break;
            case MODIFY:
                variableSelectPanel.setVisible(true);
                statistics.setVisible(false);
                valuePanel.setVisible(true);
                graphic.setVisible(false);
                sgt.setVisible(false);
                xyPanel.setVisible(false);
                histoPanel.setVisible(false);
                capturePanel.setVisible(false);
                statModifyPanel.setVisible(true);
                popupSelectViewCheckbox(modify);
                viewFFT.setVisible(false);
                Mode = Mode.MANUAL;
                reportOptionsMenu.setEnabled(false);
                SetButtonAction(statModifyPanel.getValue());
                break;
            case TEXT:
                variableSelectPanel.setVisible(true);
                statistics.setVisible(false);
                valuePanel.setVisible(true);
                graphic.setVisible(false);
                sgt.setVisible(false);
                xyPanel.setVisible(false);
                histoPanel.setVisible(false);
                capturePanel.setVisible(false);
                statModifyPanel.setVisible(false);
                popupSelectViewCheckbox(text);
                viewFFT.setVisible(false);
                Mode = Mode.MANUAL;
                reportOptionsMenu.setEnabled(false);
                break;
            case NUMBERGRAPH:
                variableSelectPanel.setVisible(true);
                statistics.setVisible(true);
                valuePanel.setVisible(true);
                graphic.setVisible(true);
                sgt.setVisible(false);
                xyPanel.setVisible(false);
                histoPanel.setVisible(false);
                statModifyPanel.setVisible(false);
                viewFFT.setVisible(false);
                popupSelectViewCheckbox(numberGraph);
                reportOptionsMenu.setEnabled(true);
                break;
           case SIMPLEGRAPH:
                variableSelectPanel.setVisible(true);
                statistics.setVisible(false);
                valuePanel.setVisible(true);
                graphic.setVisible(true);
                sgt.setVisible(false);
                xyPanel.setVisible(false);
                histoPanel.setVisible(false);
                statModifyPanel.setVisible(false);
                viewFFT.setVisible(false);
                popupSelectViewCheckbox(simpleGraph);
                reportOptionsMenu.setEnabled(true);
                break;
            case ONLYGRAPH:
                variableSelectPanel.setVisible(true);
                statistics.setVisible(false);
                valuePanel.setVisible(false);
                graphic.setVisible(true);
                sgt.setVisible(false);
                xyPanel.setVisible(false);
                histoPanel.setVisible(false);
                statModifyPanel.setVisible(false);
                viewFFT.setVisible(false);
                popupSelectViewCheckbox(onlyGraph);
                reportOptionsMenu.setEnabled(true);
                break;
            case HISTOGRAM:
                variableSelectPanel.setVisible(false);
                statistics.setVisible(false);
                valuePanel.setVisible(false);
                graphic.setVisible(false);
                statModifyPanel.setVisible(false);
                xyPanel.setVisible(false);
                viewFFT.setVisible(false);
                histoPanel.setVisible(true);
                popupSelectViewCheckbox(histogram);
                reportOptionsMenu.setEnabled(true);
                break;
            case XY:
                variableSelectPanel.setVisible(false);
                statistics.setVisible(false);
                valuePanel.setVisible(false);
                graphic.setVisible(false);
                statModifyPanel.setVisible(false);
                xyPanel.setVisible(true);
                histoPanel.setVisible(false);
                viewFFT.setVisible(false);
                popupSelectViewCheckbox(xy);
                reportOptionsMenu.setEnabled(true);
                break;
            case SCOPE:
                variableSelectPanel.setVisible(false);
                statistics.setVisible(false);
                valuePanel.setVisible(false);
                graphic.setVisible(false);
                statModifyPanel.setVisible(false);
                sgt.setVisible(true);
                xyPanel.setVisible(false);
                viewFFT.setVisible(true);
                histoPanel.setVisible(false);
                popupSelectViewCheckbox(scope);
                reportOptionsMenu.setEnabled(true);
                break;
        }
        switch(Mode) {
            case REPORT:
                capturePanel.setTriggerVisible(false);
                capturePanel.setVisible(true);
                capturePanel.setAutoMode();
                pollingPanel.setVisible(false);
                reportOptionsMenu.setVisible(true);
                popupSelectModeCheckbox(report);
                break;
            case MANUAL:
                capturePanel.setVisible(!view.equals(View.MODIFY));
                capturePanel.setManualMode();
                pollingPanel.setVisible(false);
                reportOptionsMenu.setVisible(false);
                popupSelectModeCheckbox(manual);
                break;
            case POLLING:
                capturePanel.setVisible(false);
                pollingPanel.setVisible(true);
                reportOptionsMenu.setVisible(false);
                popupSelectModeCheckbox(pollingTime);
                break;
        }
    }


    private void restartPollingThread() {
        if (pt == null) {
            pt= new PollingThread(1000);
            pt.start();
            pt.addPollingThreadListener(new PollingThread.PollingThreadListener() {

                public void call() {
                    setStatsValues();
                }
            });
        } else {
            pt.startPolling();
        }
    }

    private void pausePollingThread() {
        if (pt != null) {
            pt.stopPolling();
        }
    }

    private void stopPollingThread() {
        if (pt != null) {
            pt.finish();
        }
    }



    private void restartReport() {
        for (int i=0;i<stats.size();i++) {
            if (!((StatControl) stats.get(i)).isReporting()) {
                try {
                    ((StatControl) stats.get(i)).startReport();
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
 
    }

    private void pauseReport() {
        for (int i=0;i<stats.size();i++) {
            if (((StatControl) stats.get(i)).isReporting()) {
                try {
                    ((StatControl) stats.get(i)).stopReport();
                } catch (ErrorInCommandException ex) {
                    Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
 
    }


    private void modeUpdate(){
        switch(Mode) {
            case REPORT:
                graphic.setOnlyVisiblePoints(true);
                setWindow(getSelectedStatControl().getWindow()*getSelectedStatControl().getSize());
                pausePollingThread();
                restartReport();
                break;
            case MANUAL:
                pauseReport();
                pausePollingThread();
                break;
            case POLLING:
                pauseReport();
                restartPollingThread();
                break;
        }
    }

    public void ValueUpdate(StatControl stat, XYValue value) {
        setData(stat, value);
    }

    private void setData(StatControl stat, XYValue xy) {
        if (view.equals(View.SCOPE)) {
            sgdata.setValue(xy);
        } else {
            for (int j = 0; j < xy.len; j++) {
                nextValue(xy.getX()[j], String.valueOf(xy.getY()[j]), stats.getId(stat), -1, false);
            }
            histoPanel.addData(xy.getY(),stat.getObjectName() + ":" + stat.toString());
            histoPanel.drawGraphic();

            xyPanel.addData((xy.getY()),stat.toString());
            xyPanel.drawGraphic();
            graphic.drawGraphic();
            /*if (graphic.isOnlyVisiblePoints()) {
                setWindow(xy.len);
            }*/
        }
    }

    private void setData(StatControl stat,ValueStamp[] vs) {
        double[] y = new double[vs.length];
        if (view.equals(View.SCOPE)) {
            double[] x = new double[vs.length];
            for (int j = 0; j < vs.length; j++) {
                x[j] = tstampTodouble((int) vs[j].getTime() + (int) (j/stat.getSize()), j % stat.getSize(), stat.getSize());
                y[j] = Double.valueOf(vs[j].getValue());
            }
            sgdata.setValue(new XYValue(x, y));
        } else {
            for (int j = 0; j < vs.length; j++) {
                nextValue(tstampTodouble((int) vs[j].getTime() + (int) (j/stat.getSize()), j % stat.getSize(), stat.getSize()), vs[j].getValue(), stats.getId(stat), -1, false);
                y[j] = Double.valueOf(vs[j].getValue());

            }
            histoPanel.addData(y,stat.getObjectName() + ":" + stat.toString());
            histoPanel.drawGraphic();
            xyPanel.addData(y,stat.toString());
            xyPanel.drawGraphic();
            graphic.drawGraphic();
            if (graphic.isOnlyVisiblePoints()) {
                setWindow(vs.length);
            }

        }
    }

    private double tstampTodouble(int tstamp, int idx, int size) {
        return (double) (tstamp + (double) idx / size);
    }

    private class setStatsThread extends SwingWorker<ValueStamp[],Void> {
        private StatControl stat;
        private int id;

        setStatsThread(StatControl stat, int id) {
            this.stat = stat;
            this.id=id;
        }
        @Override
        protected ValueStamp[] doInBackground() throws Exception {
            try {
                return stat.getValue(capturePanel.getWindow()!=1?capturePanel.getWindow():0);
             } catch (ErrorInCommandException ex) {
                Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
            } catch (InvalidReportFormat ex) {
                Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
            } catch (IOException ex) {
                Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
            }
            return null;
        }

        @Override
        protected void done() {
            try {
                ValueStamp[] vs = get();
                if (vs == null) {
                    return;
                }
                setData(stat,vs);
            } catch (InterruptedException ex) {
                Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
            } catch (ExecutionException ex) {
                Logger.getLogger(ViewStatisticsGraphic.class.getName()).log(Level.SEVERE, null, ex);
            }
        }        
    }

    public void setStatsValues() {
        for (int i=0;i<stats.size();i++) {
            new setStatsThread((StatControl) stats.get(i),stats.getId(i)).execute();
            if (view.equals(View.SCOPE)) {
                break;
            }
        }
    }


    private void pollingTimeSliderStateChanged(javax.swing.event.ChangeEvent evt) {
        pollingTimeValue=pollingTimeSlider.getValue();
        int time=pollingTimeSlider.getValue()*100;
        pollingTimeTextField.setText(time + " ms");
        if(pt!=null)
            pt.setPollingTime(time);
    }


    private void createPollingSlider() {
        pollingPanel = new javax.swing.JPanel();
        pollingTimeSlider = new javax.swing.JSlider();
        pollingTimeTextField = new javax.swing.JTextField();

        pollingPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;

        pollingPanel.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), "Polling Time", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 11), java.awt.Color.black));
        pollingTimeSlider.setMinorTickSpacing(1);
        pollingTimeSlider.setPaintTicks(true);
        pollingTimeSlider.setMinimum(1);
        pollingTimeSlider.setMajorTickSpacing(10);
        pollingTimeSlider.setValue(10);
        pollingTimeSlider.setPreferredSize(new java.awt.Dimension(36, 27));
        pollingTimeSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                pollingTimeSliderStateChanged(evt);
            }
        });

        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(0, 10, 5, 0);
        gridBagConstraints1.weightx = 1.0;
        pollingPanel.add(pollingTimeSlider, gridBagConstraints1);

        pollingTimeTextField.setEditable(false);
        pollingTimeTextField.setText("1000 ms");
        pollingTimeTextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        pollingTimeTextField.setDisabledTextColor(java.awt.Color.black);
        pollingTimeTextField.setPreferredSize(new java.awt.Dimension(80, 21));
        pollingTimeTextField.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        pollingTimeTextField.setEnabled(false);
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 10, 5, 10);
        pollingPanel.add(pollingTimeTextField, gridBagConstraints1);


        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.insets = new java.awt.Insets(0, 10, 5, 10);
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.weightx = 1.0;

        add(pollingPanel, gridBagConstraints1);

    }
}
