package phalgui.Node.Stats;



/* ===========================================================
 * JFreeChart : a free chart library for the Java(tm) platform
 * ===========================================================
 *
 * (C) Copyright 2000-2004, by Object Refinery Limited and Contributors.
 *
 * Project Info:  http://www.jfree.org/jfreechart/index.html
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 * -------------------------------
 * XYLineAndShapeRendererDemo.java
 * -------------------------------
 * (C) Copyright 2004, by Object Refinery Limited and Contributors.
 *
 * Original Author:  David Gilbert (for Object Refinery Limited);
 * Contributor(s):   -;
 *
 * $Id: XYLineAndShapeRendererDemo.java,v 1.1 2004/06/07 10:37:49 mungady Exp $
 *
 * Changes
 * -------
 * 07-Jun-2004 : Version 1 (DG);
 *
 */

import java.util.Vector;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * A simple demonstration of the {@link XYLineAndShapeRenderer} class.
 */
public class XYPanel extends JPanel {

    private JFreeChart chart;
    final XYSeriesCollection dataset = new XYSeriesCollection();
    private Vector categories = new Vector<Category>();

    /**
     * Constructs the demo application.
     *
     * @param title  the frame title.
     */
    public XYPanel(final String title) {
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));

        chart = ChartFactory.createXYLineChart(
            title,
            "Real",
            "Imag",
            dataset,
            PlotOrientation.VERTICAL,
            true,
            false,
            false
        );
        XYPlot plot = (XYPlot) chart.getPlot();
        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer(false,true);
        plot.setRenderer(renderer);
        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 300));
        add(chartPanel);
    }

    void addData(double y[], String category) {
        int n = categories.indexOf(new CategorySearch(category));
        Category c;
        if (n==-1) {
            c = new Category(category,y);
            categories.add(c);
            if ((categories.size() % 2)==0) {
                dataset.addSeries(c.serie);
            }
        } else {
            c = (Category) categories.get(n);
        }
        c.addData(y);

        if ((categories.size()%2)==0) {
            for (int i=0;i<categories.size();i+=2) {
                Category c1 = (Category) categories.get(i);
                Category c2 = (Category) categories.get(i+1);

                c1.serie.clear();
                c2.serie.clear();
                for (int j=0;j<c1.y.length && j<c2.y.length;j++) {
                    c2.serie.add(c1.y[j],c2.y[j]);
                }
            }

        }
    }


    void drawGraphic() {
        chart.getPlot().datasetChanged(new DatasetChangeEvent(this,dataset));
    }

    class CategorySearch {
        private final String name;
        CategorySearch(String name) {
            this.name = name;
        }
        @Override
        public boolean equals(Object x) {
            if (x instanceof Category) {
                return ((Category) x).name.equals(name);
            } else {
                return false;
            }
        }
    }

    class Category {
        private final String name;
        double y[];
        XYSeries serie;
        Category(String name, double y[]) {
            this.name = name;
            this.y = y;
            this.serie = new XYSeries(name);
        }
        void addData(double newdata[]) {
            double x[];
            x = new double[y.length + newdata.length];
            for (int i=0;i<y.length;i++) {
                x[i]=y[i];
            }
            for (int i=y.length;i<y.length+newdata.length;i++) {
                x[i]=newdata[i-y.length];
            }
            y=x;
        }
        void clearData() {
            y = new double[0];
        }
    }
}

