/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.Waveform;

/* ===========================================================
 * JFreeChart : a free chart library for the Java(tm) platform
 * ===========================================================
 *
 * (C) Copyright 2000-2004, by Object Refinery Limited and Contributors.
 *
 * Project Info:  http://www.jfree.org/jfreechart/index.html
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 * -------------------------
 * StackedBarChartDemo2.java
 * -------------------------
 * (C) Copyright 2002-2004, by Object Refinery Limited and Contributors.
 *
 * Original Author:  David Gilbert (for Object Refinery Limited);
 * Contributor(s):   -;
 *
 * $Id: StackedBarChartDemo2.java,v 1.11 2004/04/26 19:12:03 taqua Exp $
 *
 * Changes
 * -------
 * 07-Nov-2002 : Version 1 (DG);
 * 13-May-2003 : Renamed StackedHorizontalBarChartDemo --> StackedBarChartDemo2 (DG);
 *
 */

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JToggleButton;
import phalControl.PlatformControl.WaveformNotFoundException;
import phalControl.WaveformControl;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.StackedBarRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DatasetChangeEvent;
import phalControl.commandManager.ObjectInfo;
import phalgui.Control.MainControl.PlatformNotConnected;
import phalgui.PhalguiApp;
import phalgui.Utils.DnD.TransfResizePanel;
import phalgui.Utils.PollingThread;

/**
 * A simple demonstration application showing how to create a stacked bar chart
 * using data from a {@link CategoryDataset}.  This demo also has item labels displayed.
 *
 */
public class WaveformDiagram extends TransfResizePanel {
    private final WaveformControl waveform;
    ValueAxis axis;
    int maxperiod=-9999;
    JCheckBox showLegendCheckBox = new JCheckBox("Legend");
    JToggleButton refreshButton = new JToggleButton("Auto-Refresh");
    final CategoryDataset dataset;
    boolean refreshing;
    boolean showLegend = false;
    PollingThread pt;
    String title,waveformName;
    LegendTitle legend;
    JLabel missedFrames = new JLabel();
    int resetCounter,resetTstamp, totalfaults, lastTS;
    JButton missedResetButton = new JButton("Reset");
    
    /**
     * Creates a new demo.
     *
     * @param title  the frame title.
     */
    public WaveformDiagram(String title, String waveformName) throws PlatformNotConnected, WaveformNotFoundException {
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setLayout(new java.awt.GridBagLayout());
        this.waveform = ((PhalguiApp) PhalguiApp.getApplication()).getMainControl().getPlatform().getWaveform(waveformName);
        this.title = title;
        this.waveformName = waveformName;

        dataset = createDataset();
        final JFreeChart chart = createChart(dataset,"");
        final ChartPanel chartPanel = new ChartPanel(chart);

        legend = chart.getLegend();
        pt = new PollingThread(1000);
        pt.addPollingThreadListener(new PollingThread.PollingThreadListener() {

            public void call() {
                updateDataset(dataset);
                chart.getPlot().datasetChanged(new DatasetChangeEvent(this,dataset));
                axis.setLowerBound(0);
                axis.setUpperBound(maxperiod);
            }
        });
        pt.start();
        pt.stopPolling();
        refreshing=false;

        JButton refresh = new JButton("Refresh");
        refresh.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                updateDataset(dataset);
                chart.getPlot().datasetChanged(new DatasetChangeEvent(this,dataset));
                axis.setLowerBound(0);
                axis.setUpperBound(maxperiod);
            }
        });

        add(refresh,new java.awt.GridBagConstraints());


        add(refreshButton,new java.awt.GridBagConstraints());
        refreshButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (!refreshing) {
                    pt.startPolling();
                    refreshing=true;
                } else {
                    refreshing=false;
                    pt.stopPolling();
                }
            }
        });
        showLegendCheckBox.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                if (showLegend) {
                    showLegend=false;
                    chart.removeLegend();
                } else {
                    showLegend=true;
                    chart.addLegend(legend);
                }
            }
        });
        add(showLegendCheckBox,new java.awt.GridBagConstraints());
        if (showLegend) {
            chart.addLegend(legend);
        } else {
            chart.removeLegend();
        }

        add(missedFrames,new java.awt.GridBagConstraints());
        add(missedResetButton, new java.awt.GridBagConstraints());
        missedResetButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                resetCounter+=totalfaults;
                resetTstamp=lastTS;
                updateDataset(dataset);
                chart.getPlot().datasetChanged(new DatasetChangeEvent(this,dataset));
                axis.setLowerBound(0);
                axis.setUpperBound(maxperiod);
            }
        });
        
        GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.weighty = 1;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        add(chartPanel,gridBagConstraints);

        axis.setLowerBound(0);
        axis.setUpperBound(maxperiod);
    }

    public String toString() {
        return "diagram:" + waveformName;
    }

    public WaveformDiagram(String[] info) throws PlatformNotConnected, WaveformNotFoundException {
        this(info[0],info[1]);
    }

    /**
     * Creates a sample dataset.
     *
     * @return a sample dataset.
     */
    private CategoryDataset createDataset() {
        DefaultCategoryDataset result = new DefaultCategoryDataset();
        updateDataset(result);
        return result;
    }

    public void updateDataset(CategoryDataset result) {
        try {
            waveform.refresh();
            totalfaults=0;
            ObjectInfo[] execs = waveform.getExecInfo();
            for (int i = execs.length - 1; i >= 0; i--) {
                ((DefaultCategoryDataset) result).addValue(execs[i].cpu_usec, execs[i].toString(), execs[i].getPEString());
                if (execs[i].period > maxperiod) {
                    maxperiod = execs[i].period;
                }
                totalfaults+=execs[i].nof_rtfaults;
            }
            totalfaults-=resetCounter;
            lastTS = execs[0].tstamp;
            DecimalFormat df = new DecimalFormat("0.###E0");
            missedFrames.setText("      " + totalfaults + " faults, " + df.format((double) totalfaults/lastTS) + " faults/Tslot    " );

        } catch (ErrorInCommandException ex) {
            Logger.getLogger(WaveformDiagram.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Creates a sample chart.
     *
     * @param dataset  the dataset.
     *
     * @return a sample chart.
     */
    private JFreeChart createChart(final CategoryDataset dataset, String title) {

        final JFreeChart chart = ChartFactory.createStackedBarChart(
            title,
            "Processor",                  // domain axis label
            "Time (usec)",                     // range axis label
            dataset,                     // data
            PlotOrientation.HORIZONTAL,  // the plot orientation
            true,                        // include legend
            true,                        // tooltips
            false                        // urls
        );

        final CategoryPlot plot = (CategoryPlot) chart.getPlot();
        final StackedBarRenderer renderer = (StackedBarRenderer) plot.getRenderer();
        axis = plot.getRangeAxis();
        return chart;
    }

    @Override
    public String[] getTransferData() {
        String[] info = new String[2];
        info[0] = title;
        info[1] = waveformName;
        return info;
    }

    @Override
    public void Close() {

    }

    @Override
    public void refreshPanel() {
        updateDataset(dataset);
    }

}