/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Node.Waveform;

import java.awt.Component;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import phalControl.PlatformControl.WaveformNotFoundException;
import phalControl.commandManager.ObjectInfo;
import phalControl.commandManager.commandManager.ErrorInCommandException;
import phalgui.Control.MainControl.PlatformNotConnected;
import phalgui.Node.Repositories.RepositoryObject;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Properties;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.SwingWorker;
import javax.swing.event.TreeSelectionEvent;
import phalControl.StatControl;
import phalControl.WaveformControl.Status;
import phalgui.Control.MainControl;
import phalControl.commandManager.StatInfo;
import phalgui.Node.Executable.ExecutableEntry;
import phalgui.Node.Repositories.RepositoryObject.FileType;
import phalgui.PhalguiView;
import phalgui.Node.NodeObject;
import phalgui.Control.StatusPanel;
import phalgui.Node.Executable.ExecutableGraphic;
import phalgui.Node.Repositories.WaveformRepository;
import phalgui.Tree.TreeMenuItem;
import phalgui.Tree.TreePopup;
import phalgui.Utils.Graphics.GraphicsObject;
import phalgui.Utils.ReadWriteTextFile;

/**
 *
 * @author root
 */
public class WaveformEntry extends NodeObject  {

    File file;
    
    MainControl main;
    String name;
    ImageIcon loadedIcon = new ImageIcon(PhalguiView.class.getResource("resources/tree.gif"));
    String base_path;
    private Object view;
    private StatusPanel statusPanel;
    private String[] objects;
    private WaveformParser WaveformParser;
    private Object diagram;
    private Properties config=null;

    public WaveformEntry(MainControl main, StatusPanel statusPanel,
            String name, File file)  throws IOException  {
        this(main, statusPanel, name, file, null);
    }

    public WaveformEntry(MainControl main, StatusPanel statusPanel,
            String name, File file, ImageIcon icon) throws IOException  {
        super(name, icon);
        this.statusPanel = statusPanel;
        this.name = name;
        this.main = main;
        this.file = file;  
        if (!this.file.exists()) {
            this.file.createNewFile();
        }
        this.base_path = file.getPath().substring(0, file.getPath().lastIndexOf(FileType.WAVEFORMS.subdir));
        setPopup(new TreePopup(buildMenu()));

        objects = ScanObjects();
        for (int i=0;i<objects.length;i++) {
            ExecutableEntry entry = new ExecutableEntry(main, objects[i], base_path, this);
            add(entry);
            entry.Refresh();
        }
    }

    public String[] getObjects() {
        return objects;
    }

    public TreeMenuItem[] buildMenu() {
        TreeMenuItem items[] = new TreeMenuItem[8];
        items[0] = new TreeMenuItem("RT Info") {
            public void action() { ViewInfo(); }
            public boolean actionEnabled() { return isLoaded(); }
        };
        items[1] = new TreeMenuItem("RT Diagram") {
            public void action() { ViewDiagram(); }
            public boolean actionEnabled() { return isLoaded(); }
        };
        items[2] = new TreeMenuItem("Edit") {
            public void action() { Edit(); }
            public boolean actionEnabled() { return false; }
        };
        items[3] = new TreeMenuItem("View") {
            public void action() { View(); }
            public boolean actionEnabled() { return isLoaded(); }
        };
        items[4] = new TreeMenuItem("Remove") {
            public void action() { Remove(); }
            public boolean actionEnabled() { return true; }
        };
        items[5] = new TreeMenuItem("Refresh") {
            public void action() {Reload();}
            public boolean actionEnabled() { return true; }
        };
        items[6] = new TreeMenuItem("Rename") {
            public void action() { Rename(); }
            public boolean actionEnabled() { return isEditEnabled(); }
        };
        items[7] = new TreeMenuItem("Test load & init") {
            public void action() { Test(); }
            public boolean actionEnabled() { return !isLoaded(); }
        };

        return items;
    }

      private void Test() {
        int cnt = 0;
        try {
            while (cnt < 500) {
                main.getPlatform().loadWaveform(name);
                main.getPlatform().getWaveform(name).setStatus(Status.INIT);
                main.getPlatform().getWaveform(name).setStatus(Status.RUN);
                Thread.sleep(1000);
                main.getPlatform().stopWaveform(name);
                main.Log("Testing " + cnt);
                cnt++;
            }
        } catch (InterruptedException ex) {
            Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
        } catch (PlatformNotConnected ex) {
            Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
        } catch (WaveformNotFoundException ex) {
            Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ErrorInCommandException ex) {
            Logger.getLogger(StatusPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
        main.Log("loaded/stoped " + cnt + " times correctly");
    }

    private void Reload() {
        SwingWorker w = new SwingWorker<Void,Void>() {

            @Override
            protected Void doInBackground() throws Exception {
                try {
                    try {
                        try {
                            main.getPlatform().getWaveform(name).refresh();
                        } catch (ErrorInCommandException ex) {
                            Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    } catch (WaveformNotFoundException ex) {
                        Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } catch (PlatformNotConnected ex) {
                    Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
                }
                return null;
            }  
            @Override
            protected void done() {
                Refresh();
            }
        };
        w.execute();
    }

    @Override
    public void DoubleClick(MouseEvent evt) {
        Edit();
    }

    public String getReposPath() {
        return base_path;
    }

    public boolean isEditEnabled() {
        if (app_editor != null) 
            return !app_editor.isShowing();
        return true;
    }

    @Override
    public String toString() {
        return name;
    }

    public void ViewInfo()  {
       if (view == null) {
            try {
                view = new WaveformInfoView("RT info for: " + name, main.getPlatform().getWaveform(name));
            } catch (PlatformNotConnected ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            } catch (WaveformNotFoundException ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        if (!main.getTabPane().hasComponent((Component) view)) {
            main.getTabPane().addTab(name + " Info", (Component) view, this);
        }
        main.getTabPane().setSelectedComponent((Component) view);
    }

    public void ViewDiagram()  {
       //if (diagram == null) {
            try {
                try {
                    diagram = new WaveformDiagram("Scheduling Diagram", name);
                } catch (WaveformNotFoundException ex) {
                    Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
                }
            } catch (PlatformNotConnected ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            }
       // }

        if (!main.getTabPane().hasComponent((Component) diagram)) {
            main.getTabPane().addTab("Scheduling Diagram", (Component) diagram, this);
        }
        main.getTabPane().setSelectedComponent((Component) diagram);
    }

    WaveformEdit app_editor;
    public void Edit() {
        if (app_editor == null) {
            try {
                app_editor = new WaveformEdit(main, this);
                WaveformParser = new WaveformParser(file);
                app_editor.readExecs(WaveformParser.parse());
            } catch (PlatformNotConnected ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            } catch (WaveformNotFoundException ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            } catch (ErrorInCommandException ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        if (!main.getTabPane().hasComponent(app_editor)) {
            main.getTabPane().addTab("Waveform " + name, app_editor, this);
        }
        main.getTabPane().setSelectedComponent(app_editor);
        
    }

    WaveformView app_viewer;
    public void View() {
        if (app_viewer == null) {
            try {
                app_viewer = new WaveformView(main, name);
                main.getPlatform().refresh();
                app_viewer.readExecs(main.getPlatform().getWaveform(name).getExecInfo());
                if (!main.getTabPane().hasComponent(app_viewer)) {
                    main.getTabPane().addTab("Waveform " + name, app_viewer, this);
                }
                main.getTabPane().setSelectedComponent(app_viewer);
                config = ((WaveformRepository) this.getParent()).getConfig();
                if (config!=null) {
                    GraphicsObject obj[] = (GraphicsObject[]) app_viewer.getObjects();
                    for (int i=0;i<obj.length;i++) {
                        String sx = config.getProperty("waveformview." + toString() + "." + obj[i].toString() + ".x");
                        String sy = config.getProperty("waveformview." + toString() + "." + obj[i].toString() + ".y");
                        if (sx!=null && sy!=null) {
                            obj[i].setPos(Double.valueOf(sx),Double.valueOf(sy));
                        }
                    }
                }
            } catch (PlatformNotConnected ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            } catch (WaveformNotFoundException ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            } catch (ErrorInCommandException ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            if (!main.getTabPane().hasComponent(app_viewer)) {
                main.getTabPane().addTab("Waveform " + name, app_viewer, this);
            }
        }
    }
/*
    @Override
    public void Save() {  }

    @Override
    public boolean isSaved() {
        if (app_editor!=null)
            return app_editor.getText().equals(ReadWriteTextFile.getContents(file));
        else
            return true;
    }
*/
    @Override
    public void Remove() {
        if (JOptionPane.showConfirmDialog(main.getFrame(), "Are you sure you want to delete waveform " + toString() +  "?")
                == JOptionPane.YES_OPTION) {
            if (!file.delete()) {
                JOptionPane.showMessageDialog(main.getFrame(), "Could not delete file");
            } else {
                removeSelf();
            }
        }
    }

    @Override
    public void Selected(TreeSelectionEvent e) { 
        statusPanel.selectWaveform(this);
    }

    private boolean isLoaded() {
        try {
            return main.getPlatform().isWaveformLoaded(name);
        } catch (PlatformNotConnected ex) {
            return false;
        }
    }

   
    @Override
    public ImageIcon getIcon() {
        if (isLoaded()) {
            return loadedIcon;
        }
        return super.getIcon();
    }

    @Override
    public void Rename() {
        String s = JOptionPane.showInputDialog("Enter new name for Waveform:");
        
        String path = file.getPath().substring(0,file.getPath().lastIndexOf(File.separatorChar));
        try {
            if (!file.renameTo(new File(path + File.separator + s + RepositoryObject.FileType.WAVEFORMS.Filter))) {
                JOptionPane.showMessageDialog(main.getFrame(), "Error renaming file.", "Rename", JOptionPane.ERROR_MESSAGE);
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(main.getFrame(), "Error renaming file: " + ex.getMessage(), "Rename", JOptionPane.ERROR_MESSAGE);
        }
        name = s;
        
        Redraw();
         
    }


    public static final String objname_field = "obj_name=";
    private String[] ScanObjects() {
        ArrayList objectslist = new ArrayList();
        String appdesc = ReadWriteTextFile.getContents(file);
        String obj;
        int objidx1,objidx2;
        if (appdesc == null)
            return null;
        while((objidx1 = appdesc.indexOf(objname_field)) != -1) {
            objidx1 += objname_field.length();
            appdesc = appdesc.substring(objidx1);
            if ((objidx2 = appdesc.indexOf("\n")) == -1)
                return null;
            obj = appdesc.substring(0,objidx2);
            objectslist.add(obj);
            appdesc = appdesc.substring(objidx2);
        }
        String[] objectsarray = new String[objectslist.size()];
        for (int i=0;i<objectslist.size();i++) {
            objectsarray[i] = ((String) objectslist.get(i)).replace("\r", "");
        }
        return objectsarray;
    
    }


     @Override
    public Transferable createTransferable(JComponent c) {
         if (diagram == null) {
            try {
                try {
                    diagram = new WaveformDiagram("Scheduling for " + name, name);
                } catch (WaveformNotFoundException ex) {
                    Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
                }
            } catch (PlatformNotConnected ex) {
                Logger.getLogger(WaveformEntry.class.getName()).log(Level.SEVERE, null, ex);
            }
         }

        return (Transferable) diagram;
    }


       @Override
    public boolean WriteConfig(Properties config) {
       if (app_viewer==null) {
           return true;
       }
       GraphicsObject obj[] = (GraphicsObject[]) app_viewer.getObjects();
       for (int i=0;i<obj.length;i++) {
           config.setProperty("waveformview." + toString() + "." + obj[i].toString() + ".x",String.valueOf(obj[i].getX()));
           config.setProperty("waveformview." + toString() + "." + obj[i].toString() + ".y",String.valueOf(obj[i].getY()));
       }
       return true;
    }

}
