/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * WaveformInfoView.java
 *
 * Created on 26-nov-2008, 14:18:34
 */

package phalgui.Node.Waveform;

import java.text.DecimalFormat;
import phalControl.WaveformControl;
import phalControl.commandManager.ObjectInfo;
import phalgui.Utils.NormalTableModel;

/**
 *
 * @author Usuario
 */
public class WaveformInfoView extends javax.swing.JPanel {

    NormalTableModel model;
    private WaveformControl waveform;
    Integer totalfaults=0;

    String rowsTotal[] = {"", "MEAN", "MAX"};
    private Integer resetCounter=0,resetTstamp=0,lastTS=0;
    
    /** Creates new form WaveformInfoView */
    public WaveformInfoView(String title, WaveformControl waveform) {
        initComponents();
        model = new NormalTableModel(ObjectInfo.listColumns());
        table.setModel(model);
        Refresh();
        this.title.setText(title);
        this.waveform = waveform;

        totalUsLabel.setText("Total CPU: N/A");
        totalMacLabel.setText("Total MAC/tslot: N/A");
        totalMacSecLabel.setText("Total MAC/sec: N/A");
        totalRelLabel.setText("Utilization: N/A");
        totalFaults.setText("Total Faults: N/A");
        totalFaultsFreq.setText("Faults Freq: N/A");
        clearFaults.setText("Clear Faults Counter");
    }

    public void Refresh() {
        clearTable();
        ObjectInfo[] execs;
        try {
            waveform.refresh();
            execs = waveform.getExecInfo();
        } catch (Exception ex) {
            System.out.println("Error updating execs table: " + ex.getMessage());
            return;
        }
        if (execs == null)
            return;

        Integer totalmeanus=0, totalmeanmacs=0,totalmaxus=0,totalmaxmacs=0;
        Integer totalmeanmacsec=0,totalmaxmacsec=0;
        Double totalmeanrel=0.0,totalmaxrel=0.0,totalfaultsFreq=0.0;
        totalfaults=0;
        for (int i=0; i<execs.length; i++) {
            addProcess(execs[i].toStrings());
            totalmeanus+=execs[i].cpu_usec;
            totalmaxus+=execs[i].max_usec;
            totalmeanmacs+=execs[i].mean_mac;
            totalmaxmacs+=execs[i].max_mac;
            totalmeanmacsec+=(int) (1000000*((double) (execs[i].mean_mac.doubleValue()/execs[i].period.doubleValue())));
            totalmaxmacsec+=(int) (1000000*((double) (execs[i].max_mac.doubleValue()/execs[i].period.doubleValue())));
            totalmeanrel+=execs[i].cpu_usec.doubleValue()*100/execs[i].period.doubleValue();
            totalmaxrel+=execs[i].max_usec.doubleValue()*100/execs[i].period.doubleValue();
            totalfaults+=execs[i].nof_rtfaults;
        }
        totalfaults-=resetCounter;
        lastTS=execs[0].tstamp;
        totalfaultsFreq=totalfaults.doubleValue()/(execs[0].tstamp.doubleValue()-resetTstamp);
        DecimalFormat df = new DecimalFormat("00.00");
        totalUsLabel.setText("Total CPU: " + totalmeanus.toString() + " us mean, " + totalmaxus.toString() + " us max");
        totalMacLabel.setText("Total MAC/tslot: " + totalmeanmacs.toString() + " mean, " + totalmaxmacs.toString() + " max");
        totalMacSecLabel.setText("Total MAC/sec: " + totalmeanmacsec.toString() + " mean, " + totalmaxmacsec.toString() + " max");
        totalRelLabel.setText("Utilization: " + df.format(totalmeanrel) + "% mean, " + df.format(totalmaxrel) + "% max");
        totalFaults.setText("Total Faults: " + totalfaults);
        df = new DecimalFormat("0.000000");
        totalFaultsFreq.setText("Faults Freq: " + df.format(totalfaultsFreq) + " faults/ts");
        table.updateUI();
    }


    void addProcess(Object row[]) {
        model.addRow(row);
    }

    void clearTable() {
        model.clearRows();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jLabel2 = new javax.swing.JLabel();
        title = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        table = new javax.swing.JTable();
        Refresh = new javax.swing.JButton();
        Clear = new javax.swing.JButton();
        totalUsLabel = new javax.swing.JLabel();
        totalMacLabel = new javax.swing.JLabel();
        totalRelLabel = new javax.swing.JLabel();
        totalMacSecLabel = new javax.swing.JLabel();
        totalFaults = new javax.swing.JLabel();
        totalFaultsFreq = new javax.swing.JLabel();
        clearFaults = new javax.swing.JButton();

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(phalgui.PhalguiApp.class).getContext().getResourceMap(WaveformInfoView.class);
        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        setName("Form"); // NOI18N
        setLayout(new java.awt.GridBagLayout());

        title.setFont(resourceMap.getFont("title.font")); // NOI18N
        title.setText(resourceMap.getString("title.text")); // NOI18N
        title.setName("title"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 20, 0);
        add(title, gridBagConstraints);

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        table.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null}
            },
            new String [] {
                "Process Name", "PID", "Status", "Tslot", "CPU (usec)", "CPU (%)"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, true, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        table.setColumnSelectionAllowed(true);
        table.setName("table"); // NOI18N
        table.setRowHeight(30);
        table.getTableHeader().setReorderingAllowed(false);
        jScrollPane1.setViewportView(table);
        table.getColumnModel().getSelectionModel().setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        table.getColumnModel().getColumn(0).setHeaderValue(resourceMap.getString("table.columnModel.title0")); // NOI18N
        table.getColumnModel().getColumn(1).setPreferredWidth(50);
        table.getColumnModel().getColumn(1).setHeaderValue(resourceMap.getString("table.columnModel.title1")); // NOI18N
        table.getColumnModel().getColumn(2).setPreferredWidth(50);
        table.getColumnModel().getColumn(2).setHeaderValue(resourceMap.getString("table.columnModel.title2")); // NOI18N
        table.getColumnModel().getColumn(3).setPreferredWidth(50);
        table.getColumnModel().getColumn(3).setHeaderValue(resourceMap.getString("table.columnModel.title4")); // NOI18N
        table.getColumnModel().getColumn(4).setHeaderValue(resourceMap.getString("table.columnModel.title3")); // NOI18N
        table.getColumnModel().getColumn(5).setHeaderValue(resourceMap.getString("table.columnModel.title5")); // NOI18N

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(20, 4, 0, 0);
        add(jScrollPane1, gridBagConstraints);

        Refresh.setText(resourceMap.getString("Refresh.text")); // NOI18N
        Refresh.setName("Refresh"); // NOI18N
        Refresh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                RefreshActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        add(Refresh, gridBagConstraints);

        Clear.setText(resourceMap.getString("Clear.text")); // NOI18N
        Clear.setName("Clear"); // NOI18N
        Clear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ClearActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 0);
        add(Clear, gridBagConstraints);

        totalUsLabel.setText(resourceMap.getString("totalUsLabel.text")); // NOI18N
        totalUsLabel.setName("totalUsLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 25, 0, 0);
        add(totalUsLabel, gridBagConstraints);

        totalMacLabel.setText(resourceMap.getString("totalMacLabel.text")); // NOI18N
        totalMacLabel.setName("totalMacLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 25, 0, 0);
        add(totalMacLabel, gridBagConstraints);

        totalRelLabel.setText(resourceMap.getString("totalRelLabel.text")); // NOI18N
        totalRelLabel.setName("totalRelLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 25, 0, 0);
        add(totalRelLabel, gridBagConstraints);

        totalMacSecLabel.setText(resourceMap.getString("totalMacSecLabel.text")); // NOI18N
        totalMacSecLabel.setName("totalMacSecLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 25, 0, 0);
        add(totalMacSecLabel, gridBagConstraints);

        totalFaults.setText(resourceMap.getString("totalFaults.text")); // NOI18N
        totalFaults.setName("totalFaults"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 25, 0, 0);
        add(totalFaults, gridBagConstraints);

        totalFaultsFreq.setText(resourceMap.getString("totalFaultsFreq.text")); // NOI18N
        totalFaultsFreq.setName("totalFaultsFreq"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 25, 0, 0);
        add(totalFaultsFreq, gridBagConstraints);

        clearFaults.setText(resourceMap.getString("clearFaults.text")); // NOI18N
        clearFaults.setName("clearFaults"); // NOI18N
        clearFaults.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearFaultsActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        add(clearFaults, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents

    private void RefreshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_RefreshActionPerformed
        // TODO add your handling code here:
        Refresh();
}//GEN-LAST:event_RefreshActionPerformed

    private void ClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ClearActionPerformed
        // TODO add your handling code here:
        model.clearRows();
        table.updateUI();
    }//GEN-LAST:event_ClearActionPerformed

    private void clearFaultsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearFaultsActionPerformed
        // TODO add your handling code here:
        resetCounter+=totalfaults;
        resetTstamp=lastTS;
        Refresh();
    }//GEN-LAST:event_clearFaultsActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton Clear;
    private javax.swing.JButton Refresh;
    private javax.swing.JButton clearFaults;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable table;
    private javax.swing.JLabel title;
    private javax.swing.JLabel totalFaults;
    private javax.swing.JLabel totalFaultsFreq;
    private javax.swing.JLabel totalMacLabel;
    private javax.swing.JLabel totalMacSecLabel;
    private javax.swing.JLabel totalRelLabel;
    private javax.swing.JLabel totalUsLabel;
    // End of variables declaration//GEN-END:variables

}
