/*
 * PhalguiApp.java
 */
package phalgui;

import java.awt.Frame;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.Properties;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import org.jdesktop.application.Application;
import org.jdesktop.application.SingleFrameApplication;
import phalgui.Control.MainControl;


/**
 * The main class of the application.
 */
public class PhalguiApp extends SingleFrameApplication implements WindowListener {

    private PhalguiView view;
    static final String CONFIG_FILE = System.getProperty("user.home") + "/.phalgui.conf";
 

    /**
     * At startup create and show the main frame of the application.
     */
    @Override
    protected void startup() {
        view = new PhalguiView(this);
        view.getFrame().addWindowListener(this);
        JFrame f = this.getMainFrame();
        show(view);
        f.pack();
        f.setExtendedState(Frame.MAXIMIZED_BOTH);
        view.DefaultOutput().append(" --- ALOE-OE GUI ---");
        view.DefaultOutput().append("\n\n");
        view.DefaultOutput().append("Reading saved configuration...\n");
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                ReadProps(new File(CONFIG_FILE));
            }
        });
    }

    public void WriteProps(File file) {
        OutputStream out;
        InputStream in;
        Properties props = new Properties();
        try {
            in = new FileInputStream(file);
            props.load(in);
            view.main_tree.WriteConfig(props);
            RandomAccessFile r = new RandomAccessFile(file, "rw");
            r.setLength(0);
            out = new FileOutputStream(file);
            props.store(out, "Main configuration");
            out.close();
        } catch (FileNotFoundException ex) {
            System.out.print("Error opening configuration file: " + ex.getMessage());
        } catch (IOException ex) {
            System.out.print("Error reading configuration file: " + ex.getMessage());
        }
    }

    public void ReadProps(File file) {
        InputStream s;
        Properties props = new Properties();
        try {
            s = new FileInputStream(file);
            props.load(s);
            view.main_tree.ReadConfig(props);
            s.close();
        } catch (FileNotFoundException ex) {
            System.out.print("Error opening configuration file: " + ex.getMessage());
        } catch (IOException ex) {
            System.out.print("Error reading configuration file: " + ex.getMessage());
        }
    }

    public MainControl getMainControl() {
        return view.main;
    }
    
    public PhalguiView getView() {
        return view;
    }

    /**
     * This method is to initialize the specified window by injecting resources.
     * Windows shown in our application come fully initialized from the GUI
     * builder, so this additional configuration is not needed.
     */
    @Override
    protected void configureWindow(java.awt.Window root) {
    }

    /**
     * A convenient static getter for the application instance.
     * @return the instance of PhalguiApp
     */
    public static PhalguiApp getApplication() {
        return Application.getInstance(PhalguiApp.class);
    }

    /**
     * Main method launching the application.
     */
    public static void main(String[] args) {
        launch(PhalguiApp.class, args);
    }
   


    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
        windowClosed(e);
    }

    public void windowClosed(WindowEvent e) {
        view.body_tabs.closeAll();
        WriteProps(new File(CONFIG_FILE));
        System.exit(0);
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }
}
