/*
 * PhalguiView.java
 */

package phalgui;

import phalgui.Tree.MainTree;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.JTree.*;
import phalgui.Control.MainControl;
import phalgui.Control.OutputLog;
import phalgui.Control.StatusBar;
import phalgui.Control.StatusPanel;
import phalgui.Node.MainEntry;
import phalgui.Node.Repositories.RepositoryObject;
import phalgui.Node.NodeObject;
import phalgui.Node.Repositories.DefaultRepository;
import phalgui.Node.Repositories.WaveformRepository;
import phalgui.Utils.JTabbedPaneWithIcon;

/**
 * The application's main frame.
 */
public class PhalguiView extends FrameView {
    
    private PhalguiApp app;
   
    public MainTree main_tree;
    
    public JTextArea DefaultOutput() {
        return default_output;
    }

    public ResourceMap getResources() {
        return getResourceMap();
    }
    
    public PhalguiView(SingleFrameApplication ap) {
        super(ap);
        
        app = PhalguiApp.getApplication();

        initComponents();
        initMyComponents();

        MainSplit.setContinuousLayout(true);
        TopSplit.setContinuousLayout(true);

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
        
        
        
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = PhalguiApp.getApplication().getMainFrame();
            aboutBox = new PhalguiAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        PhalguiApp.getApplication().show(aboutBox);
    }
    
    
    
    JTabbedPaneWithIcon body_tabs;
    StartPage start_page;
    MainControl main;
    
    private void initMyComponents() {


        MainEntry root = new MainEntry("ALOE Management");
        main_tree = new MainTree(root);
        panel_tree.setViewportView(main_tree);


        body_tabs = new JTabbedPaneWithIcon(PhalguiApp.getApplication().getMainFrame());
        StatusBar statusBar = new StatusBar(statusIcon,statusMessageLabel);
        OutputLog outputLog = new OutputLog(output_tabbedpane);
        main = new MainControl(outputLog, body_tabs,this.getFrame(),statusBar, main_tree);

        MainSplit.setDividerLocation(0.7);
        MainSplit.setResizeWeight(0.9);
        TopSplit.setDividerLocation(0.15);
        TopSplit.setRightComponent(body_tabs);
        TopSplit.setOneTouchExpandable(true);
        TopSplit.setResizeWeight(0.0);

        StatusPanel statusP = new StatusPanel(main, jSetupButton, jInitButton, jRunButton,
                jPauseButton, jStopButton);

        WaveformRepository waveform = new WaveformRepository(main, statusP, "Waveforms",null,
                new ImageIcon(PhalguiView.class.getResource("resources/repository.gif")));
        root.add(waveform);

        NodeObject node = new DefaultRepository(main, "Platforms",
                RepositoryObject.FileType.PLATFORMS,
                new ImageIcon(PhalguiView.class.getResource("resources/tree.gif")));
        root.add(node);

        node = new DefaultRepository(main, "StatPanels",
                RepositoryObject.FileType.PANELS,
                new ImageIcon(PhalguiView.class.getResource("resources/graph.gif")));
        root.add(node);
    
    }   
  
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        jMenu2 = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenuItem4 = new javax.swing.JMenuItem();
        jMenuItem5 = new javax.swing.JMenuItem();
        jMenuItem6 = new javax.swing.JMenuItem();
        jMenuItem2 = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        jMenuItem7 = new javax.swing.JMenuItem();
        jMenuItem8 = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem3 = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        statusMessageLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        statusAnimationLabel = new javax.swing.JLabel();
        statusIcon = new javax.swing.JLabel();
        MainSplit = new javax.swing.JSplitPane();
        TopSplit = new javax.swing.JSplitPane();
        jControlPanel = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jSetupButton = new javax.swing.JButton();
        jInitButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jRunButton = new javax.swing.JButton();
        jPauseButton = new javax.swing.JButton();
        jStopButton = new javax.swing.JButton();
        panel_tree = new javax.swing.JScrollPane();
        output_tabbedpane = new javax.swing.JTabbedPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        default_output = new javax.swing.JTextArea();

        menuBar.setName("menuBar"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(phalgui.PhalguiApp.class).getContext().getResourceMap(PhalguiView.class);
        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        jMenu2.setText(resourceMap.getString("jMenu2.text")); // NOI18N
        jMenu2.setEnabled(false);
        jMenu2.setName("jMenu2"); // NOI18N

        jMenuItem1.setText(resourceMap.getString("jMenuItem1.text")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        jMenu2.add(jMenuItem1);

        jMenuItem4.setText(resourceMap.getString("jMenuItem4.text")); // NOI18N
        jMenuItem4.setName("jMenuItem4"); // NOI18N
        jMenu2.add(jMenuItem4);

        jMenuItem5.setText(resourceMap.getString("jMenuItem5.text")); // NOI18N
        jMenuItem5.setName("jMenuItem5"); // NOI18N
        jMenu2.add(jMenuItem5);

        fileMenu.add(jMenu2);

        jMenuItem6.setText(resourceMap.getString("jMenuItem6.text")); // NOI18N
        jMenuItem6.setEnabled(false);
        jMenuItem6.setName("jMenuItem6"); // NOI18N
        fileMenu.add(jMenuItem6);

        jMenuItem2.setText(resourceMap.getString("jMenuItem2.text")); // NOI18N
        jMenuItem2.setEnabled(false);
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        fileMenu.add(jMenuItem2);

        jSeparator1.setName("jSeparator1"); // NOI18N
        fileMenu.add(jSeparator1);

        jMenuItem7.setText(resourceMap.getString("jMenuItem7.text")); // NOI18N
        jMenuItem7.setName("jMenuItem7"); // NOI18N
        jMenuItem7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem7ActionPerformed(evt);
            }
        });
        fileMenu.add(jMenuItem7);

        jMenuItem8.setText(resourceMap.getString("jMenuItem8.text")); // NOI18N
        jMenuItem8.setName("jMenuItem8"); // NOI18N
        jMenuItem8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem8ActionPerformed(evt);
            }
        });
        fileMenu.add(jMenuItem8);

        jSeparator2.setName("jSeparator2"); // NOI18N
        fileMenu.add(jSeparator2);

        exitMenuItem.setText(resourceMap.getString("exitMenuItem.text")); // NOI18N
        exitMenuItem.setToolTipText(resourceMap.getString("exitMenuItem.toolTipText")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        jMenu1.setEnabled(false);
        jMenu1.setName("jMenu1"); // NOI18N

        jMenuItem3.setText(resourceMap.getString("jMenuItem3.text")); // NOI18N
        jMenuItem3.setName("jMenuItem3"); // NOI18N
        jMenu1.add(jMenuItem3);

        menuBar.add(jMenu1);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(phalgui.PhalguiApp.class).getContext().getActionMap(PhalguiView.class, this);
        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N
        statusPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        statusMessageLabel.setText(resourceMap.getString("statusMessageLabel.text")); // NOI18N
        statusMessageLabel.setName("statusMessageLabel"); // NOI18N
        statusPanel.add(statusMessageLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(40, 0, -1, 20));

        progressBar.setName("progressBar"); // NOI18N
        statusPanel.add(progressBar, new org.netbeans.lib.awtextra.AbsoluteConstraints(1170, 0, -1, 10));

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        statusAnimationLabel.setText(resourceMap.getString("statusAnimationLabel.text")); // NOI18N
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N
        statusPanel.add(statusAnimationLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(810, 0, 10, 10));

        statusIcon.setIcon(resourceMap.getIcon("statusIcon.icon")); // NOI18N
        statusIcon.setText(resourceMap.getString("statusIcon.text")); // NOI18N
        statusIcon.setName("statusIcon"); // NOI18N
        statusPanel.add(statusIcon, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 0, -1, 20));

        MainSplit.setDividerLocation(250);
        MainSplit.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        MainSplit.setName("MainSplit"); // NOI18N

        TopSplit.setDividerLocation(250);
        TopSplit.setName("TopSplit"); // NOI18N

        jControlPanel.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jControlPanel.setName("jControlPanel"); // NOI18N
        jControlPanel.setPreferredSize(new java.awt.Dimension(250, 345));
        jControlPanel.setLayout(new java.awt.GridBagLayout());

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setLayout(new java.awt.GridBagLayout());

        jSetupButton.setText(resourceMap.getString("jSetupButton.text")); // NOI18N
        jSetupButton.setToolTipText(resourceMap.getString("jSetupButton.toolTipText")); // NOI18N
        jSetupButton.setMargin(new java.awt.Insets(2, 10, 2, 10));
        jSetupButton.setName("jSetupButton"); // NOI18N
        jPanel2.add(jSetupButton, new java.awt.GridBagConstraints());

        jInitButton.setText(resourceMap.getString("jInitButton.text")); // NOI18N
        jInitButton.setToolTipText(resourceMap.getString("jInitButton.toolTipText")); // NOI18N
        jInitButton.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jInitButton.setName("jInitButton"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        jPanel2.add(jInitButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(20, 2, 0, 0);
        jControlPanel.add(jPanel2, gridBagConstraints);

        jPanel3.setName("jPanel3"); // NOI18N

        jRunButton.setIcon(resourceMap.getIcon("jRunButton.icon")); // NOI18N
        jRunButton.setToolTipText(resourceMap.getString("jRunButton.toolTipText")); // NOI18N
        jRunButton.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jRunButton.setName("jRunButton"); // NOI18N
        jPanel3.add(jRunButton);

        jPauseButton.setIcon(resourceMap.getIcon("jPauseButton.icon")); // NOI18N
        jPauseButton.setToolTipText(resourceMap.getString("jPauseButton.toolTipText")); // NOI18N
        jPauseButton.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jPauseButton.setName("jPauseButton"); // NOI18N
        jPanel3.add(jPauseButton);

        jStopButton.setIcon(resourceMap.getIcon("jStopButton.icon")); // NOI18N
        jStopButton.setToolTipText(resourceMap.getString("jStopButton.toolTipText")); // NOI18N
        jStopButton.setMargin(new java.awt.Insets(2, 5, 2, 5));
        jStopButton.setName("jStopButton"); // NOI18N
        jPanel3.add(jStopButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 4;
        gridBagConstraints.ipady = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(10, 16, 0, 16);
        jControlPanel.add(jPanel3, gridBagConstraints);

        panel_tree.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        panel_tree.setMinimumSize(new java.awt.Dimension(70, 200));
        panel_tree.setName("panel_tree"); // NOI18N
        panel_tree.setPreferredSize(new java.awt.Dimension(70, 200));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 80;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHEAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 3, 0);
        jControlPanel.add(panel_tree, gridBagConstraints);

        TopSplit.setLeftComponent(jControlPanel);

        MainSplit.setLeftComponent(TopSplit);

        output_tabbedpane.setName("output_tabbedpane"); // NOI18N
        output_tabbedpane.setPreferredSize(new java.awt.Dimension(0, 0));

        jScrollPane1.setAutoscrolls(true);
        jScrollPane1.setName("jScrollPane1"); // NOI18N

        default_output.setColumns(20);
        default_output.setRows(5);
        default_output.setName("default_output"); // NOI18N
        jScrollPane1.setViewportView(default_output);

        output_tabbedpane.addTab(resourceMap.getString("jScrollPane1.TabConstraints.tabTitle"), jScrollPane1); // NOI18N

        MainSplit.setRightComponent(output_tabbedpane);

        setComponent(MainSplit);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

private void jMenuItem7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem7ActionPerformed
    final JFileChooser fc = new JFileChooser();

    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
    if (fc.showOpenDialog(this.getFrame()) == JFileChooser.APPROVE_OPTION) {
        app.ReadProps(fc.getSelectedFile());
    }
}//GEN-LAST:event_jMenuItem7ActionPerformed

private void jMenuItem8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem8ActionPerformed
    final JFileChooser fc = new JFileChooser();

    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
    if (fc.showSaveDialog(this.getFrame()) == JFileChooser.APPROVE_OPTION) {
        if (fc.getSelectedFile().exists()) {
            if (JOptionPane.showConfirmDialog(this.getFrame(), 
                    "File exists. Do you want to overwritte it?", 
                    "Save Configuration", JOptionPane.YES_NO_OPTION)
                    != JOptionPane.YES_OPTION) {
                return;
            } 
        }
        app.WriteProps(fc.getSelectedFile());
    }
}//GEN-LAST:event_jMenuItem8ActionPerformed

private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
    this.getFrame().setVisible(false);
    this.getFrame().dispose();
}//GEN-LAST:event_exitMenuItemActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JSplitPane MainSplit;
    private javax.swing.JSplitPane TopSplit;
    private javax.swing.JTextArea default_output;
    private javax.swing.JPanel jControlPanel;
    private javax.swing.JButton jInitButton;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JMenuItem jMenuItem6;
    private javax.swing.JMenuItem jMenuItem7;
    private javax.swing.JMenuItem jMenuItem8;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JButton jPauseButton;
    private javax.swing.JButton jRunButton;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JButton jSetupButton;
    private javax.swing.JButton jStopButton;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JTabbedPane output_tabbedpane;
    private javax.swing.JScrollPane panel_tree;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusIcon;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
}
