/*
 * Extension.java
 *
 * Created on 7 de diciembre de 2001, 00:55
 */

package phalgui.Utils.Arrows;


import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.*;

/**
 * This class implements the arrows file view. Sets the icons into the fileChooser
 * depending on the extension of the files.
 * @author  Santi Oliveras
 * @version 1.0  8/05/2002
 * @see ArrowsFilter
 */
public class ArrowsFileView extends FileView {
    
    // ******** private variables **********************************************
    
    // the user directory and the file system separator
    private String userDir = System.getProperty("user.dir"),     // find actual directory
            file_sep = System.getProperty("file.separator");       
            
    // the images associated to the icons
    private ImageIcon jpgIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"jpgIcon.gif");
    private ImageIcon gifIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"gifIcon.gif");
    private ImageIcon tiffIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"tiffIcon.gif");
    private ImageIcon iniIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"iniIcon.gif");
    private ImageIcon logIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"logIcon.gif");
    private ImageIcon dvwIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"dvwIcon.jpg");
    private ImageIcon graphIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"graph.gif");
    private ImageIcon pagIcon = new ImageIcon(userDir+file_sep+"Images"+file_sep+"pagIcon.jpg");
    
    
    // ******** public methods *************************************************
    
    public String getName(File f) {
        return null; // let the L&F FileView figure this out
    }
    
    public String getDescription(File f) {
        return null; // let the L&F FileView figure this out
    }
    
    public Boolean isTraversable(File f) {
        return null; // let the L&F FileView figure this out
    }
    
    /** Returns a string with the type description.
     * <BR>The descriptions associated to the extension are the next.
     *<PRE>
     * <U>Description</U>              <U>Extensions associated</U>
     * JPEG Image               *.jpeg, *.jpg
     * GIF Image                *.gif
     * TIFF Image               *.tiff, *.tiff
     * INI File                 *.ini
     * LOG File                 *.log
     * Graphic File             *.graph
     * Panels Agrupation File   *.pag
     * Dinamic Viewer File      *.dvw
     *</PRE>
     * @param f The file name
     * @see ArrowsFilter
     */
    public String getTypeDescription(File f) {
        String extension = Extension.getExtension(f);
        String type = null;

        if (extension != null) {
            if (extension.equals(Extension.JPEG) ||
                extension.equals(Extension.JPG)) {
                type = "JPEG Image";
            } else if (extension.equals(Extension.GIF)){
                type = "GIF Image";
            } else if (extension.equals(Extension.TIFF) ||
                       extension.equals(Extension.TIF)) {
                type = "TIFF Image";
            } else if (extension.equals(Extension.INI)){
                type = "INI File"; 
            } else if (extension.equals(Extension.LOG)){
                type = "LOG File"; 
            } else if (extension.equals(Extension.GRAPH)){
                type = "Graphic File";
            } else if (extension.equals(Extension.PAG)){
                type = "Panels Agrupation File";
            } else if (extension.equals(Extension.DVW)){
                type = "Dinamic Viewer File"; 
            }
        }
        return type;
    }
    
    /** Returns the icon to be seen in the fileChooser depending on the extension type.
     * @see ArrowsFilter
     */
    public Icon getIcon(File f) {
        String extension = Extension.getExtension(f);
        Icon icon = null;

        if (extension != null) {
            if (extension.equals(Extension.JPEG) ||
                extension.equals(Extension.JPG)) {
                icon = jpgIcon;
            } else if (extension.equals(Extension.GIF)) {
                icon = gifIcon;
            } else if (extension.equals(Extension.TIFF) ||
                       extension.equals(Extension.TIF)) {
                icon = tiffIcon;
            } else if (extension.equals(Extension.INI)) {
                icon = iniIcon;
            } else if (extension.equals(Extension.LOG)) {
                icon = logIcon;
            } else if (extension.equals(Extension.GRAPH)) {
                icon = graphIcon;
            } else if (extension.equals(Extension.PAG)) {
                icon = pagIcon;
            } else if (extension.equals(Extension.DVW)) {
                icon = dvwIcon;
            } 
            
        }
        return icon;
    }
}
