package phalgui.Utils.Arrows;

// *************************************************************************
// Copyright (C) 1997, by   DeTeMobil Deutsche Telekom MobilNet GmbH
//			    Roddestr. 12
//			    48153 M�nster
//			    (Germany)
// *************************************************************************
// programme-name   :       StringParser.java
// version	    :       0.8.15
// date(YYMMDD)     :       971009
// author(s)	    :       Recker, Stephan;
// remark	    :
// *************************************************************************
// purpose	    :
// *************************************************************************

/*   divides strings into different parts defined by delimiters   */
 
// {{{ imports

import java.lang.*;

// }}}

/**
 * This class contains the fields and methods for dividing
 * a string into substrings that are seperated by a delimiter.
 *
 * By making an instance of this class you have to set the
 * string you want to divide and a flag.
 * If the flag is set true, the delimiter will be contained
 * in the substrings, if false it will not.
 *
 * @version    08.15 16/10/97
 * @author     Stephan Recker
 *
 */

public class StringParser {

// {{{ declarations

  private String string;
  private boolean showdelim;       // flag for the Use of delimiters:
    // true : delimiters will be written in new string
  private int actualPos = 0;       // actual position in string


// }}}

// {{{ set-methods

    /**
     * sets string to divide
     *
     * @param str string to divide
     */

  public void setString(String str) {
      string = str;
      actualPos=0;
  }

    /**
     * sets delimiter-flag
     *
     * @param b boolean for delimiter flag
     */

  public void setShowDelim(boolean b) {
      showdelim = b;
  }

// }}}

// {{{ constructor

    /**
     * @param str string to divide
     * @param sd delimiter-flag
     */

  public StringParser(String str, boolean sd) {
      this();
      string = str;
      showdelim = sd;
  }

  public StringParser() {}

// }}}

// {{{ hasNextToken

    /**
     * This method checks, if string could be divided in substrings
     *
     * @param delim delimiter between substrings
     *
     * @return true if string could be divided
     */

  public boolean hasNextToken(char delim) {
      int oldPos;

      oldPos = actualPos;       // keeps old position

      while ((actualPos < string.length()) &&
	     !((new Character(string.charAt(actualPos))).equals(new Character(delim)))) {
	  // search for specified delimiter
	  actualPos++;
      }
      if (actualPos == string.length()) {  // at the end of the string => no delimiter found
	  actualPos = oldPos; // division into substrings is not possible
	  return false; // set back to old position
      }
      else {
	  actualPos = oldPos;
	  return true;
      }
  }

    /**
     * This method checks, if a substring to the end of the
     * string exist.
     * (overloaded method)
     *
     * @return true if substring exists
     */

  public boolean hasNextToken() {

      if (actualPos < string.length()) {
	  return true;
      }	 // see previous method
      else {
	  return false;
      }
  }

// }}}

// {{{ getNextToken

    /**
     * This method returns substring between the actual
     * position and the delimiter
     *
     * @param delim delimiter between substrings
     *
     * @return correspondent substring
     */

  public String getNextToken(char delim) {
      String returnString;
      int i = actualPos;

      while ((i < string.length()) &&      // searches for delimiter
	     !((new Character(string.charAt(i))).equals(new Character(delim)))) {
	  i++;
      }
      if (i < string.length()) {
	  if (showdelim) {
	      // returns substring containing the delimiter
	      returnString =  string.substring(actualPos,i);
	      actualPos = i+1;
	      return returnString;
	  }
	  else {
	      // returns substring without delimiter
	      returnString =  string.substring(actualPos,i);
	      actualPos = i+1;
	      return returnString;
	  }
      }
      else {
	  return null;
      }
  }

    /**
     * This method returns substring between the actual
     * position and the end of the string.
     * (overloaded method)
     *
     * @return correspondent substring
     */

  public String getNextToken() {
      if (actualPos < string.length())  {
	  return string.substring(actualPos).trim();      // returns rest of string
      }
      else {
	  return null;
      }
  }
  
  /**
     * This method returns substring after the last delimiter.
     *
     * @param delim Last delimiter. 
     *
     * @return correspondent substring
     */
  
  public String getFinalToken(char delim){
      
      while(hasNextToken(delim)){
            getNextToken(delim);}
      if(actualPos==0)
          return null;
      if(hasNextToken())
          return getNextToken();  
      return null;
  }
// }}}
}
