/*
 * Trace.java
 *
 * Created on 26 de octubre de 2001, 18:40
 */

package phalgui.Utils.Arrows;

/**
 * This class supports the printing of the traces into the standart output.
 * There are different levels of traces to see more or less information.
 * @author  Santi Oliveras
 * @version 1.0  8/05/2002
 */ 
public class Trace {
    
    // ******** public variables ***********************************************
    
    /** Integer with the level of file lines. */
    public static final int FILELINES=3;
    
    /** Integer with the level of module information lines. */
    public static final int MODULESTRUCT=3;
    
    /** Integer with the level of ini file lines. */
    public static final int INIFILESTRUCT=3;
    
    /** Integer with the level of error traces. (Maximum level) */
    public static final int ERRORS=0;
    
    /** Integer with the level of the traces with maximum level. */
    public static final int MAXLEVEL=1;    
    
    /** Integer with the level of the time traces. */
    public static final int TIMEINFO=2;   
    
    /** Integer with the level of the traces with normal level. */
    public static final int NORMALLEVEL=2;     
    
    /** Integer with the level of the traces with minimum level. */
    public static final int MINLEVEL=4;    
    
    /** The current trace level. The maximum level is 0. */
    public static int level=3; 
    
    /** If true the traces with higher level than current level are printed. */
    public static boolean printHigh=true;
    
    /** If true, class situation information is printed. */
    public static boolean siteInfo=true;
    
    /** If true, no traces are printed. */
    public static boolean noTraces=false;

    
    // ******** public methods *************************************************
    
    /** Prints to the standard output the message if the level configurations allow it.
     * @param message String with the message to print
     * @param priority Integer with the trace priority
     * @param file String with the class where the trace is called
     * @param line Integer with the file line where the trace is called
     */
    public static void print(String message,int priority,String file,int line){
        if(noTraces)  return;
        if(priority==level){
            if(siteInfo)
                System.out.println("FILE: "+file+"  LINE: "+line+"    "+message);
            else
                System.out.println(message);
        }
        if((priority<level)&(printHigh)){
            if(siteInfo)
                System.out.println("FILE: "+file+"  LINE: "+line+"    "+message);
            else
                System.out.println(message);
        }    
    }
    
    /** Prints to the standard output an integer if the level configurations allow it.
     * @param message Integer with the integer to print
     * @param priority Integer with the trace priority
     * @param file String with the class where the trace is called
     * @param line Integer with the file line where the trace is called
     */
    public static void print(int message,int priority,String file,int line){
        print(new Integer(message).toString(),priority,file,line);
    }
    
    /** Prints to the standard output a boolean if the level configurations allow it.
     * @param message Boolean with the boolean to print
     * @param priority Integer with the trace priority
     * @param file String with the class where the trace is called
     * @param line Integer with the file line where the trace is called
     */
    public static void print(boolean message,int priority,String file,int line){
        if(message)
            print("true",priority,file,line);
        else       
            print("false",priority,file,line);    
    }

        
}
