/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Utils.DnD;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.IOException;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingWorker;

/**
 *
 * @author Usuario
 */
public abstract class DropablePanel extends JPanel implements DropTargetListener {


    private final Cursor droppableCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR),
            notDroppableCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private DataFlavor flavor;

    public DropablePanel(DataFlavor flavor) {
        new DropTarget(this, this);
        this.flavor = flavor;
    }
    public void dragOver(DropTargetDragEvent dtde) {
        if (!this.getCursor().equals(droppableCursor)) {
            this.setCursor(droppableCursor);
        }
    }
    public void dropActionChanged(DropTargetDragEvent dtde) {}
    public void dragExit(DropTargetEvent dte) {
        this.setCursor(notDroppableCursor);
    }

    /**
     * <p>The user drops the item. Performs the drag and drop calculations and layout.</p>
     * @param dtde
     */
    public void drop(final DropTargetDropEvent dtde) {

        // Done with cursors, dropping
        this.setCursor(Cursor.getDefaultCursor());

        final Object transferableObj;
        final Transferable transferable = dtde.getTransferable();

       // What does the Transferable support
        if (transferable.isDataFlavorSupported(flavor)) {
            try {
                transferableObj = dtde.getTransferable().getTransferData(flavor);
            } catch (UnsupportedFlavorException ex) {
                Logger.getLogger(DropablePanel.class.getName()).log(Level.SEVERE, null, ex);
                return;
            } catch (IOException ex) {
                Logger.getLogger(DropablePanel.class.getName()).log(Level.SEVERE, null, ex);
                return;
            }

            SwingWorker w = new SwingWorker<Object,Void>() {

                @Override
                protected Object doInBackground() throws Exception {
                    if (transferableObj instanceof TransfObject) {
                        return ((TransfObject) transferableObj).newObject(flavor);
                    } else {
                        return transferableObj;
                    }
                }
                @Override
                protected void done() {
                    try {
                        dropObject(get(), dtde);
                    } catch (InterruptedException ex) {
                        Logger.getLogger(DropablePanel.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (ExecutionException ex) {
                        Logger.getLogger(DropablePanel.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            };
            w.execute();
        } 
    }

    public abstract void dropObject(Object object, DropTargetDropEvent dtde);

    public void dragEnter(DropTargetDragEvent dtde) {

    }

}
