/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Utils;

import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.basic.BasicButtonUI;
import phalgui.Node.NodeObject;


/**
 * A JTabbedPane which has a close ('X') icon on each tab.
 *
 * To add a tab, use the method addTab(String, Component)
 *
 * To have an extra icon on each tab (e.g. like in JBuilder, showing the file type) use
 * the method addTab(String, Component, Icon). Only clicking the 'X' closes the tab.
 */
public class JTabbedPaneWithIcon extends JTabbedPane {

    public JFrame pframe;

    public JTabbedPaneWithIcon(JFrame frame) {
        super();
        pframe = frame;
    }

    @Override
    public void addTab(String title, Component component) {
        super.addTab(title, component);
        ButtonTabComponent button = new ButtonTabComponent(this);
        super.setTabComponentAt(super.getTabCount() - 1, button);
    }

    public void addTab(String title, Component component, NodeObject object) {
        super.addTab(title, component);
        ButtonTabComponent button = new ButtonTabComponent(this, object);
        super.setTabComponentAt(super.getTabCount() - 1, button);
    }

    public void closeAll() {
        for (int i=0;i<this.getTabCount();i++) {
            ((ButtonTabComponent) this.getTabComponentAt(i)).object.Close();
        }
    }

    public boolean hasComponent(Component component) {
        int i;
        Component[] comps = getComponents();
        for (i=0;i<comps.length;i++) {
            if (comps[i].equals(component))
                return true;
        }
        return false;
    }
}
class ButtonTabComponent extends JPanel {

    private final JTabbedPaneWithIcon pane;
    public NodeObject object = null;
    JLabel label;
    public ButtonTabComponent(final JTabbedPaneWithIcon pane) {
        //unset default FlowLayout' gaps
        super(new FlowLayout(FlowLayout.LEFT, 0, 0));
        if (pane == null) {
            throw new NullPointerException("TabbedPane is null");
        }
        this.pane = pane;
        setOpaque(false);

        //make JLabel read titles from JTabbedPane
        label = new JLabel() {

            @Override
            public String getText() {
                int i = pane.indexOfTabComponent(ButtonTabComponent.this);
                if (i != -1) {
                    return pane.getTitleAt(i);
                }
                return null;
            }

        };

        add(label);
        //add more space between the label and the button
        label.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        //tab button
        JButton button = new TabButton();
        add(button);
        //add more space to the top of the component
        setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0));
        this.addMouseListener(new TabMouseListener(this));
    }

    public ButtonTabComponent(JTabbedPaneWithIcon pane, NodeObject object) {
        this(pane);
        this.object = object;
    }

    public void removetab() {
        int i = pane.indexOfTabComponent(ButtonTabComponent.this);
        if (object == null) {
            pane.remove(i);
        } else {
            if (object.Close()) {
                pane.remove(i);
            }
        }
    }

    private class TabMouseListener implements MouseListener,ActionListener {

        ButtonTabComponent tab;
        JPopupMenu popup;
        JMenuItem towindow,close;
        private TabMouseListener(ButtonTabComponent tab) {
            this.tab = tab;
            popup = new JPopupMenu();
            towindow = new JMenuItem("To window");
            close = new JMenuItem("Close");
            popup.add(towindow);
            popup.add(close);
            towindow.addActionListener(this);
            towindow.setActionCommand("window");
            close.addActionListener(this);
            close.setActionCommand("close");
        }

        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON3) {
                popup.show((JComponent) e.getSource(), e.getX(), e.getY());
                popup.setVisible(true);
            } else {
                int i = pane.indexOfTabComponent(tab);
                pane.setSelectedIndex(i);
            }
        }


        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("close")) {
                removetab();
            } else {
                int i = pane.indexOfTabComponent(tab);
                Component comp = pane.getComponentAt(i);
                JFrame frame = new JFrame(pane.getTitleAt(i));
                frame.getContentPane().add(comp, BorderLayout.CENTER);
                frame.pack();
                frame.setVisible(true);
                frame.addWindowListener(new WindowAdapter() {

                    @Override
                    public void windowClosing(WindowEvent e) {
                        object.Close();
                    }
                });
            }
        }

        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
    }

    private class TabButton extends JButton implements ActionListener {

        public TabButton() {
            int size = 17;
            setPreferredSize(new Dimension(size, size));
            setToolTipText("close this tab");
            //Make the button looks the same for all Laf's
            setUI(new BasicButtonUI());
            //Make it transparent
            setContentAreaFilled(false);
            //No need to be focusable
            setFocusable(false);
            setBorder(BorderFactory.createEtchedBorder());
            setBorderPainted(false);
            //Making nice rollover effect
            //we use the same listener for all buttons
            addMouseListener(buttonMouseListener);
            setRolloverEnabled(true);
            //Close the proper tab by clicking the button
            addActionListener(this);
        }

        public void actionPerformed(ActionEvent e) {
            removetab();
        }

        //we don't want to update UI for this button
        public void updateUI() {
        }

        //paint the cross
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            Graphics2D g2 = (Graphics2D) g.create();
            //shift the image for pressed buttons
            if (getModel().isPressed()) {
                g2.translate(1, 1);
            }
            g2.setStroke(new BasicStroke(2));
            g2.setColor(Color.BLACK);
            if (getModel().isRollover()) {
                g2.setColor(Color.MAGENTA);
            }
            int delta = 6;
            g2.drawLine(delta, delta, getWidth() - delta - 1, getHeight() - delta - 1);
            g2.drawLine(getWidth() - delta - 1, delta, delta, getHeight() - delta - 1);
            g2.dispose();
        }

    }
    private final static MouseListener buttonMouseListener = new  

          MouseAdapter( ) {
               
                
                    
                public void mouseEntered(MouseEvent e) {
            Component component = e.getComponent();
            if (component instanceof AbstractButton) {
                AbstractButton button = (AbstractButton) component;
                button.setBorderPainted(true);
            }
        }

        public void mouseExited(MouseEvent e) {
            Component component = e.getComponent();
            if (component instanceof AbstractButton) {
                AbstractButton button = (AbstractButton) component;
                button.setBorderPainted(false);
            }
        }
    };


}
