/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Utils;

import phalgui.Node.MultiPanel.*;
import java.awt.Color;
import java.awt.Component;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import javax.swing.DropMode;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.TransferHandler;
import javax.swing.border.LineBorder;
import javax.swing.table.TableCellRenderer;
import phalgui.Node.NodeObject;
import phalgui.Utils.DnD.TransfResizePanel;



/**
 *
 * @author Usuario
 */
public class PanelTable extends JTable {

    Component parent;
    PanelTableModel model;
    boolean autofit;

    public PanelTable(Object defaultObject) {
       this(defaultObject,null);
    }

    PanelTable(Object defaultObject, Component parent) {
        super();
        this.model = model = new PanelTableModel(defaultObject);
        setModel(model);
        setDefaultRenderer(getColumnClass(0), new panelCellRenderer());
        setCellSelectionEnabled(true);
        setDropMode(DropMode.ON);
        setTransferHandler(new TableTransferHandler(this));
        setDragEnabled(true);
        setAutoCreateColumnsFromModel(true);
        this.autofit = true;
        this.parent = parent;
    }

    void addObject(Object data, int rowAtPoint, int columnAtPoint) {
        model.addObject(data, rowAtPoint, columnAtPoint);
    }

 
    void removeObject(int selectedRow, int selectedColumn) {
        model.removeObject(selectedRow, selectedColumn);
    }

    void removeSelectedObjects() {
        for (int i=0;i<this.getSelectedRowCount();i++) {
            removeObject(this.getSelectedRows()[i], this.getSelectedColumns()[i]);
        }
    }

    void setAutofit(boolean enabled) {
        autofit = enabled;
    /*    if (enabled) {
            setRowHeight(this.getHeight()/getRowCount());
        } else {
            for (int i=0;i<getRowCount();i++) {
                int tallest=0;
                for (int j=0;j<getColumnCount();j++) {
                    if (((Component) model.getValueAt(i, j)).getPreferredSize().height > tallest) {
                        tallest = ((Component) model.getValueAt(i, j)).getPreferredSize().height;
                    }
                }
                setRowHeight(i,tallest);
            }
        }
     */   this.updateUI();

    }

    @Override
    public int getRowHeight() {
        if (autofit) {
            return getParent().getHeight()/this.getRowCount();
        } else {
            return ((Component) model.getValueAt(0, 0)).getPreferredSize().height;
        }
    }
  
    @Override
    public int getHeight() {
        if (parent != null) {
            return parent.getHeight();
        } else {
            return this.getPreferredSize().height;
        }
    }
}
class TableTransferHandler extends TransferHandler {

    int transfer_index=-1;
    PanelTable table;
    public TableTransferHandler(PanelTable table) {
        this.table = table;
        transfer_index=-1;
    }
    @Override
    public boolean canImport(TransferHandler.TransferSupport info) {
        if (!info.isDrop()) {
            return false;
        }
        if (!info.isDataFlavorSupported( TransfResizePanel.PANEL)) {
            return false;
        }
        return true;
    }

    @Override
    public boolean importData(TransferHandler.TransferSupport info) {
        if (!canImport(info)) {
            return false;
        }
        Object data;
        try {
            data = info.getTransferable().getTransferData(TransfResizePanel.PANEL);
        } catch (UnsupportedFlavorException e) {
            transfer_index=-1;
            return false;
        } catch (IOException e) {
            transfer_index=-1;
            return false;
        }

        int newrow = table.rowAtPoint(info.getDropLocation().getDropPoint());
        int newcol = table.columnAtPoint(info.getDropLocation().getDropPoint());
        if (transfer_index!=-1) {
            table.model.removeObjectAt(transfer_index);
            transfer_index = -1;
        }
        table.addObject(data, newrow, newcol);
        table.updateUI();
        return true;
    }
 
    @Override
    protected Transferable createTransferable(JComponent c) {
       Object obj;
       if (transfer_index != -1) {
           return null;
       }
       int row = table.getSelectedRow();
       int col = table.getSelectedColumn();
       obj = table.model.getValueAt(row, col);
       transfer_index = table.model.indexOf(row, col);
       if (obj instanceof Transferable) {
           return (Transferable) obj;
       }
       return null;
    }

    @Override
    public int getSourceActions(JComponent c) {
        return TransferHandler.COPY_OR_MOVE;
    }


}
class panelCellRenderer implements TableCellRenderer
{
    JPanel panel;
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        panel = new JPanel();
        panel.add((Component) value);
        if (isSelected) {
            panel.setBorder(new LineBorder(Color.red));
        } else {
            panel.setBorder(new LineBorder(Color.white));
        }
        return panel;
    }
}
