/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package phalgui.Utils;

import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

public class PollingThread extends Thread {

    // ******** private variables **********************************************
    private boolean stop = false;  // boolean for stop thread.
    private boolean end = false; // boolean for end the thread.
    private int pollingTime;      // the pollingTime for each poll.

    // ******** constructor ****************************************************
    public PollingThread(int pollingTime) {
        setPollingTime(pollingTime);  // set pollingTime for the thread
    }

    // ******** public methods *************************************************
    /** The run-method handles the polls and waits a defined
     * pollingTime, which can be set by the setpollingTime method. If the
     * controlProtocol state isn't RUN, the thread is stopped at the starting of
     * this method.
     * @see MessengerClient#getValues
     */
    @Override
    public void run() {
        int i = 0;

        while (!end) {
            if (stop) {
                synchronized (this) {
                    while (stop) {
                        try {
                            wait();
                        } catch (InterruptedException ex) {
                            Logger.getLogger(PollingThread.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                }
            }

            if (!end) {
                try {
                    callListeners();
                    sleep(pollingTime);
                } catch (InterruptedException ex) {
                    Logger.getLogger(PollingThread.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    public synchronized void stopPolling() {
        stop = true;
        notify();
    }

    public synchronized void startPolling() {
        stop = false;
        notify();
    }

    /** Finishes the thread. End variable is set true and the run method is finished. */
    public synchronized void finish() {
        end = true;
        stop = false;
        notify();
    }

    /** Sets the polling time.
     * @param miliseconds Integer with the interval of miliseconds for each poll.
     */
    public void setPollingTime(int miliseconds) {
        pollingTime = miliseconds;
    }

    private void callListeners() {
        for (int i=0;i<listeners.size();i++) {
            ((PollingThreadListener) listeners.get(i)).call();
        }
    }

    public interface PollingThreadListener {
        public void call();
    }

    Vector listeners = new Vector<PollingThreadListener>();

    public void addPollingThreadListener(PollingThreadListener listener) {
        listeners.add(listener);
    }
}
