/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phalgui.Utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.SwingWorker;

/**
 *
 * @author ismael
 */
public class iFileReader {
    private File file;
    private RandomAccessFile raf;
    private ReaderThread rthread;
    long cur_pos;
    final static int OPEN_TOUT = 10, OPEN_SLEEP_MS=200;
    
    public iFileReader(File file, int refresh_interval) throws FileNotFoundException {
        this(file);
        setRefreshInterval(refresh_interval);
    }
    public iFileReader(File file) throws FileNotFoundException{
        this.file=file;
        int cnt=0;
        while (raf == null) {
            try {
                raf = new RandomAccessFile(file,"r");
            } catch (FileNotFoundException ex) {
                cnt++;
                if (cnt==OPEN_TOUT) {
                    throw ex;
                }
                try {
                    Thread.sleep(OPEN_SLEEP_MS);
                } catch (InterruptedException ex1) {
                    throw ex;
                }
            }
        }
        rthread = new ReaderThread();
        new Thread(rthread,"ReaderThread:" + file.getName()).start();
    }

    public File getFile() {
        return file;
    }

    public void seekToEnd() throws IOException {
        cur_pos = getLength();
    }

    public void stopReader() {
        rthread.running=false;
    }
    public void setRefreshInterval(int milisec) {
        rthread.sleep_ms=milisec;
    }
    public String readNext(long length) throws IOException {
        String s = read(cur_pos,cur_pos+length);
        cur_pos += length;
        return s;
    }
    public String readNext() throws IOException {
        if (getLength()<cur_pos) {
            cur_pos=getLength();
        }
        return readNext(getLength()-cur_pos);
    }
    public String read() throws IOException {
        return read(0,getLength());
    }
    public String read(long start) throws IOException {
        if (start<getLength())
            return read(start,getLength());
        else
            return null;
    }

    public String readlast(long length) throws IOException {
        if (length>getLength())
            length=getLength();
        return read(getLength()-length,getLength());
    }

    public String readfirst(long length) throws IOException {
        if (length>getLength())
            length=getLength();
        return read(0,length);
    }
    int current_allocated=1024*1024;
    byte[] buff = new byte[current_allocated];
    public String read(long start, long end) throws IOException {
        String s;
        if (start==end)
            return null;
        int len = (int) end - (int) start;
        if (len<0) {
            System.out.println("Trying to read negative value!! " + len);
            return null;
        }
        if (len>current_allocated) {
            System.out.println("Trying to read too many bytes: " + len);
            return new String("\n");
        }
        raf.seek(start);
        if (raf.read(buff, 0, len)==-1)
            s=null;
        else
            s=new String(buff,0,len);
        return s;
    }
    public long getLength() throws IOException {
        return raf.length();
    }


    private TextUpdateListener textUpdateListener;
    private LastLinesListener LastLinesListener;
    
    public void callTextUpdateListener() {
        if (textUpdateListener==null)
            return;
        SwingWorker w = new SwingWorker<Void,Void>() {

            @Override
            protected Void doInBackground() throws Exception {
                textUpdateListener.TextUpdated();
                return null;
            }
        };
        w.execute();
    }
    public void setTextUpdateListeners(TextUpdateListener listener) {
        this.textUpdateListener = listener;
    }
    public interface TextUpdateListener {
        void TextUpdated();
    }
    public void callLastLineListener(final String lines[]) {
        if (LastLinesListener==null)
            return;
        SwingWorker w = new SwingWorker<Void,Void>() {

            @Override
            protected Void doInBackground() throws Exception {
                LastLinesListener.LastLines(lines);
                return null;
            }
        };
        w.execute();
    }
    public interface LastLinesListener {
        void LastLines(String lines[]);
    }
    public void setLastLinesListener(LastLinesListener listener) {
        this.LastLinesListener = listener;
    }

    private class ReaderThread extends Thread {
        boolean running=true;
        int sleep_ms=500;
        @Override
        public void run() {
            long len=0;
            try {
                len = raf.length();
            } catch (IOException ex) {
                Logger.getLogger(iFileReader.class.getName()).log(Level.SEVERE, null, ex);
                running=false;
            }
            while(running) {
                try {
                    if (raf.length() != len) {
                        if (textUpdateListener!=null) {
                            callTextUpdateListener();
                        } else if (LastLinesListener!=null) {
                            if (raf.length() > len) {
                                String s;
                                if ((s = readNext())!= null) {
                                    String[] lines = s.split("\\n");
                                    callLastLineListener(lines);
                                    //System.out.println("Called with " + lines.length + " lines");
                                    s=null;
                                    lines=null;
                                }
                            }
                        }
                    }
                    len = raf.length();
                    sleep(sleep_ms);
                } catch (IOException ex) {
                    Logger.getLogger(iFileReader.class.getName()).log(Level.SEVERE, null, ex);
                    running=false;
                } catch (InterruptedException ex) {
                    Logger.getLogger(iFileReader.class.getName()).log(Level.SEVERE, null, ex);
                    running=false;
                }
            }
        }
        public ReaderThread() {}
    }
}
